@@ -100,30 +100,47 @@ from __future__ import annotations
 
 import ast
 import builtins
-import itertools
 import inspect
+import itertools
 import re
-import sys
 
 from abc import ABC, abstractmethod
-from pathlib import Path
 from argparse import ArgumentTypeError
 from contextlib import suppress
-from functools import update_wrapper, reduce, lru_cache
-from typing import TYPE_CHECKING, get_type_hints
-from typing import AnyStr, Deque, Optional, Tuple, Union, Mapping, Any, List, TypeVar, Iterable, ByteString, Callable
+from functools import lru_cache, reduce, update_wrapper
+from pathlib import Path
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    AnyStr,
+    Callable,
+    Deque,
+    Iterable,
+    Mapping,
+    TypeVar,
+    Union,
+    overload,
+)
 
+from refinery.lib.annotations import get_type_hints
 from refinery.lib.frame import Chunk
-from refinery.lib.tools import isbuffer, infinitize, one, normalize_to_identifier, exception_to_string
-from refinery.lib.types import NoMask, RepeatedInteger
-from refinery.lib.meta import is_valid_variable_name, metavars, Percentage
+from refinery.lib.meta import Percentage, is_valid_variable_name, metavars
 from refinery.lib.patterns import formats
+from refinery.lib.tools import (
+    bounds,
+    exception_to_string,
+    infinitize,
+    isbuffer,
+    normalize_to_identifier,
+    one,
+)
+from refinery.lib.types import NoMask, RepeatedInteger, buf
 
 if TYPE_CHECKING:
     from refinery import Unit
 
 FinalType = TypeVar('FinalType')
-DelayedType = Callable[[ByteString], FinalType]
+DelayedType = Callable[[buf], FinalType]
 MaybeDelayedType = Union[DelayedType[FinalType], FinalType]
 
 _DEFAULT_BITS = 64
@@ -134,7 +151,6 @@ class ParserError(ArgumentTypeError):
     """
     An exception raised by the `refinery.lib.argformats.PythonExpression` parser.
     """
-    pass
 
 
 class ParserVariableMissing(ParserError):
@@ -142,24 +158,21 @@ class ParserVariableMissing(ParserError):
     Raised when the `refinery.lib.argformats.PythonExpression` parser fails to evaluate an
     expression because of a missing variable.
     """
-    pass
 
 
 class LazyEvaluation:
     """
-    Empty parent class for any unit that throws `refinery.lib.argformats.TooLazy`.
+    Empty parent class for any parse result that throws `refinery.lib.argformats.TooLazy`.
     """
-    pass
 
 
 class PythonExpression:
     """
-    Implements a parser for any Python expression with a prescribed set of variable
-    names permitted to occur in the expression. The resulting object is a callable
-    which can be given the string representation of such an expression. In turn, the
-    result of this operation is either the value of the expression if no variables
-    were present, or a callable which expects keyword arguments corresponding to the
-    permitted variable names.
+    Implements a parser for any Python expression with a prescribed set of variable names permitted
+    to occur in the expression. The resulting object is a callable which can be given the string
+    representation of such an expression. In turn, the result of this operation is either the value
+    of the expression if no variables were present, or a callable which expects keyword arguments
+    corresponding to the permitted variable names.
     """
     def __init__(self, definition: AnyStr, *variables, constants=None, all_variables_allowed=False, mask=None):
         self.definition = definition = definition.strip()
@@ -173,14 +186,10 @@ class PythonExpression:
             raise ParserError(F'The provided expression could not be parsed: {definition!s}; {exception_to_string(error)}')
 
         class Postprocessor(ast.NodeTransformer):
-            if sys.version_info >= (3, 8):
-                def visit_Constant(self, node: ast.Constant):
-                    if not isinstance(node.value, str):
-                        return node
-                    return ast.Constant(value=node.value.encode('utf8'))
-            else:
-                def visit_Str(self, node: ast.Str):
-                    return ast.Bytes(s=node.s.encode('utf8'))
+            def visit_Constant(self, node: ast.Constant):
+                if not isinstance(node.value, str):
+                    return node
+                return ast.Constant(value=node.value.encode('utf8'))
 
             def visit_MatMult(self, node: ast.MatMult) -> Any:
                 return ast.BitXor()
@@ -258,10 +267,10 @@ class PythonExpression:
 
 class SliceAgain(LazyEvaluation):
     """
-    Raised by `refinery.lib.argformats.sliceobj` to indicate that meta variables
-    are required to compute this slice.
+    Raised by `refinery.lib.argformats.sliceobj` to indicate that meta variables are required to
+    compute this slice.
     """
-    def __init__(self, expr: Union[DelayedBinaryArgument, str]):
+    def __init__(self, expr: DelayedBinaryArgument | str):
         self.expr = expr
 
     def __call__(self, data):
@@ -280,29 +289,58 @@ def percent(expression: str):
     return float(expression)
 
 
-def relslice(expression: Union[int, str, slice], data: Optional[Chunk] = None) -> Union[slice, SliceAgain]:
+@overload
+def relslice(expression: int | str | slice, data: Chunk) -> slice:
+    ...
+
+
+@overload
+def relslice(expression: int | str | slice) -> slice | SliceAgain:
+    ...
+
+
+def relslice(expression: int | str | slice, data: Chunk | None = None) -> slice | SliceAgain:
     """
     Uses `refinery.lib.argformats.sliceobj` to parse a slice from the input, but
     interprets the second part of the slice as a relative length (which can also
     be negative).
     """
-    bounds = sliceobj(expression, data)
-    if (stop := bounds.stop) is not None:
-        start = bounds.start or 0
+    s = sliceobj(expression, data)
+    if isinstance(s, slice) and (stop := s.stop) is not None:
+        start = s.start or 0
         stop += start
-        if (step := bounds.step) is None and stop < start:
+        if (step := s.step) is None and stop < start:
             step = -1
-        bounds = slice(start, stop, step)
-    return bounds
+        s = slice(start, stop, step)
+    return s
+
+
+@overload
+def sliceobj(
+    expression: int | str | slice,
+    data: Chunk,
+    intok: bool = False,
+    final: bool = False,
+) -> slice | int:
+    ...
+
+
+@overload
+def sliceobj(
+    expression: int | str | slice,
+    data: Chunk | None = None,
+    intok: bool = False,
+    final: bool = False,
+) -> slice | int | SliceAgain:
+    ...
 
 
 def sliceobj(
-    expression: Union[int, str, slice],
-    data: Optional[Chunk] = None,
-    range=False,
-    intok=False,
-    final=False,
-) -> Union[slice, SliceAgain]:
+    expression: int | str | slice,
+    data: Chunk | None = None,
+    intok: bool = False,
+    final: bool = False,
+) -> slice | int | SliceAgain:
     """
     Uses `refinery.lib.argformats.PythonExpression` to parse slice expressions
     where the bounds can be given as arithmetic expressions. For example, this
@@ -312,6 +350,8 @@ def sliceobj(
     if isinstance(expression, (slice, SliceAgain)):
         return expression
     if isinstance(expression, int):
+        if intok:
+            return expression
         return slice(expression, expression + 1, 1)
     if isinstance(expression, (bytes, bytearray)):
         expression = expression.decode('utf8')
@@ -322,7 +362,7 @@ def sliceobj(
         variables = metavars(data)
         if is_valid_variable_name(expression):
             try:
-                return sliceobj(variables[expression], data, intok=intok, range=range, final=True)
+                return sliceobj(variables[expression], data, intok=intok, final=True)
             except Exception:
                 pass
 
@@ -337,25 +377,16 @@ def sliceobj(
             raise
         elif data is not None:
             parser = DelayedNumSeqArgument(expression)
-            return sliceobj(parser(data), data, range=range, intok=intok, final=True)
+            return sliceobj(parser(data), data, intok=intok, final=True)
         else:
             return SliceAgain(expression)
     if len(sliced) == 1:
         k = sliced[0]
         if intok:
             return k
-        if range:
-            return slice(0, k, 1)
         return slice(k, k + 1) if k + 1 else slice(k, None, None)
-    if range:
-        range_defaults = (0, None, 1)
-        k = len(range_defaults) - len(sliced)
-        if k > 0:
-            sliced.extend(range_defaults[-k:])
     for k, item in enumerate(sliced):
         if item is None:
-            if range:
-                sliced[k] = range_defaults[k]
             continue
         if isinstance(item, int):
             continue
@@ -395,7 +426,6 @@ class TooLazy(Exception):
     Exception which indicates that an argument parser requires input data before it can be
     evaluated.
     """
-    pass
 
 
 class VariableMissing(ArgumentTypeError):
@@ -447,8 +477,8 @@ class DelayedArgumentDispatch:
         self.final = {}
         self.units = {}
 
-    def _get_unit(self, name: str, *args) -> Optional[Unit]:
-        name, rev, empty = normalize_to_identifier(name).partition(_REVERSE_SIGN)
+    def _get_unit(self, name: str, *args) -> Unit | None:
+        empty, rev, name = normalize_to_identifier(name).rpartition(_REVERSE_SIGN)
         if empty:
             return None
         uhash = hash((name,) + args)
@@ -508,7 +538,7 @@ class DelayedArgumentDispatch:
         return _register
 
 
-def LazyPythonExpression(expression: str, variables: Optional[dict] = None) -> MaybeDelayedType[Any]:
+def LazyPythonExpression(expression: str, variables: dict | None = None) -> MaybeDelayedType[Any]:
     """
     Wraps the given expression for use as a `refinery.lib.argformats.multibin` expression. If it
     contains no variables, the expression is evaluated immediately, otherwise the function returns
@@ -590,7 +620,7 @@ class DelayedArgument(LazyEvaluation):
             self.seed = expression
         self.modifiers.reverse()
 
-    def _split_expression(self, expression: str, reverse: bool = False) -> Tuple[str, Optional[str]]:
+    def _split_expression(self, expression: str, reverse: bool = False) -> tuple[str, str | None]:
         argument_list_visited = False
         brackets = 0
         inc = self._ARG_BEGIN_TOKEN
@@ -621,7 +651,7 @@ class DelayedArgument(LazyEvaluation):
                 return (head, tail) if reverse else (tail, head)
         return expression, None
 
-    def _split_modifier(self, expression: str, reverse: bool = False) -> Tuple[Optional[str], Tuple[str], str]:
+    def _split_modifier(self, expression: str, reverse: bool = False) -> tuple[str | None, tuple[str], str]:
         brackets = 0
         name = None
         argoffset = 0
@@ -659,7 +689,7 @@ class DelayedArgument(LazyEvaluation):
                 raise ArgumentTypeError(F'Unexpected error parsing "{expression}".')
         return name, arguments, rest
 
-    def __call__(self, data: Optional[ByteString] = None) -> bytes:
+    def __call__(self, data: buf | None = None) -> bytes:
         arg = self.seed
         mod = iter(self.modifiers)
         if not self.finalized:
@@ -731,10 +761,10 @@ class DelayedArgument(LazyEvaluation):
         """
         return string.encode('LATIN-1')
 
-    @handler.register('h!', 'H!')
-    def hexencode(self, string: bytes) -> bytes:
-        """
-        The modifier `h!` (or `H!`) encodes the input as hexadecimal.
+    @handler.register(F'{_REVERSE_SIGN}h', F'{_REVERSE_SIGN}H')
+    def bang_h(self, string: bytes) -> bytes:
+        F"""
+        The modifier `{_REVERSE_SIGN}h` (or `{_REVERSE_SIGN}H`) encodes the input as hexadecimal.
         """
         import base64
         return base64.b16encode(string)
@@ -756,6 +786,15 @@ class DelayedArgument(LazyEvaluation):
         from refinery.units.encoding.esc import esc
         return esc().process(string)
 
+    @handler.register(F'{_REVERSE_SIGN}n')
+    def bang_n(self, string: bytes) -> bytes:
+        F"""
+        The modifier `{_REVERSE_SIGN}n:string` returns an escaped and quoted version of the
+        input string.
+        """
+        from refinery.units.encoding.esc import esc
+        return esc(quoted=True).reverse(string)
+
     @handler.register('q', 'Q', final=True)
     def q(self, string: str) -> bytes:
         """
@@ -769,7 +808,7 @@ class DelayedArgument(LazyEvaluation):
         """
         Returns the contents of the file located at the given path. This path may contain
         wildcard characters, but this pattern has to match a single file. It is also possible
-        to use the handler as `read[offset]` or as `read[offset:count]` to read `count` many
+        to use the handler as `read[count]` or as `read[offset:count]` to read `count` many
         bytes from the file at the given offset.
         """
         return self._file(path, region)
@@ -792,17 +831,27 @@ class DelayedArgument(LazyEvaluation):
         else:
             return self._file(path, region)
 
-    def _file(self, pattern: str, region: str) -> Optional[bytes]:
-        def read(data: Optional[Chunk] = None):
+    def _file(self, pattern: str, region: str) -> bytes | None:
+        def read(data: Chunk | None = None):
             if not region:
-                bounds = slice(0, None)
+                k = slice(0, None)
             else:
-                bounds = sliceobj(region, data, range=True)
-            if bounds.step and bounds.step != 1:
-                raise ValueError('Step size is not supported for file slices.')
+                k = sliceobj(region, data, intok=True)
+            if isinstance(k, SliceAgain):
+                raise TooLazy
+            if isinstance(k, int):
+                offset = 0
+                length = k
+            else:
+                if k.step and k.step != 1:
+                    raise ValueError('Step size is not supported for file slices.')
+                offset = k.start or 0
+                length = k.stop
+            if length and length < 0:
+                raise ValueError('The count for a read operation must be non-negative.')
             with open(path, 'rb') as stream:
-                stream.seek(bounds.start or 0)
-                return stream.read(bounds.stop)
+                stream.seek(offset)
+                return stream.read(length)
         try:
             path: Path = one(Path.cwd().glob(pattern))
         except (NotImplementedError, LookupError):
@@ -815,27 +864,25 @@ class DelayedArgument(LazyEvaluation):
             return read
 
     @handler.register('range', final=True)
-    def range(self, region: str) -> bytes:
+    def range(self, region: str) -> Iterable[int]:
         """
         Implements the final modifier `range:bounds` to generate a sequence of bytes, where
         `bounds` is parsed as a `refinery.lib.argformats.sliceobj` with one exception: If
         `bounds` is just a single integer, it is interpreted as the upper bound for a sequence
         of bytes starting at zero.
         """
-        def compute_range(data: Optional[Chunk] = None):
+        def compute_range(data: Chunk | None = None):
             try:
-                bounds = sliceobj(region, data, range=True)
+                s = sliceobj(region, data, intok=True)
             except ParserVariableMissing:
                 raise TooLazy
-            if pending(bounds):
+            if isinstance(s, SliceAgain):
                 raise TooLazy
-            start = bounds.start or 0
-            stop = bounds.stop
-            step = bounds.step or 1
-            if stop is None:
-                return itertools.islice(itertools.count(), start, None, step)
-            result = range(start, stop, step)
-            if 0 <= start and stop <= 0x100:
+            if isinstance(s, int):
+                s = slice(None, s, None)
+            result = bounds[s]
+            result.max -= 1
+            if 0 <= result.min and result.max <= 0x100:
                 result = bytearray(result)
             return result
         try:
@@ -853,7 +900,7 @@ class DelayedArgument(LazyEvaluation):
         return os.environ[name]
 
     @handler.register('pos')
-    def pos(self, regex: ByteString, occurrence: int = 0) -> int:
+    def pos(self, regex: buf, occurrence: int = 0) -> int:
         """
         The handler pos[k=0]:[regex] returns the position of the k-th occurrence of the regular
         expression [regex]. The value `k` can be set to `-1` to return the position of the last
@@ -889,12 +936,12 @@ class DelayedArgument(LazyEvaluation):
         return _pos
 
     @handler.register('rx')
-    def rx(self, str: bytes) -> bytes:
+    def rx(self, string: bytes) -> bytes:
         """
         The handler `rx:str` returns a regular expression which matches the exact string
         sequence given by `str`, with special regular expression control characters escaped.
         """
-        return re.escape(str)
+        return re.escape(string)
 
     @handler.register('c', 'copy', final=True)
     def copy(self, region: str) -> bytes:
@@ -914,46 +961,46 @@ class DelayedArgument(LazyEvaluation):
         The handler `cut:region` work like `refinery.lib.argformats.DelayedBinaryArgument.copy`,
         but the bytes are removed from the input data after copying them.
         """
-        def _cut(data: Union[bytearray, Chunk]):
+        def _cut(data: bytearray | Chunk):
             bounds = relslice(region, data)
             result = bytearray(data[bounds])
             del data[bounds]
             return result
         return _cut
 
-    @handler.register('pp')
-    def pp(self, input: bytes, region: Union[slice, str] = slice(1, None, 1), separator: bytes = B'/') -> bytes:
+    @handler.register('pd')
+    def pd(self, input: bytes, region: slice | str = slice(1, None, 1), separator: bytes = B'/') -> bytes:
         """
-        The handler `pp[region=1:,sep=/]:input` splits the input at the given string `sep`
+        The handler `pd[region=1:,sep=/]:input` splits the input at the given string `sep`
         and reverses the sequence. Parts are then selected according to the `region` parameter,
         which can be any expression in Python slice syntax. Afterwards, the resulting sequence
         is reversed again and joined at the `sep` parameter. For example, the expression
 
-            pp[:2]:/path/to/some/item/and/more
+            pd[2:]:/path/to/some/item/and/more
 
         will return the string `/path/to/some/item`.
         """
-        def _pp(data: Optional[Chunk] = None):
+        def _pd(data: Chunk | None = None):
             b: slice = sliceobj(region, data=data, final=True)
-            pp = input.split(separator)
-            pp.reverse()
-            pp = pp[b]
-            pp.reverse()
-            return separator.join(pp)
+            pd = input.split(separator)
+            pd.reverse()
+            pd = pd[b]
+            pd.reverse()
+            return separator.join(pd)
         try:
-            return _pp()
+            return _pd()
         except TooLazy:
-            return _pp
+            return _pd
 
     @handler.register('pb')
     def pb(self, input: bytes, separator: bytes = B'/') -> bytes:
         """
-        The handler `pb:input` is equivalent to `pp[0:1]:input` and corresponds to "basename".
+        The handler `pb:input` is equivalent to `pd[:1]:input` and corresponds to "basename".
         """
-        return self.pp(input, slice(0, 1, 1), separator=separator)
+        return self.pd(input, slice(0, 1, 1), separator=separator)
 
     @handler.register('pn')
-    def pn(self, path: Union[str, bytes]) -> bytes:
+    def pn(self, path: str | bytes) -> bytes:
         """
         The handler `pn:/path/to/file.ext` returns `/path/to/file`, i.e. the path without its
         extension. The handler name is short for "path name". If the last part does not have
@@ -964,10 +1011,10 @@ class DelayedArgument(LazyEvaluation):
         return Path(path).with_suffix('').as_posix().encode()
 
     @handler.register('px')
-    def px(self, path: Union[str, bytes]) -> bytes:
+    def px(self, path: str | bytes) -> bytes:
         """
-        The handler `px:/path/to/file.ext` returns `ext`, i.e. the extension
-        extension. The handler name is short for "path name".
+        The handler `px:/path/to/file.ext` returns `ext`, i.e. the extension extension. The
+        handler name is short for "path eXtension".
         """
         if not isinstance(path, str):
             path = path.decode('utf8')
@@ -1044,7 +1091,7 @@ class DelayedArgument(LazyEvaluation):
         return LazyPythonExpression(expression)
 
     @handler.register('btoi')
-    def btoi(self, binary: ByteString, size=None, step=None) -> Iterable[int]:
+    def btoi(self, binary: buf, size=None, step=None) -> Iterable[int]:
         """
         The modifier `btoi[size=0,step=0]:data` uses `refinery.lib.chunks.unpack` to convert a
         sequence of bytes into a sequence of integers.
@@ -1077,7 +1124,7 @@ class DelayedArgument(LazyEvaluation):
         return list(chunks.unpack(binary, size, bigE, step))
 
     @handler.register('itob')
-    def itob(self, integers: Iterable[int], size=None) -> ByteString:
+    def itob(self, integers: Iterable[int], size=None) -> buf:
         """
         The modifier `itob[size=0]:integers` is the inverse of `btoi` and works in the same way,
         except that the case `size=0` is handled in the following way: The handler inspects all
@@ -1142,7 +1189,7 @@ class DelayedArgument(LazyEvaluation):
         return delay
 
     @handler.register('take')
-    def take(self, it: Iterable[int], bounds: Optional[str] = None):
+    def take(self, it: Iterable[int], bounds: str | None = None):
         """
         The handler `take[start:stop:step]` expects an integer sequence as input and applies a slice
         to it. Slices are given in Python syntax, so `take[::2]` will extract every second item from
@@ -1197,7 +1244,7 @@ class DelayedArgument(LazyEvaluation):
     def prng(
         self,
         size: str,
-        seed: Optional[str] = None
+        seed: str | None = None
     ) -> bytes:
         """
         The `prng[seed]:count` handler generates `count` random bytes using Python's built-in random
@@ -1232,9 +1279,9 @@ class DelayedArgument(LazyEvaluation):
     def accu(
         self,
         spec: str,
-        seed: Optional[str] = None,
-        skip: Optional[str] = None,
-        precision: Optional[str] = None
+        seed: str | None = None,
+        skip: str | None = None,
+        precision: str | None = None
     ) -> Iterable[int]:
         """
         The final handler
@@ -1282,7 +1329,7 @@ class DelayedArgument(LazyEvaluation):
         precision = precision and int(precision, 0) or _DEFAULT_BITS
         mask = precision and (1 << precision) - 1
 
-        def finalize(data: Optional[Chunk] = None):
+        def finalize(data: Chunk | None = None):
             @lru_cache(maxsize=512, typed=False)
             def accumulate(A):
                 return update(meta, A=A)
@@ -1307,7 +1354,7 @@ class DelayedArgument(LazyEvaluation):
             return finalize()
 
     @handler.register('be')
-    def be(self, arg: Union[int, ByteString], size: Optional[str] = None) -> int:
+    def be(self, arg: int | buf, size: str | None = None) -> int:
         """
         The handler `be[size=0]:data` converts a binary input into the integer that it encodes in
         big endian format, and vice versa. The optional parameter `size` can be used to specify
@@ -1325,7 +1372,7 @@ class DelayedArgument(LazyEvaluation):
             return int.from_bytes(arg[:size], 'big')
 
     @handler.register('le')
-    def le(self, arg: Union[int, ByteString], size: Optional[str] = None) -> int:
+    def le(self, arg: int | buf, size: str | None = None) -> int:
         """
         The handler `be[size=0]:data` converts a binary input into the integer that it encodes in
         little endian format, and vice versa. The optional parameter `size` can be used to specify
@@ -1343,7 +1390,7 @@ class DelayedArgument(LazyEvaluation):
             return int.from_bytes(arg[:size], 'little')
 
     @handler.register('reduce')
-    def reduce(self, it: Iterable[int], reduction: str, seed: Optional[str] = None) -> int:
+    def reduce(self, it: Iterable[int], reduction: str, seed: str | None = None) -> int:
         """
         The handler `reduce[reduction, seed=0]` has two parameters. The string `reduction` is a
         Python expression that involves the two special variables `S` (the state) and `B`
@@ -1354,7 +1401,7 @@ class DelayedArgument(LazyEvaluation):
         seed = seed and PythonExpression.Lazy(seed)
         reduction = PythonExpression.Lazy(reduction)
 
-        def finalize(data: Optional[Chunk] = None):
+        def finalize(data: Chunk | None = None):
             def _reduction(S, B):
                 v = reduction(args, S=S, B=B)
                 return v
@@ -1373,7 +1420,7 @@ class DelayedBinaryArgument(DelayedArgument):
     are implemented in `refinery.lib.argformats.DelayedArgument`.
     """
 
-    def __call__(self, data: Optional[ByteString] = None) -> bytes:
+    def __call__(self, data: buf | None = None) -> bytes:
         value = super().__call__(data=data)
         if not isbuffer(value):
             if isinstance(value, str):
@@ -1395,10 +1442,10 @@ class DelayedPathArgument(DelayedBinaryArgument):
     with the contents of an equally named file on disk.
     """
     def default_handler(self, expression: str) -> bytes:
-        try:
-            return utf8(expression)
-        except Exception:
-            return expression
+        return utf8(expression)
+
+    def __call__(self, data: buf | None = None) -> str:
+        return super().__call__(data).decode('utf8')
 
 
 class DelayedNumSeqArgument(DelayedArgument):
@@ -1430,7 +1477,7 @@ class DelayedNumSeqArgument(DelayedArgument):
                 return super().default_handler(expression)
             return expression
 
-    def __call__(self, data: Optional[Union[ByteString, Chunk]] = None) -> Iterable[int]:
+    def __call__(self, data: buf | Chunk | None = None) -> Iterable[int]:
         value = super().__call__(data)
         if isbuffer(value):
             return value
@@ -1503,7 +1550,7 @@ class DelayedRegexpArgument(DelayedArgument):
                 tick += 1
                 pattern = str(formats.get(match[1], indicators.get(match[1], match[0])))
                 pattern = re.sub(r'(?<=\(\?P[<=])__(\w+)__', F'__\\1_{tick}__', pattern)
-                return '(?:{})'.format(pattern)
+                return f'(?:{pattern})'
 
             def replace_variable_assignment(match):
                 return F'(?P<{match[1]}>'
@@ -1529,12 +1576,12 @@ class DelayedRegexpArgument(DelayedArgument):
         """
         from refinery.lib.patterns import formats
         try:
-            return formats[name]
+            return formats[name].value.bin_pattern
         except LookupError:
             raise ArgumentTypeError(
                 F'Based on the prefix "f:", the parser looked for a carve format named "{name}".'
-                U' No such format is known; prefix the entire expression with "s:" if this was '
-                U'unintended, otherwise correct the format name spelling.')
+                ' No such format is known; prefix the entire expression with "s:" if this was '
+                'unintended, otherwise correct the format name spelling.')
 
     @handler.register('yara', 'y', 'Y')
     def yara(self, pattern: bytes) -> bytes:
@@ -1592,20 +1639,20 @@ class DelayedNumberArgument(DelayedArgument):
     The class can be initialized with numerical bounds and checks the validity of the input after
     having evaluated all handlers.
     """
-    def __init__(self, expression: str, min: int, max: int):
+    def __init__(self, expression: str, min: int | None, max: int | None):
         self.min = min
         self.max = max
         super().__init__(expression)
 
-    def __call__(self, data: Union[ByteString, Chunk, None] = None) -> int:
+    def __call__(self, data: buf | Chunk | None = None) -> int:
         value = super().__call__(data)
         if not isinstance(value, int):
             tv = type(value).__name__
             raise ArgumentTypeError(F'The value computed from {self.expression} is of type {tv}, it should be an integer.')
         if self.min is not None and value < self.min or self.max is not None and value > self.max:
             a = '-∞' if self.min is None else self.min
-            b = u'∞' if self.max is None else self.max
-            raise ArgumentTypeError(F'value {value} is out of bounds [{a}, {b}]')
+            b = '∞' if self.max is None else self.max
+            raise ArgumentTypeError(F'value {value} is out of bounds [{a},{b}]')
         return value
 
     def default_handler(self, expression: str) -> int:
@@ -1618,12 +1665,12 @@ class DelayedNumberArgument(DelayedArgument):
 class ___number:
     __name__ = 'number'
 
-    def __init__(self, min=None, max=None):
+    def __init__(self, min: int | None = None, max: int | None = None):
         self.min = min
         self.max = max
 
-    def __getitem__(self, bounds):
-        return self.__class__(bounds.start, bounds.stop)
+    def __getitem__(self, k: slice):
+        return self.__class__(k.start, k.stop)
 
     def __call__(self, value):
         if isinstance(value, int):
@@ -1651,7 +1698,7 @@ expressions.
 """
 
 
-def numseq(expression: Union[int, str], reverse=False, seed=None, typecheck=True) -> Union[Iterable[int], DelayedNumSeqArgument]:
+def numseq(expression: int | str, reverse=False, seed=None, typecheck=True) -> Iterable[int] | DelayedNumSeqArgument:
     """
     This is the argument parser type that uses `refinery.lib.argformats.DelayedNumSeqArgument`.
     """
@@ -1663,19 +1710,17 @@ def numseq(expression: Union[int, str], reverse=False, seed=None, typecheck=True
     return arg
 
 
-def pathvar(expression: Union[str, bytes, bytearray]) -> Union[bytes, DelayedArgument]:
+def pathvar(expression: str) -> str | DelayedArgument:
     """
     This is the argument parser type that uses `refinery.lib.argformats.DelayedPathArgument`.
     """
-    if not isinstance(expression, str):
-        return bytes(expression)
     arg = DelayedPathArgument(expression)
     with suppress(TooLazy):
         return arg()
     return arg
 
 
-def multibin(expression: Union[str, bytes, bytearray], reverse=False, seed=None) -> Union[bytes, DelayedArgument]:
+def multibin(expression: str | bytes | bytearray, reverse=False, seed=None) -> bytes | DelayedArgument:
     """
     This is the argument parser type that uses `refinery.lib.argformats.DelayedBinaryArgument`.
     """
@@ -1687,7 +1732,7 @@ def multibin(expression: Union[str, bytes, bytearray], reverse=False, seed=None)
     return arg
 
 
-def regexp(expression: str) -> Union[int, bytes, DelayedRegexpArgument]:
+def regexp(expression: str) -> int | bytes | DelayedRegexpArgument:
     """
     This is the argument parser type that uses `refinery.lib.argformats.DelayedRegexpArgument`.
     """
@@ -1750,7 +1795,7 @@ def OptionFactory(options: Mapping[str, Any], ignorecase: bool = False):
                         name = key
                         break
             if name not in options:
-                raise ValueError('The option %s is not one of these: %s' % (name, list(options)))
+                raise ValueError('The option {} is not one of these: {}'.format(name, list(options)))
             self.mode = options[name]
             self.name = name
 
@@ -1774,7 +1819,7 @@ def extract_options(symbols, prefix: str, *exceptions: str):
     return {k: v for k, v in candidates.items() if isinstance(v, int)}
 
 
-def pending(argument: Union[Any, Iterable[Any]]) -> bool:
+def pending(argument: Any | Iterable[Any]) -> bool:
     """
     This function returns a boolean value which indicates whether the given
     argument is a `refinery.lib.argformats.LazyEvaluation`.
@@ -1784,7 +1829,7 @@ def pending(argument: Union[Any, Iterable[Any]]) -> bool:
     return isinstance(argument, LazyEvaluation)
 
 
-def manifest(argument: Union[Any, List[Any]], data: bytearray) -> Union[Any, List[Any]]:
+def manifest(argument: Any | list[Any], data: bytearray) -> Any | list[Any]:
     """
     Returns the manifestation of a `refinery.lib.argformats.LazyEvaluation`
     on the given data. This function can change the data.
