@@ -1,18 +1,16 @@
 from __future__ import annotations
 
-from typing import ClassVar
-from uuid import UUID
-from dataclasses import dataclass, field
-from functools import cached_property
-
 import codecs
 import math
 
+from dataclasses import dataclass, field
+from functools import cached_property
+from typing import ClassVar
+from uuid import UUID
 
-from refinery.lib.structures import Struct, StructReader
+from refinery.lib.lcid import DEFAULT_CODEPAGE, LCID
 from refinery.lib.lzx import LzxDecoder
-from refinery.lib.lcid import LCID, DEFAULT_CODEPAGE
-
+from refinery.lib.structures import Struct, StructReader
 
 _LZX_HLP = UUID('0a9007c6-4076-11d3-8789-0000f8105754')
 _LZX_CHM = UUID('7fc28940-9d31-11d0-9b27-00a0c91e9c7c')
@@ -321,12 +319,12 @@ class CHM(Struct):
             for k in range(dh.total_chunks):
                 body = reader.read_exactly(m)
                 if body[:4] == IndexChunk.Magic:
-                    self.index.append(IndexChunk(body, d))
+                    self.index.append(IndexChunk.Parse(body, d))
                     continue
                 if body[:4] == ListingChunk.Magic:
                     if not (dh.listing1st <= k <= dh.listingLst):
                         raise ValueError(F'Chunk {k} has magic {ListingChunk.Magic.decode()} but is out of listing range.')
-                    chunk = ListingChunk(body, d)
+                    chunk = ListingChunk.Parse(body, d)
                     for entry in chunk.entries:
                         name = entry.name
                         if name.startswith('/#') or name.startswith('/$'):
