@@ -3,11 +3,11 @@ Contains functions to aid in deobfuscation.
 """
 from __future__ import annotations
 
-from typing import Optional, Any
-
 import ast
 import re
 
+from typing import Any
+
 
 class ExpressionParsingFailure(ValueError):
     pass
@@ -39,7 +39,12 @@ _ALLOWED_NODE_TYPES = frozenset({
 })
 
 
-def cautious_eval(definition: str, size_limit: Optional[int] = None, walker: Optional[ast.NodeTransformer] = None) -> Any:
+def cautious_eval(
+    definition: str,
+    size_limit: int | None = None,
+    walker: ast.NodeTransformer | None = None,
+    environment: dict[str, Any] | None = None,
+) -> Any:
     """
     Very, very, very, very, very carefully evaluate a Python expression.
     """
@@ -51,7 +56,12 @@ def cautious_eval(definition: str, size_limit: Optional[int] = None, walker: Opt
 
     if size_limit and len(definition) > size_limit:
         raise Abort(F'Size limit {size_limit} was exceeded while parsing')
-    if any(x not in '.^%|&~<>()-+/*0123456789xabcdefABCDEF' for x in definition):
+
+    test = definition
+    if environment:
+        for symbol in environment:
+            test = re.sub(RF'\b{symbol}\b', '', test)
+    if any(x not in '.^%|&~<>()-+/*0123456789xabcdefABCDEF' for x in test):
         raise Abort('Unknown characters in expression')
     try:
         expression = ast.parse(definition)
@@ -70,17 +80,23 @@ def cautious_eval(definition: str, size_limit: Optional[int] = None, walker: Opt
         raise Abort('Not a Python expression')
 
     nodes = list(nodes)
-    types = set(type(node) for node in nodes)
+    types = {type(node) for node in nodes}
 
     if not types <= _ALLOWED_NODE_TYPES:
         problematic = types - _ALLOWED_NODE_TYPES
         raise Abort('Expression contains operations that are not allowed: {}'.format(', '.join(str(p) for p in problematic)))
 
-    return eval(definition)
+    return eval(definition, environment)
 
 
-def cautious_eval_or_default(definition: str, default: Optional[Any] = None, size_limit: Optional[int] = None) -> Any:
+def cautious_eval_or_default(
+    definition: str,
+    default: Any | None = None,
+    size_limit: int | None = None,
+    walker: ast.NodeTransformer | None = None,
+    environment: dict[str, Any] | None = None,
+):
     try:
-        return cautious_eval(definition)
+        return cautious_eval(definition, size_limit, walker, environment)
     except ExpressionParsingFailure:
         return default
