@@ -1,8 +1,7 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
 """
 A library to parse .NET headers and metadata.
 """
+from __future__ import annotations
 
 
 def integer_from_ldc(ins: bytes):
