@@ -9,40 +9,31 @@ The provided interface is the same for all executables. It powers the following
 """
 from __future__ import annotations
 
-import sys
-import itertools
 import codecs
-
-from typing import NamedTuple, TYPE_CHECKING
+import itertools
 
 from abc import ABC, abstractmethod
 from enum import Enum
 from functools import lru_cache
+from typing import TYPE_CHECKING, NamedTuple
 
 from refinery.lib import lief
-from refinery.lib.types import INF, ByteStr
 from refinery.lib.shared import capstone as cs
+from refinery.lib.types import INF, buf
 
 if TYPE_CHECKING:
-    from lief.ELF import Binary as ELFBinary
-    from lief.MachO import Binary as MachOBinary
-    from lief.MachO import FatBinary as MachOFatBinary
-    from lief.PE import Binary as PEBinary
-
-    from capstone import Cs
-
     from typing import (
         ClassVar,
         Generator,
         Iterable,
-        List,
-        Optional,
-        Type,
-        TypeVar,
         Union,
     )
 
-    _T = TypeVar('_T')
+    from capstone import Cs
+    from lief.ELF import Binary as ELFBinary
+    from lief.MachO import Binary as MachOBinary
+    from lief.MachO import FatBinary as MachOFatBinary
+    from lief.PE import Binary as PEBinary
 
     AnyLIEF = Union[
         MachOBinary,
@@ -117,7 +108,7 @@ class Range(NamedTuple):
     def __repr__(self):
         return F'<{self.__class__.__name__}:{self!s}>'
 
-    def __sub__(self, them: Range) -> List[Range]:
+    def __sub__(self, them: Range) -> list[Range]:
         pieces = []
         if self.lower < them.lower:
             pieces.append(Range(self.lower, min(them.lower, self.upper)))
@@ -216,7 +207,7 @@ class Section(NamedTuple):
     virtual: Range
     synthetic: bool
 
-    def as_segment(self, populate_sections=False) -> Segment:
+    def as_segment(self: Section, populate_sections=False) -> Segment:
         sections = [self] if populate_sections else None
         return Segment(self.physical, self.virtual, sections, self.name)
 
@@ -244,13 +235,13 @@ class Relocation(NamedTuple):
 
 class Symbol(NamedTuple):
     address: int
-    name: Optional[str] = None
-    size: Optional[int] = None
-    function: Optional[bool] = None
-    exported: Optional[bool] = None
-    imported: Optional[bool] = None
+    name: str | None
+    size: int | None
+    function: bool
+    exported: bool
+    imported: bool
     is_entry: bool = False
-    section: Optional[Section] = None
+    section: Section | None = None
 
     def get_name(self):
         name = self.name
@@ -270,8 +261,8 @@ class Segment(NamedTuple):
     """
     physical: Range
     virtual: Range
-    sections: Optional[List[Section]]
-    name: Optional[str] = None
+    sections: list[Section] | None
+    name: str | None = None
 
     def as_section(self) -> Section:
         if self.name is None:
@@ -299,7 +290,7 @@ class CompartmentNotFound(LookupError):
     find a `refinery.lib.executable.Segment` that contains the given location.
     """
     def __init__(self, lt: LT, location: int):
-        super().__init__(F'Unable to find a segment that contains the {lt.value} 0x{location:X}.')
+        super().__init__(F'Unable to find a segment that contains the {lt.value} {location:#x}.')
         self.location_type = lt
         self.location = location
 
@@ -309,28 +300,29 @@ class Executable(ABC):
     An abstract representation of a parsed executable in memory.
     """
 
-    _data: ByteStr
-    _head: AnyLIEF
-    _base: Optional[int]
-    _type: ET
+    _data: buf
+    _head: AnyLIEF | None
+    _base: int | None
 
     blob: ClassVar[bool] = False
 
-    @classmethod
-    def Load(cls: Type[_T], data: ByteStr, base: Optional[int] = None) -> _T:
-        """
-        Uses the `refinery.lib.executable.exeroute` function to parse the input data with one of
-        the following specializations of this class:
+    @property
+    @abstractmethod
+    def _type(self) -> ET:
+        pass
 
-        - `refinery.lib.executable.ExecutableELF`
-        - `refinery.lib.executable.ExecutableMachO`
-        - `refinery.lib.executable.ExecutablePE`
+    @classmethod
+    def Load(cls, data: buf, base: int | None = None) -> LIEF:
+        """
+        Uses the `refinery.lib.lief.load` function to parse the input data.
         """
         if (parsed := lief.load(data)) is None:
             raise ValueError('LIEF was unable to parse the input.')
+        if isinstance(parsed, lief.COFF.Binary):
+            raise NotImplementedError
         return LIEF(parsed, data, base)
 
-    def __init__(self, head: AnyLIEF, data: ByteStr, base: Optional[int] = None):
+    def __init__(self, head: AnyLIEF | None, data: buf, base: int | None = None):
         self._data = data
         self._head = head
         self._base = base
@@ -349,10 +341,10 @@ class Executable(ABC):
         """
         return self._type
 
-    def __getitem__(self, key: Union[int, slice, Range]):
+    def __getitem__(self, key: int | slice | Range):
         return self.read(key)
 
-    def __contains__(self, key: Union[int, slice, Range]):
+    def __contains__(self, key: int | slice | Range):
         try:
             self.read(key)
         except LookupError:
@@ -360,7 +352,7 @@ class Executable(ABC):
         else:
             return True
 
-    def read_integer(self, address: int, size: Optional[int] = None):
+    def read_integer(self, address: int, size: int | None = None):
         """
         Read an integer at the given address. The size defaults to the pointer size in bytes.
         """
@@ -368,7 +360,7 @@ class Executable(ABC):
             size = self.pointer_size_in_bytes
         return int.from_bytes(self[address:address + size], self.byte_order().value)
 
-    def read(self, key: Union[int, slice, Range]) -> memoryview:
+    def read(self, key: int | slice | Range) -> memoryview:
         """
         Read data from the binary based on a given address. If the input `key` is a single integer,
         the function reads a single byte from the given address.
@@ -376,6 +368,8 @@ class Executable(ABC):
         if isinstance(key, Range):
             key = slice(key.lower, key.upper)
         elif isinstance(key, int):
+            if key < 0:
+                raise LookupError(F'Reading from negative address -0x{-key:X}.')
             key = slice(key, key + 1, 1)
         if key.start is None:
             raise LookupError(R'Slice indices with unspecified start are not supported.')
@@ -394,7 +388,7 @@ class Executable(ABC):
         return self.data[box.physical.position:end]
 
     @staticmethod
-    def ascii(string: Union[str, ByteStr]) -> str:
+    def ascii(string: str | buf) -> str:
         """
         If the input `string` is a `str` instance, the function returns the input value. Byte
         strings are truncated to the first occurrence of a null byte and then decoded using
@@ -434,8 +428,7 @@ class Executable(ABC):
         Return a (readonly) view to the raw bytes of the executable image.
         """
         view = memoryview(self._data)
-        if sys.version_info >= (3, 8):
-            view = view.toreadonly()
+        view = view.toreadonly()
         return view
 
     @property
@@ -499,7 +492,7 @@ class Executable(ABC):
         for section in self.sections():
             upper = max(upper, section.virtual.upper)
             lower = min(lower, section.virtual.lower)
-        if upper < lower:
+        if not isinstance(lower, int) or upper < lower:
             raise RuntimeError(F'The computed address space upper bound 0x{upper:X} is less than the computed lower bound 0x{lower:X}.')
         return Range(lower, upper)
 
@@ -524,10 +517,10 @@ class Executable(ABC):
             raise CompartmentNotFound(lt, location)
 
     @abstractmethod
-    def _symbols(self) -> Generator[Symbol, None, None]:
+    def _symbols(self) -> Generator[Symbol]:
         ...
 
-    def symbols(self) -> Generator[Symbol, None, None]:
+    def symbols(self) -> Generator[Symbol]:
         """
         Generates a list of symbols in the executable.
         """
@@ -557,30 +550,30 @@ class Executable(ABC):
         ...
 
     @abstractmethod
-    def _sections(self) -> Generator[Section, None, None]:
+    def _sections(self) -> Generator[Section]:
         ...
 
     @abstractmethod
-    def _segments(self, populate_sections=False) -> Generator[Segment, None, None]:
+    def _segments(self, populate_sections=False) -> Generator[Segment]:
         ...
 
     @abstractmethod
-    def _relocations(self) -> Generator[Relocation, None, None]:
+    def _relocations(self) -> Generator[Relocation]:
         ...
 
-    def segments(self, populate_sections=False) -> Generator[Segment, None, None]:
+    def segments(self, populate_sections=False) -> Generator[Segment]:
         """
         An iterable of all `refinery.lib.executable.Segment`s in this executable.
         """
         yield from self._segments(populate_sections=populate_sections)
 
-    def relocations(self) -> Generator[Relocation, None, None]:
+    def relocations(self) -> Generator[Relocation]:
         """
         An iterable of all `refinery.lib.executable.Relocation`s defined in this executable.
         """
         yield from self._relocations()
 
-    def sections(self) -> Generator[Section, None, None]:
+    def sections(self) -> Generator[Section]:
         """
         An iterable of all `refinery.lib.executable.Section`s in this executable.
         """
@@ -627,7 +620,7 @@ class Executable(ABC):
             Arch.X32     : (cs.CS_ARCH_X86,   cs.CS_MODE_32),     # noqa
             Arch.X64     : (cs.CS_ARCH_X86,   cs.CS_MODE_64),     # noqa
             Arch.ARM32   : (cs.CS_ARCH_ARM,   cs.CS_MODE_ARM),    # noqa
-            Arch.ARM64   : (cs.CS_ARCH_ARM,   cs.CS_MODE_THUMB),  # noqa
+            Arch.ARM64   : (cs.CS_ARCH_ARM64, cs.CS_MODE_ARM),    # noqa
             Arch.MIPS16  : (cs.CS_ARCH_MIPS,  cs.CS_MODE_16),     # noqa
             Arch.MIPS32  : (cs.CS_ARCH_MIPS,  cs.CS_MODE_32),     # noqa
             Arch.MIPS64  : (cs.CS_ARCH_MIPS,  cs.CS_MODE_64),     # noqa
@@ -652,13 +645,16 @@ class ExecutableCodeBlob(Executable):
     provided in the constructor for this object.
     """
 
-    _head: Type[None] = None
-    _type = ET.BLOB
+    _head = None
     _byte_order: BO
     _arch: Arch
 
     blob = True
 
+    @property
+    def _type(self):
+        return ET.BLOB
+
     def __init__(self, data, base=None, arch: Arch = Arch.X32, byte_order: BO = BO.LE):
         super().__init__(None, data, base)
         self._byte_order = byte_order
@@ -673,41 +669,52 @@ class ExecutableCodeBlob(Executable):
     def arch(self) -> Arch:
         return self._arch
 
-    def _symbols(self) -> Generator[Symbol, None, None]:
-        yield Symbol(0, is_entry=True)
+    def _symbols(self) -> Generator[Symbol]:
+        section, = self._sections()
+        yield Symbol(
+            self.base,
+            None,
+            None,
+            function=True,
+            exported=True,
+            imported=False,
+            is_entry=True,
+            section=section,
+        )
 
     def _relocations(self):
         yield from ()
 
-    def _sections(self) -> Generator[Section, None, None]:
+    def _sections(self) -> Generator[Section]:
         v = Range(self.base, self.base + len(self.data))
         p = Range(0, len(self.data))
         yield Section('blob', p, v, False)
 
-    def _segments(self, populate_sections=False) -> Generator[Segment, None, None]:
+    def _segments(self, populate_sections=False) -> Generator[Segment]:
         for s in self.sections():
             yield s.as_segment(populate_sections=populate_sections)
 
 
 class LIEF(Executable):
 
-    _head: AnyLIEF
-    _type: Union[ET.PE, ET.ELF, ET.MachO]
-
     @property
-    def _lh(self) -> lief.Binary:
+    def _lh(self) -> lief.AbstractBinary:
         return self._first_header.abstract
 
     @property
-    def _first_header(self) -> Union[MachOBinary, ELFBinary, PEBinary]:
+    def _first_header(self) -> MachOBinary | ELFBinary | PEBinary:
         head = self._head
-        if isinstance(self._head, lief.MachO.FatBinary):
+        if head is None:
+            raise AttributeError
+        if isinstance(head, lief.COFF.Binary):
+            raise NotImplementedError
+        if isinstance(head, lief.MachO.FatBinary):
             head = head.at(0)
         return head
 
     @property
     def _type(self):
-        EF = lief.Binary.FORMATS
+        EF = lief.AbstractBinary.FORMATS
         HF = self._lh.format
         if HF is EF.UNKNOWN:
             raise AttributeError('Unknown executable type.')
@@ -717,11 +724,7 @@ class LIEF(Executable):
         return self._lh.imagebase
 
     def byte_order(self) -> BO:
-        LE = lief.Header.ENDIANNESS
-        return {
-            LE.BIG    : BO.BE,
-            LE.LITTLE : BO.LE,
-        }.get(self._lh.header.endianness, BO.LE)
+        return BO.BE if self._lh.header.endianness == lief.Header.ENDIANNESS.BIG else BO.LE
 
     def arch(self) -> Arch:
         LA = lief.Header.ARCHITECTURES
@@ -763,11 +766,9 @@ class LIEF(Executable):
 
     def _relocations(self):
         ps = self.pointer_size_in_bytes
-        it: Iterable[lief.Relocation] = iter(self._lh.relocations)
 
-        if self.is_elf:
-            for rel in it:
-                assert isinstance(rel, lief.ELF.Relocation)
+        if isinstance((head := self._first_header), lief.ELF.Binary):
+            for rel in head.object_relocations:
                 if (symbol := rel.symbol) is None:
                     continue
                 if '_RELATIVE' in (tn := rel.type.name):
@@ -776,55 +777,154 @@ class LIEF(Executable):
                     base = symbol.value
                 else:
                     continue
-                yield Relocation(rel.address, base + rel.addend, ps)
-        elif (delta := (base := self.base) - self.image_defined_base()) != 0:
+                addr = self.rebase_img_to_usr(rel.address)
+                yield Relocation(addr, base + rel.addend, ps)
+        elif (delta := self.base - (base := self.image_defined_base())) != 0:
             def relocation(addr: int, size: int):
-                return Relocation(addr, self.read_integer(addr, size) + delta & mask, size)
-            mask = (1 << self.pointer_size) - 1
-            for rel in it:
+                addr = self.rebase_img_to_usr(addr)
+                mask = (1 << size) - 1
+                rval = self.read_integer(addr, size) + delta & mask
+                return Relocation(addr, rval, size // 8)
+
+            def relocation_pe(entry: lief.PE.RelocationEntry):
+                if entry.type == lief.PE.RelocationEntry.BASE_TYPES.HIGHLOW:
+                    size = 32
+                elif entry.type == lief.PE.RelocationEntry.BASE_TYPES.DIR64:
+                    size = 64
+                else:
+                    return
+                if size != entry.size:
+                    raise RuntimeError(F'Unexpected relocation size: Guessed {size}, LIEF says {entry.size}.')
+                yield relocation(base + entry.address, size)
+
+            for rel in self._lh.relocations:
                 if isinstance(rel, lief.MachO.Relocation):
-                    yield relocation(rel.address, rel.size // 8)
+                    yield relocation(rel.address, rel.size)
+                elif isinstance(rel, lief.PE.RelocationEntry):
+                    yield from relocation_pe(rel)
                 elif isinstance(rel, lief.PE.Relocation):
-                    rva = rel.virtual_address
-                    if (delta := (base := self.base) - self.image_defined_base()) == 0:
-                        continue
                     for entry in rel.entries:
-                        if entry.type == lief.PE.RelocationEntry.BASE_TYPES.HIGHLOW:
-                            size = 4
-                        elif entry.type == lief.PE.RelocationEntry.BASE_TYPES.DIR64:
-                            size = 8
-                        else:
-                            continue
-                        if size != entry.size:
-                            raise RuntimeError(F'Unexpected relocation size: Guessed {size}, LIEF says {entry.size}.')
-                        yield relocation(base + rva + entry.position, size)
+                        yield from relocation_pe(entry)
                 else:
-                    raise TypeError(F'Unexpected relocation type: {type(rel).__qualname__}')
+                    t = type(rel)
+                    raise TypeError(
+                        F'Unexpected relocation type: {t.__module__}.{t.__qualname__}')
+
+    def _symbols(self) -> Generator[Symbol]:
+        yield Symbol(
+            self.rebase_img_to_usr(self._lh.entrypoint),
+            None,
+            None,
+            function=True,
+            exported=True,
+            imported=False,
+            is_entry=True,
+        )
 
-    def _symbols(self) -> Generator[Symbol, None, None]:
-        yield Symbol(self._lh.entrypoint, is_entry=True)
         it: Iterable[lief.Symbol] = self._lh.symbols
+        ps = self.pointer_size_in_bytes
+        imports_done: set[str] = set()
+
+        if isinstance((head := self._first_header), lief.PE.Binary):
+            for imp in head.imports:
+                dll = self.ascii(imp.name).lower()
+                dll, _, ext = dll.rpartition('.')
+                dll = dll or ext
+                for symbol in imp.entries:
+                    if symbol.is_ordinal:
+                        name = F'@{symbol.ordinal}'
+                    else:
+                        name = self.ascii(symbol.demangled_name) or self.ascii(symbol.name)
+                    yield Symbol(
+                        symbol.iat_address + self.base,
+                        F'{dll}.{name}',
+                        ps,
+                        function=True,
+                        exported=False,
+                        imported=True,
+                    )
+        elif isinstance(head, lief.MachO.Binary):
+            for dlyd in (
+                head.dyld_chained_fixups,
+                head.dyld_info,
+            ):
+                if not dlyd:
+                    continue
+                for binding in dlyd.bindings:
+                    name = binding.symbol.demangled_name or binding.symbol.name
+                    name = self.ascii(name)
+                    addr = self.rebase_img_to_usr(binding.address)
+                    imports_done.add(name)
+                    yield Symbol(
+                        addr,
+                        name,
+                        ps,
+                        function=True,
+                        exported=False,
+                        imported=True,
+                    )
+        elif isinstance(head, lief.ELF.Binary):
+            for bindings in (
+                head.pltgot_relocations,
+                head.dynamic_relocations,
+            ):
+                if not bindings:
+                    continue
+                for binding in bindings:
+                    name = binding.symbol.demangled_name or binding.symbol.name
+                    name = self.ascii(name)
+                    addr = self.rebase_img_to_usr(binding.address)
+                    imports_done.add(name)
+                    yield Symbol(
+                        addr,
+                        name,
+                        ps,
+                        function=True,
+                        exported=False,
+                        imported=True,
+                    )
+
         for symbol in it:
-            addr = symbol.value
+            addr = self.rebase_img_to_usr(value := symbol.value)
             name = self.ascii(symbol.name)
             size = symbol.size
-            if isinstance(symbol, lief.PE.Symbol):
+            if isinstance(symbol, lief.COFF.Symbol):
+                name = self.ascii(symbol.demangled_name) or name
+                code = symbol.is_function or (
+                    symbol.complex_type == lief.COFF.Symbol.COMPLEX_TYPE.FUNCTION)
+                ext = symbol.is_external
+                imp = not ext
                 yield Symbol(
-                    addr, name, size,
-                    function=symbol.complex_type == lief.PE.SYMBOL_COMPLEX_TYPES.FUNCTION,
+                    addr,
+                    name,
+                    size,
+                    function=code,
+                    exported=ext,
+                    imported=imp,
                     section=self._convert_section(s) if (s := symbol.section) else None,
                 )
             elif isinstance(symbol, lief.PE.ImportEntry):
-                yield Symbol(
-                    addr, name, size,
-                    function=True,
-                    imported=True,
-                    exported=False,
-                )
-            elif isinstance(symbol, lief.ELF.Symbol):
+                continue
+            elif isinstance(symbol, lief.PE.ExportEntry):
                 name = self.ascii(symbol.demangled_name) or name
                 yield Symbol(
-                    addr, name, size,
+                    value + self.base,
+                    name,
+                    ps,
+                    function=True,
+                    exported=True,
+                    imported=False,
+                )
+            elif isinstance(symbol, lief.ELF.Symbol):
+                if symbol.imported and symbol.is_function:
+                    continue
+                name = self.ascii(symbol.demangled_name) or name
+                if name in imports_done:
+                    continue
+                yield Symbol(
+                    addr,
+                    name,
+                    size,
                     exported=symbol.exported,
                     imported=symbol.imported,
                     function=symbol.is_function,
@@ -832,15 +932,27 @@ class LIEF(Executable):
                 )
             elif isinstance(symbol, lief.MachO.Symbol):
                 name = self.ascii(symbol.demangled_name) or name
+                if name in imports_done:
+                    continue
                 yield Symbol(
-                    addr, name, size,
+                    addr,
+                    name,
+                    size,
+                    function=True,
                     imported=symbol.is_external,
                     exported=symbol.has_export_info,
                 )
             else:
-                yield Symbol(addr, name, size)
+                yield Symbol(
+                    addr,
+                    name,
+                    size,
+                    function=False,
+                    imported=False,
+                    exported=False,
+                )
 
-    def _convert_section(self, section: lief.Section, segment_name: Optional[str] = None) -> Section:
+    def _convert_section(self, section: lief.Section, segment_name: str | None = None) -> Section:
         p_lower = section.offset
         p_upper = p_lower + section.size
 
@@ -873,7 +985,7 @@ class LIEF(Executable):
     def is_macho(self):
         return isinstance(self._head, (lief.MachO.Binary, lief.MachO.FatBinary))
 
-    def _sections(self) -> Generator[Section, None, None]:
+    def _sections(self) -> Generator[Section]:
         if self.is_pe:
             it: Iterable[lief.Section] = self._lh.sections
             for section in it:
@@ -894,23 +1006,23 @@ class LIEF(Executable):
             if sections := segment.sections:
                 yield from sections
 
-    def _segments(self, populate_sections=False) -> Generator[Segment, None, None]:
+    def _segments(self, populate_sections=False) -> Generator[Segment]:
         if self.is_pe:
             for section in self.sections():
                 yield section.as_segment(populate_sections)
         else:
-            it: Iterable[Union[
-                lief.ELF.Segment,
-                lief.MachO.SegmentCommand
-            ]] = self._first_header.segments
-            for segment in it:
+            fh = self._first_header
+            assert isinstance(fh, (lief.MachO.Binary, lief.ELF.Binary))
+            for segment in fh.segments:
                 p_lower = segment.file_offset
                 try:
                     p_upper = p_lower + segment.file_size
                 except AttributeError:
                     p_upper = p_lower + segment.physical_size
                 v_lower = segment.virtual_address
-                v_lower = self.rebase_usr_to_img(v_lower)
+                v_lower = self.rebase_img_to_usr(v_lower)
+                if v_lower < 0:
+                    continue # __PAGEZERO
                 v_upper = v_lower + segment.virtual_size
                 try:
                     name = segment.name
