@@ -4,8 +4,8 @@ Parsing of the Java Class file format as per
 """
 from __future__ import annotations
 
-from typing import Union, Any, Dict, List, ByteString, Type, Optional, TypeVar, Generic
 from enum import IntEnum
+from typing import Any, Generic, TypeVar
 
 from refinery.lib.structures import (
     AttrType,
@@ -14,6 +14,7 @@ from refinery.lib.structures import (
     Struct,
     StructReader,
 )
+from refinery.lib.types import buf
 
 __all__ = (
     'opc',
@@ -35,8 +36,8 @@ __all__ = (
 )
 
 
-class Index(PerInstanceAttribute):
-    def __init__(self, jtype: Type[AttrType]):
+class Index(PerInstanceAttribute[AttrType]):
+    def __init__(self, jtype: type[AttrType]):
         super().__init__()
         self.jtype = jtype
 
@@ -171,7 +172,7 @@ class JvAccessFlags(Struct):
         ) = reader.read_flags(16)
 
 
-ParserType = TypeVar('ParserType')
+ParserType = TypeVar('ParserType', bound=Struct)
 
 
 class JvAttribute(JvStructWithName, Generic[ParserType]):
@@ -180,8 +181,8 @@ class JvAttribute(JvStructWithName, Generic[ParserType]):
         self.name = reader.u16()
         self.data = reader.read(reader.u32())
 
-    def parse(self, parser: Type[ParserType]) -> ParserType:
-        return parser(self.data, pool=self.pool)
+    def parse(self, parser: type[ParserType]) -> ParserType:
+        return parser.Parse(self.data, pool=self.pool)
 
 
 class JvClassMember(JvStructWithName):
@@ -506,7 +507,7 @@ class JvOpCode(Struct):
     def __init__(self, reader: StructReader, pool: list):
         with StreamDetour(reader):
             self.code = opc(reader.read_byte())
-            self.table: Optional[Dict[int, int]] = None
+            self.table: dict[int, int] | None = None
             try:
                 fmt = self.OPC_ARGMAP[self.code]
             except KeyError:
@@ -568,18 +569,18 @@ class JvCode(Struct):
         reader.bigendian = True
         self.max_stack = reader.u16()
         self.max_locals = reader.u16()
-        self.disassembly: List[JvOpCode] = []
+        self.disassembly: list[JvOpCode] = []
         with StructReader(reader.read(reader.u32())) as code:
             code.bigendian = True
             while not code.eof:
-                self.disassembly.append(JvOpCode(code, pool=pool))
+                self.disassembly.append(JvOpCode.Parse(code, pool=pool))
         self.exceptions = [JvException(reader      ) for _ in range(reader.u16())] # noqa
         self.attributes = [JvAttribute(reader, pool) for _ in range(reader.u16())] # noqa
 
 
 class JvClassFile(Struct):
 
-    TYPEHANDLER: Dict[JvConstType, Struct] = {
+    TYPEHANDLER: dict[JvConstType, Struct] = {
         JvConstType.Class            : JvString,
         JvConstType.String           : JvString,
         JvConstType.Field            : JvClassProperty,
@@ -605,10 +606,10 @@ class JvClassFile(Struct):
         major = reader.u16()
         self.version = (major, minor)
 
-        self.pool: List[Union[Struct, int, float, str]] = []
+        self.pool: list[Struct | int | float | str] = []
         self._read_pool(reader)
 
-        self.strings: List[str] = {
+        self.strings: list[str] = {
             s.value for s in self.pool if isinstance(s, Struct) and s.tag == JvConstType.String}
 
         self.access = JvAccessFlags(reader)
@@ -638,7 +639,7 @@ class JvClassFile(Struct):
             raise ValueError('Failed parsing Attributes.')
 
     @staticmethod
-    def decode_utf8m(string: ByteString) -> str:
+    def decode_utf8m(string: buf) -> str:
         """
         Based on the following code:
         https://gist.github.com/BarelyAliveMau5/000e7e453b6d4ebd0cb06f39bc2e7aec
