@@ -4,14 +4,12 @@ json module in order to support custom encoding of already serializable types.
 """
 from __future__ import annotations
 
-from typing import List, Tuple, Union
-
 import datetime
 import json
 import re
 import uuid
 
-from refinery.lib.types import ByteStr
+from refinery.lib.types import buf
 
 
 class JSONEncoderExMeta(type):
@@ -78,6 +76,8 @@ class JSONEncoderEx(json.JSONEncoder, metaclass=JSONEncoderExMeta):
     def default(self, obj):
         if isinstance(obj, datetime.datetime):
             return obj.isoformat(' ', 'seconds')
+        if isinstance(obj, uuid.UUID):
+            return str(obj).upper()
         return super().default(obj)
 
     def __init__(self, *args, **kwargs):
@@ -107,7 +107,7 @@ class BytesEncoder(JSONEncoderEx):
     def handled(cls, obj) -> bool:
         return cls._is_byte_array(obj) or super().handled(obj)
 
-    def encode_bytes(self, obj: ByteStr):
+    def encode_bytes(self, obj: buf):
         raise NotImplementedError
 
     def default(self, obj):
@@ -120,7 +120,7 @@ class BytesAsArrayEncoder(BytesEncoder):
     """
     This JSON Encoder encodes byte strings as arrays of integers.
     """
-    def encode_bytes(self, obj: ByteStr):
+    def encode_bytes(self, obj: buf):
         return self.encode_raw('[{}]'.format(','.join(str(b & 0xFF) for b in obj)))
 
 
@@ -128,7 +128,7 @@ class BytesAsStringEncoder(BytesEncoder):
     """
     This JSON Encoder encodes byte strings as escaped strings.
     """
-    def encode_bytes(self, obj: ByteStr):
+    def encode_bytes(self, obj: buf):
         if not isinstance(obj, (bytes, bytearray)):
             if not isinstance(obj, memoryview):
                 obj = (b & 0xFF for b in obj)
@@ -136,7 +136,7 @@ class BytesAsStringEncoder(BytesEncoder):
         return obj.decode('latin1')
 
 
-def flattened(data: dict, prefix='', separator='.') -> List[Tuple[str, Union[int, float, str]]]:
+def flattened(data: dict, prefix='', separator='.') -> list[tuple[str, int | float | str]]:
     def flatten(cursor, prefix):
         if isinstance(cursor, dict):
             for key, value in cursor.items():
