@@ -4,48 +4,46 @@ A wrapper around the LIEF library.
 from __future__ import annotations
 
 import io
+
 import lief as lib
+import lief.COFF as COFF
+import lief.ELF as ELF
+import lief.MachO as MachO
+import lief.PE as PE
 
 if True:
     lib.logging.disable()
 
-from lief import (
-    MachO,
-    PE,
-    ELF,
-    Binary,
-    Relocation,
-    Header,
-    Symbol,
-    Section,
-)
+from refinery.lib.types import buf
 
-from refinery.lib.types import ByteStr
-
-__pdoc__ = {_forward: False for _forward in [
-    'MachO', 'PE', 'ELF', 'Binary', 'Header', 'Symbol', 'Section', 'Relocation'
-]}
+AbstractBinary = lib.Binary
+Relocation = lib.Relocation
+Header = lib.Header
+Symbol = lib.Symbol
+Section = lib.Section
 
 __all__ = [
-    'load',
-    'load_pe',
-    'load_pe_fast',
-    'load_macho',
-    'string',
-    'lib',
+    'ELF',
     'MachO',
     'PE',
-    'ELF',
-    'Binary',
+    'COFF',
+    'AbstractBinary',
     'Relocation',
     'Header',
     'Symbol',
     'Section',
+    'load_pe',
+    'load_pe_fast',
+    'load_macho',
+    'load',
+    'string',
 ]
 
+__pdoc__ = {_forward: False for _forward in __all__[:9]}
+
 
 def load_pe(
-    data: ByteStr,
+    data: buf,
     parse_exports: bool = True,
     parse_imports: bool = True,
     parse_reloc: bool = True,
@@ -66,12 +64,11 @@ def load_pe(
         cfg.parse_signature = bool(parse_signature)
         if parsed := PE.parse(stream, cfg):
             return parsed
-        stream.seek(0)
-        return lib.parse(stream)
+        raise ValueError
 
 
 def load_pe_fast(
-    data: ByteStr,
+    data: buf,
     parse_exports: bool = False,
     parse_imports: bool = False,
     parse_reloc: bool = False,
@@ -92,44 +89,57 @@ def load_pe_fast(
     )
 
 
-def load_macho(data: ByteStr) -> MachO.FatBinary | MachO.Binary:
+def load_macho(data: buf) -> MachO.FatBinary | MachO.Binary:
     """
     Load a MachO file using LIEF.
     """
     with io.BytesIO(data) as stream:
-        if parsed := MachO.parse(stream):
+        config = MachO.ParserConfig()
+        config.parse_dyld_bindings = True
+        config.parse_dyld_rebases = True
+        config.parse_dyld_exports = True
+        if parsed := MachO.parse(stream, config):
             return parsed
-        stream.seek(0)
-        return lib.parse(stream)
+        raise ValueError
 
 
-def load(data: ByteStr):
+def load_elf(data: buf) -> ELF.Binary:
+    """
+    Load an ELF file using LIEF.
+    """
+    with io.BytesIO(data) as stream:
+        config = ELF.ParserConfig()
+        config.parse_dyn_symbols = True
+        config.parse_relocations = True
+        config.parse_symtab_symbols = True
+        if parsed := ELF.parse(stream):
+            return parsed
+        raise ValueError
+
+
+def load(data: buf):
     """
     Load a PE, ELF, or MachO executable using LIEF. The function first attempts to parse the file
     based on its first 4 bytes using a specific LIEF parser and reverts to LIEF's general purpose
     loader if these fail.
     """
-    with io.BytesIO(data) as stream:
-        if data[:2] == B'MZ':
-            parsed = PE.parse(stream)
-        elif data[:4] == B'\x7FELF':
-            parsed = ELF.parse(stream)
-        elif set(data[:4]) <= {0xFE, 0xED, 0xFA, 0xCE, 0xCF}:
-            parsed = MachO.parse(stream)
-        else:
-            raise ValueError
-        if parsed:
-            return parsed
-        stream.seek(0)
-        return lib.parse(stream)
+    if data[:2] == B'MZ':
+        return load_pe(data)
+    if data[:4] == B'\x7FELF':
+        return load_elf(data)
+    if set(data[:4]) <= {0xFE, 0xED, 0xFA, 0xCE, 0xCF}:
+        return load_macho(data)
+    raise ValueError
 
 
-def string(value: str | bytes) -> str:
+def string(value: str | buf) -> str:
     """
     A function to convert LIEF values to a string, regardless of whether it is exposed as bytes
     or string by the foreign interface.
     """
     if not isinstance(value, str):
+        if isinstance(value, memoryview):
+            value = bytes(value)
         value, _, _ = value.partition(B'\0')
         value = value.decode('utf8')
     return value
