@@ -18,7 +18,15 @@ except ModuleNotFoundError:
             magic = None
 
 
-def magicparse(data, *args, **kwargs):
-    if magic:
-        data = bytes(data) if not isinstance(data, bytes) else data
-        return magic.Magic(*args, **kwargs).from_buffer(data)
+def magicparse(data, *args, **kwargs) -> str:
+    if magic is not None:
+        if not isinstance(data, bytes):
+            data = bytes(data)
+        try:
+            return magic.Magic(*args, **kwargs).from_buffer(data)
+        except magic.MagicException:
+            pass
+    elif kwargs.get('mime', False) is True:
+        return 'application/octet-stream'
+    else:
+        return 'data'
