@@ -112,24 +112,26 @@ ways to merge down the metadata from chunks inside sub-pipelines.
 from __future__ import annotations
 
 import abc
-import contextlib
-import string
 import codecs
+import contextlib
 import itertools
 import os
 import re
+import string
 
 from io import StringIO
+from typing import TYPE_CHECKING, Any, Callable, Iterable
 from urllib.parse import unquote_to_bytes
-from typing import Callable, Dict, List, Tuple, Any, Iterable, Optional, ByteString, Union, TYPE_CHECKING
 
-from refinery.lib.structures import MemoryFile
-from refinery.lib.tools import isbuffer, entropy, typename, index_of_coincidence
 from refinery.lib.environment import environment
-
+from refinery.lib.mime import get_cached_file_magic_info
+from refinery.lib.structures import MemoryFile
+from refinery.lib.tools import entropy, index_of_coincidence, isbuffer, typename
+from refinery.lib.types import buf
 
 if TYPE_CHECKING:
     from typing import Protocol
+
     from refinery.lib.frame import Chunk
 
     class _Derivation(Protocol):
@@ -137,7 +139,7 @@ if TYPE_CHECKING:
         name: str
         wrap: type
 
-        def __call__(self, object: LazyMetaOracle) -> Union[str, int, float]:
+        def __call__(self, object: LazyMetaOracle) -> str | int | float:
             ...
 
 
@@ -156,26 +158,25 @@ class CustomStringRepresentation(abc.ABC):
 
 
 _INDEX = 'index'
+_BIGINT = '__bi__'
 
-_PRINT_SAFE = set(string.printable.encode('latin1')) - set(b'|<>\t\n\r\v')
+_HIGH_ASCII = '¹²³«»¡¿¼½¾¢£¥§©®±µ·÷øÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõöùúûüýþÿ'
+_8BIT_ASCII = string.printable + _HIGH_ASCII
+_PRINT_SAFE = set(_8BIT_ASCII.encode('latin1')) - set(b'|<>\t\n\r\v')
 if os.name == 'nt':
     _PRINT_SAFE -= set(b'^"')
 else:
     _PRINT_SAFE -= set(b'&*?\'"')
-_PRINT_SAFE = bytes(_PRINT_SAFE)
-_IS_PRINT_SAFE = bytearray(256)
+
+_PT = bytearray(256)
 for p in _PRINT_SAFE:
-    _IS_PRINT_SAFE[p] = 1
+    _PT[p] = 1
 
 
 def is_print_safe(string: str):
-    if not string.isprintable():
-        return False
-    for letter in string:
-        code = ord(letter)
-        if code < len(_IS_PRINT_SAFE) and not _IS_PRINT_SAFE[code]:
-            return False
-    return True
+    return string.isprintable() or not any(
+        (code := ord(letter)) < len(_PT) and not _PT[code] for letter in string
+    )
 
 
 class ByteStringWrapper(bytearray, CustomStringRepresentation):
@@ -191,20 +192,18 @@ class ByteStringWrapper(bytearray, CustomStringRepresentation):
     }
 
     @classmethod
-    def Wrap(cls, string: Union[str, ByteString, ByteStringWrapper], codec: Optional[str] = None):
+    def Wrap(cls, string: str | buf | ByteStringWrapper, codec: str | None = None):
         if isinstance(string, cls):
             return string
         return cls(string, codec=codec)
 
-    def __init__(self, string: Union[str, ByteString], codec: Optional[str] = None):
+    def __init__(self, string: str | buf, codec: str | None = None):
         if isinstance(string, str):
             self._string = string
-            self._buffer = False
             codec = codec or 'utf8'
             string = string.encode(codec)
         elif isbuffer(string):
             self._string = None
-            self._buffer = True
         else:
             raise TypeError(F'The argument {string!r} is not a buffer or string.')
 
@@ -237,7 +236,7 @@ class ByteStringWrapper(bytearray, CustomStringRepresentation):
             codecs = self._CODECS if _codec is None else [_codec, 'latin1']
             for codec in codecs:
                 try:
-                    value = self.decode(codec)
+                    self._string = value = self.decode(codec)
                 except UnicodeError as e:
                     _error = _error or e
                 else:
@@ -257,7 +256,7 @@ class ByteStringWrapper(bytearray, CustomStringRepresentation):
 
     def __repr__(self):
         try:
-            return self._representation
+            return self._pretty
         except AttributeError:
             pass
         try:
@@ -265,24 +264,24 @@ class ByteStringWrapper(bytearray, CustomStringRepresentation):
                 prefix = None
             else:
                 try:
-                    representation = self.decode('utf-16le')
+                    pretty = self.decode('utf-16le')
                     prefix = 'u'
                 except UnicodeDecodeError:
                     prefix = None
             if prefix is None:
-                representation = self.string
+                pretty = self.string
                 prefix = self._CODECS[self.codec]
         except AttributeError:
-            representation = None
+            pretty = None
         else:
-            if not is_print_safe(representation):
-                representation = None
-            elif prefix != 's' or self.requires_prefix(representation):
-                representation = F'{prefix}:{representation}'
-        if representation is None:
-            representation = F'h:{self.hex()}'
-        self._representation = representation
-        return representation
+            if not is_print_safe(pretty):
+                pretty = None
+            elif prefix != 's' or self.requires_prefix(pretty):
+                pretty = F'{prefix}:{pretty}'
+        if pretty is None:
+            pretty = F'h:{self.hex()}'
+        self._pretty = pretty
+        return pretty
 
     def __str__(self):
         return self.string
@@ -319,7 +318,7 @@ def is_valid_variable_name(name: str, allow_wildcards: bool = False) -> bool:
         return True
 
 
-def check_variable_name(name: Optional[str], allow_derivations=False) -> None:
+def check_variable_name(name: str | None, allow_derivations=False) -> None:
     """
     All single-letter, uppercase variable names are reserved. Additionally, derived
     property names should not be overwritten.
@@ -423,22 +422,24 @@ class LazyMetaOracle(metaclass=_LazyMetaMeta):
     key `'sha256'`.
     """
 
-    derivations: Dict[str, _Derivation] = {}
+    IndexKey = _INDEX
+
+    derivations: dict[str, _Derivation] = {}
     """
     A dictionary mapping the names of common properties to anonymous functions that compute their
     corresponding value on a chunk of binary input data.
     """
 
     ghost: bool
-    chunk: ByteString
-    cache: Dict[str, Union[str, int, float]]
-    index: Optional[int]
+    chunk: buf
+    cache: dict[str, str | int | float]
+    index: int | None
 
-    history: Dict[str, List[Tuple[bool, Any]]]
-    current: Dict[str, Any]
-    updated: Dict[str, bool]
+    history: dict[str, list[tuple[bool, Any]]]
+    current: dict[str, Any]
+    updated: dict[str, bool]
 
-    def __init__(self, chunk: ByteString, scope: Optional[int] = 1, seed: Optional[Dict[str, List[Tuple[bool, Any]]]] = None):
+    def __init__(self, chunk: buf, scope: int = 1, seed: dict[str, list[tuple[bool, Any]]] | None = None):
         self.ghost = False
         self.chunk = chunk
         self.cache = {}
@@ -459,6 +460,12 @@ class LazyMetaOracle(metaclass=_LazyMetaMeta):
                 for is_link, value in reversed(stack):
                     while is_link:
                         is_link, value = stack[value]
+                    try:
+                        bigint: bytes = value[_BIGINT]
+                    except Exception:
+                        pass
+                    else:
+                        value = int.from_bytes(bigint)
                     if value is not None:
                         self.current[key] = self.autowrap(key, value)
                         self.updated[key] = False
@@ -469,7 +476,7 @@ class LazyMetaOracle(metaclass=_LazyMetaMeta):
         else:
             self.history = {}
 
-    def update(self, other: Union[dict, LazyMetaOracle]):
+    def update(self, other: dict | LazyMetaOracle):
         if isinstance(other, LazyMetaOracle):
             self.current.update(other.current)
             self.updated.update(other.updated)
@@ -532,7 +539,7 @@ class LazyMetaOracle(metaclass=_LazyMetaMeta):
             return scope - k + 1
         return scope
 
-    def serialize(self, target_scope: int) -> Dict[str, List[Tuple[bool, Any]]]:
+    def serialize(self, target_scope: int) -> dict[str, list[tuple[bool, Any]]]:
         if not target_scope:
             return {}
         current_scope = self.scope
@@ -553,6 +560,10 @@ class LazyMetaOracle(metaclass=_LazyMetaMeta):
         for key, value in self.current.items():
             if value is None:
                 raise RuntimeError(F'Meta variable "{key}" was set to None.')
+            elif isinstance(value, int) and value > 0xFFFFFFFFFFFFFFFF:
+                q, r = divmod(value.bit_length(), 8)
+                q += int(bool(r))
+                value = {_BIGINT: value.to_bytes(q)}
             try:
                 item_scope = self.rescope[key]
             except KeyError:
@@ -615,9 +626,9 @@ class LazyMetaOracle(metaclass=_LazyMetaMeta):
         self,
         spec: str,
         codec: str,
-        args: Optional[Iterable] = None,
-        symb: Optional[dict] = None,
-        used: Optional[set] = None,
+        args: Iterable | None = None,
+        symb: dict | None = None,
+        used: set | None = None,
     ) -> str:
         """
         Formats the input expression like a normal Python format string expression. Certain refinery
@@ -635,10 +646,10 @@ class LazyMetaOracle(metaclass=_LazyMetaMeta):
         self,
         spec: str,
         codec: str,
-        args: Optional[Iterable] = None,
-        symb: Optional[dict] = None,
-        used: Optional[set] = None,
-    ) -> ByteString:
+        args: Iterable | None = None,
+        symb: dict | None = None,
+        used: set | None = None,
+    ) -> buf:
         """
         Formats the input expression using a Python F-string like expression. These strings contain
         fields in the format `{expression!T:pipeline}`, where `T` is a transformation character and
@@ -662,12 +673,12 @@ class LazyMetaOracle(metaclass=_LazyMetaMeta):
         self,
         spec    : str,
         codec   : str,
-        args    : Union[list, tuple],
-        symb    : dict,
+        args    : list | tuple,
+        symb    : dict | None,
         binary  : bool,
         fixup   : bool = True,
-        used    : Optional[set] = None,
-    ) -> Union[str, ByteString]:
+        used    : set | None = None,
+    ) -> str | buf:
         """
         Formats a string using Python-like string fomatting syntax. The formatter for `binary`
         mode is different; each formatting is documented in one of the following two proxy methods:
@@ -677,7 +688,11 @@ class LazyMetaOracle(metaclass=_LazyMetaMeta):
         """
         # prevents circular import:
         from refinery.lib.argformats import (
-            DelayedNumSeqArgument, ParserError, PythonExpression, Chunk)
+            Chunk,
+            DelayedNumSeqArgument,
+            ParserError,
+            PythonExpression,
+        )
 
         symb = symb or {}
 
@@ -706,8 +721,9 @@ class LazyMetaOracle(metaclass=_LazyMetaMeta):
         autoindex = 0
 
         if binary:
+            def putstr(s: str):
+                stream.write(s.encode(codec))
             stream = MemoryFile()
-            def putstr(s: str): stream.write(s.encode(codec))
         else:
             stream = StringIO()
             putstr = stream.write
@@ -955,17 +971,14 @@ class LazyMetaOracle(metaclass=_LazyMetaMeta):
 
     @_derivation('mime')
     def _derive_mime(self):
-        from refinery.lib.mime import get_cached_file_magic_info
         return get_cached_file_magic_info(self.chunk).mime
 
     @_derivation('ext')
     def _derive_ext(self):
-        from refinery.lib.mime import get_cached_file_magic_info
         return get_cached_file_magic_info(self.chunk).extension
 
     @_derivation('magic')
     def _derive_magic(self):
-        from refinery.lib.mime import get_cached_file_magic_info
         return get_cached_file_magic_info(self.chunk).description
 
     @_derivation('size', wrap=SizeInt)
@@ -1006,7 +1019,7 @@ class LazyMetaOracle(metaclass=_LazyMetaMeta):
         return hashlib.md5(self.chunk).hexdigest()
 
 
-def metavars(chunk: Union[Chunk, ByteString]) -> LazyMetaOracle:
+def metavars(chunk: Chunk | buf) -> LazyMetaOracle:
     """
     This method is the main function used by refinery units to get the meta variable dictionary
     of an input chunk. This dictionary is wrapped using the `refinery.lib.meta.LazyMetaOracleFactory`
