@@ -8,7 +8,22 @@ import functools
 import re
 
 from refinery.lib.patterns.tlds import tlds
-from refinery.lib.tools import normalize_to_identifier, normalize_to_display
+from refinery.lib.tools import normalize_to_display, normalize_to_identifier
+
+
+def _sized_suffix(lower: int, upper: int):
+    if lower <= 0:
+        if upper <= 0:
+            return '*'
+        else:
+            return F'{{1,{upper}}}'
+    elif upper <= 0:
+        if lower == 1:
+            return '+'
+        else:
+            return F'{{{lower},}}'
+    else:
+        return F'{{{lower},{upper}}}'
 
 
 class pattern:
@@ -18,23 +33,33 @@ class pattern:
     operators.
     """
     str_pattern: str
-    bin_pattern: bytes | None
+    bin_pattern: bytes
 
     def __init__(self, pattern: str, flags: int = 0):
         self.str_pattern = pattern
         self.bin_pattern = pattern.encode('ascii')
-        self.regex_flags = flags
+        self.flags = flags
 
     def __bytes__(self):
         return self.bin_pattern
 
     @functools.cached_property
     def bin(self):
-        return re.compile(B'(%s)' % self.bin_pattern, flags=self.regex_flags)
+        return re.compile(B'(%s)' % self.bin_pattern, flags=self.flags)
 
     @functools.cached_property
     def str(self):
-        return re.compile(self.str_pattern, flags=self.regex_flags)
+        return re.compile(self.str_pattern, flags=self.flags)
+
+    def __hash__(self):
+        return hash((self.str_pattern, self.flags))
+
+    def __eq__(self, other):
+        if isinstance(other, str):
+            return self.str_pattern == other and self.flags == 0
+        if isinstance(other, pattern):
+            return self.str_pattern == other.str_pattern and self.flags == other.flags
+        return False
 
     def __str__(self):
         return self.str_pattern
@@ -61,21 +86,51 @@ class pattern:
 class alphabet(pattern):
     """
     A pattern object representing strings of letters from a given alphabet, with
-    an optional prefix and postfix.
+    an optional prefix and suffix.
     """
-    def __init__(self, repeat, prefix='', postfix='', at_least=1, at_most=None, **kwargs):
-        if not at_most:
-            count = '+' if at_least <= 1 else '{{{},}}'.format(at_least)
-        else:
-            count = '{{{},{}}}(?!{})'.format(at_least, at_most, repeat)
-
+    def __init__(
+        self,
+        repeat: str,
+        prefix: str = '',
+        suffix: str = '',
+        lower: int = 1,
+        upper: int = 0,
+        prefix_min: int = 0,
+        prefix_max: int = 0,
+        suffix_min: int = 0,
+        suffix_max: int = 0,
+        token_size: int = 1,
+        flags: int = 0,
+        **kwargs
+    ):
+        self.repeat = repeat
+        self.prefix = prefix
+        self.suffix = suffix
+        self.suffix_min = suffix_min
+        self.suffix_max = suffix_max
+        self.prefix_min = prefix_min
+        self.prefix_max = prefix_max
+        self.token_size = token_size
+        lower = lower - suffix_max - prefix_max
+        upper = upper - suffix_min - prefix_min
+        if token_size > 1:
+            lower, _r = divmod(lower, token_size)
+            if _r and lower == 0:
+                lower = _r
+            upper, _r = divmod(upper, token_size)
+            if _r and upper >= 0:
+                upper += 1
+        self.lower = lower
+        self.upper = upper
+        count = _sized_suffix(lower, upper)
         pattern.__init__(self,
             R'{b}(?:{r}){c}{a}'.format(
                 r=repeat,
                 b=prefix,
                 c=count,
-                a=postfix
+                a=suffix
             ),
+            flags,
             **kwargs
         )
 
@@ -182,19 +237,29 @@ _pattern_ipv6 = (
     R')'                                                  # (IPv4-Embedded IPv6 Address)
 )
 
-_pattern_serrated_socket = '(?:{ip}|{d})(?::\\d{{2,5}})'.format(ip=_pattern_serrated_ipv4, d=_pattern_serrated_domain)
-_pattern_defanged_socket = '(?:{ip}|{d})(?::\\d{{2,5}})'.format(ip=_pattern_defanged_ipv4, d=_pattern_defanged_domain)
+_pattern_serrated_socket = f'(?:{_pattern_serrated_ipv4}|{_pattern_serrated_domain})(?::\\d{{2,5}})'
+_pattern_defanged_socket = f'(?:{_pattern_defanged_ipv4}|{_pattern_defanged_domain})(?::\\d{{2,5}})'
 
 _pattern_serrated_hostname = _pattern_serrated_socket + '?'
 _pattern_defanged_hostname = _pattern_defanged_socket + '?'
 
 
-_pattern_integer = '[-+]?(?:0[bB][01]+|0[xX][0-9a-fA-F]+|0[1-7][0-7]*|[1-9][0-9]*|0)(?=[uU]?[iI]\\d{1,2}|[LlHh]|[^a-zA-Z0-9]|$)'
+def _sized_pattern_integer(lower: int = 0, upper: int = 0):
+    x = _sized_suffix(max(1, lower - 3), upper - 2)
+    o = _sized_suffix(max(0, lower - 3), upper - 2)
+    d = _sized_suffix(max(0, lower - 2), upper - 1)
+    return (
+        F'[-+]?(?:0[bB][01]{x}|0[xX][0-9a-fA-F]{x}|0[1-7][0-7]{o}|[1-9][0-9]{d}|0)'
+        R'(?=[uU]?[iI]\d{1,2}|[LlHh]|[^a-zA-Z0-9]|$)'
+    )
+
+
+_pattern_integer = _sized_pattern_integer()
 _pattern_float = R'[-+]?[0-9]*\.?[0-9]+(?:[eE][-+]?[0-9]+)?'
 _pattern_number = F'(?:(?:{_pattern_integer})|(?:{_pattern_float}))'
 _pattern_number = (
-    '[-+]?(?:0[bB][01]+|0[xX][0-9a-fA-F]+|0[1-7][0-7]*|(?:[1-9][0-9]*|0)(?P<fp1>\\.[0-9]*)?|(?P<fp2>\\.[0-9]+))'
-    '(?(fp1)(?:[eE][-+]?[0-9]+)?|(?(fp2)(?:[eE][-+]?[0-9]+)?|(?=[uU]?[iI]\\d{1,2}|[LlHh]|[^a-zA-Z0-9]|$)))'
+    '[-+]?(?:0[bB][01]+|0[xX][0-9a-fA-F]+|0[1-7][0-7]*|(?:[1-9][0-9]*|0)(?P<__fp1>\\.[0-9]*)?|(?P<__fp2>\\.[0-9]+))'
+    '(?(__fp1)(?:[eE][-+]?[0-9]+)?|(?(__fp2)(?:[eE][-+]?[0-9]+)?|(?=[uU]?[iI]\\d{1,2}|[LlHh]|[^a-zA-Z0-9]|$)))'
 )
 
 
@@ -245,17 +310,29 @@ _pattern_date_list = [
     R'{B}\s(?:{d}|{D}){c}{Y}(?:\s{T})?',
     R'{Y}[-:]{m}[-:]{d}(?:[T\x20]{H}:{M}(?::{S})?(?:[Z.][0-9]{{6}})?{z}?)',
     R'{m}/{d}/{Y}(?:{c}{T})?',
+    R'{A}{c}{B}{c}(?:{d}|{D}){c}{T}(?:\s\(?UTC\)?)?\s{Y}',
 ]
 
 _pattern_date = '|'.join(
     _p.format_map(_pattern_date_elements) for _p in _pattern_date_list)
 
-_pattern_cmdstr = R'''(?:"(?:""|[^"])*"|'(?:''|[^'])*')'''
+
+def _sized_pattern_string(lower: int = 0, upper: int = 0):
+    n = _sized_suffix((lower - 2) // 2, upper - 2)
+    return FR'''(?:"(?:[^"\\\r\n]|\\[^\r\n]){n}"|'(?:[^'\\\r\n]|\\[^\r\n]){n}')'''
+
+
+def _sized_pattern_cmdstr(lower: int = 0, upper: int = 0):
+    n = _sized_suffix((lower - 2) // 2, upper - 2)
+    return FR'''(?:"(?:""|[^"]){n}"|'(?:''|[^']){n}')'''
+
+
+_pattern_cmdstr = _sized_pattern_cmdstr()
 _pattern_ps1str = R'''(?:(?:@"\s*?[\r\n].*?[\r\n]"@)|(?:@'\s*?[\r\n].*?[\r\n]'@)|(?:"(?:`.|""|[^"\n])*")|(?:'(?:''|[^'\n])*'))'''
 _pattern_vbastr = R'''"(?:""|[^"])*"'''
 _pattern_vbaint = R'(?:&[bB][01]+|&[hH][0-9a-fA-F]+|&[oO][0-7]*|[-+]?(?:[1-9][0-9]*|0))(?=\b|$)'
-_pattern_string = R'''(?:"(?:[^"\\\r\n]|\\[^\r\n])*"|'(?:[^'\\\r\n]|\\[^\r\n])*')'''
-_pattern_string_multiline = R'''(?:"(?:[^"\\]|\\.)*"|'(?:[^'\\]|\\.)*')'''
+_pattern_string = _sized_pattern_string()
+_pattern_string_multiline = R'''(?:"(?:[^"\\]|\\.)*"|'(?:[^'\\]|\\.)*'|`(?:[^`\\]|\\.)*`)'''
 _pattern_urlenc_coarse = R'''(?:%[0-9a-fA-F]{2}|[0-9a-zA-Z\-\._~\?!$&=:\/#\[\]@'\(\)\*\+,;])+'''
 _pattern_urlenc = R'''(?:%[0-9a-fA-F]{2}|[0-9a-zA-Z\-\._~\?!$&=])+'''
 _pattern_urlenc_narrow = R'''(?:%[0-9a-fA-F]{2})+'''
@@ -278,7 +355,7 @@ _suffix_combined_url = R'([/?#](?:[#/=:;$!?&.,\w\+\%\-\*\'~@()](?![a-zA-Z]{2,20}
 _pattern_serrated_url = F'{_prefix_serrated_url}({_pattern_serrated_hostname}){_suffix_combined_url}'
 _pattern_defanged_url = F'{_prefix_defanged_url}({_pattern_defanged_hostname}){_suffix_combined_url}'
 
-_pattern_email = R'(?:[a-zA-Z0-9_\.\+\-]{{1,256}}?)@(?:{})'.format(_pattern_serrated_domain)
+_pattern_email = fR'(?:[a-zA-Z0-9_\.\+\-]{{1,256}}?)@(?:{_pattern_serrated_domain})'
 _pattern_guid = R'(?:\b|\{)[0-9A-Fa-f]{8}(?:\-[0-9A-Fa-f]{4}){3}\-[0-9A-Fa-f]{12}(?:\}|\b)'
 
 _pattern_pathpart_nospace = R'[-\w+,.;@\]\[{}^`~]+'  # R'[^/\\:"<>|\s\x7E-\xFF\x00-\x1F\xAD]+'
@@ -286,7 +363,10 @@ _pattern_win_path_element = R'(?:{n} ){{0,4}}{n}'.format(n=_pattern_pathpart_nos
 _pattern_nix_path_element = R'(?:{n} ){{0,1}}{n}'.format(n=_pattern_pathpart_nospace)
 _pattern_win_env_variable = R'%[a-zA-Z][a-zA-Z0-9_\-\(\)]*%'
 
-_pattern_win_path_template = R'(?:{s}|{p}|)(?P<__pathsep__>[\\\/])(?:{p}(?P=__pathsep__))*{p}(?:(?P=__pathsep__)|\b)'
+_pattern_win_path_template_abs = R'(?:{s})(?P<__pathsep__>[\\\/])(?:{p}(?P=__pathsep__))*{p}(?:(?P=__pathsep__)|\b)'
+_pattern_win_path_template_rel = R'(?:{p}|)\\(?:{p}\\)*{p}(?:\\|\b)'
+_pattern_win_path_template = F'(?:{_pattern_win_path_template_abs}|{_pattern_win_path_template_rel})'
+
 _pattern_win_root = '|'.join([
     _pattern_win_env_variable,    # environment variable
     R'[A-Za-z]:',                 # drive letter with colon
@@ -386,6 +466,8 @@ class formats(PatternEnum):
     "A hex-encoded buffer using URL escape sequences"
     intarray = tokenize(_pattern_integer, sep=R'[;,]', bound='', unique_sep=True)
     "Sequences of integers, separated by commas or semicolons"
+    strarray = tokenize(_pattern_string, sep=R'[;,]', bound='', unique_sep=True)
+    "Sequences of strings, separated by commas or semicolons"
     numarray = tokenize(_pattern_number, sep=R'[;,]', bound='', unique_sep=True)
     "Sequences of numbers, separated by commas or semicolons"
     word = alphabet(R'\\w')
@@ -402,7 +484,7 @@ class formats(PatternEnum):
     "Base58 encoded strings"
     b62 = alphabet(R'(?:[0-9A-Za-z]')
     "Base62 encoded strings"
-    b64 = alphabet(R'(?:[0-9a-zA-Z\+/]{4})', postfix=R'(?:(?:[0-9a-zA-Z\+/]{2,3})={0,3})?')
+    b64 = alphabet(R'(?:[0-9a-zA-Z\+/]{4})', suffix=R'(?:(?:[0-9a-zA-Z\+/]{2,3})={0,3})?', suffix_max=6, token_size=4)
     "Base64 encoded strings"
     b85 = alphabet(R'[-!+*()#-&^-~0-9;-Z]')
     "Base85 encoded strings"
@@ -412,23 +494,21 @@ class formats(PatternEnum):
     "Z85 encoded strings"
     b92 = pattern(_pattern_b92)
     "Base92 encoded strings"
-    b64any = alphabet(R'(?:[-\w\+/]{4})', postfix=R'(?:(?:[-\w\+/]{2,3})={0,3})?')
-    "Both URL-safe and normal Base64 alphabets."
-    b64url = alphabet(R'[-\w]{4}', postfix=R'(?:[-\w]{2,3}={0,3})?')
+    b64url = alphabet(R'[-\w]{4}', suffix=R'(?:[-\w]{2,3}={0,3})?', suffix_max=6)
     "Base64 encoded strings using URL-safe alphabet"
-    hex = alphabet(R'[0-9a-fA-F]{2}')
+    hex = alphabet(R'[0-9a-fA-F]{2}', token_size=2)
     "Hexadecimal strings"
-    uppercase_hex = alphabet(R'[0-9A-F]{2}')
+    b16 = alphabet(R'[0-9A-F]{2}', token_size=2)
     "Uppercase hexadecimal strings"
-    spaced_hex = tokenize(R'[0-9a-fA-F]+', R'\s*', bound='')
+    b16s = tokenize(R'[0-9a-fA-F]+', R'\s*', bound='')
     "Hexadecimal strings"
-    spaced_b64 = alphabet(R'[-\s\w\+/]', postfix=R'(?:={0,3})?')
+    b64s = alphabet(R'[-\s\w\+/]', suffix=R'(?:={0,3})?', suffix_max=3)
     "Base64 encoded strings, separated by whitespace"
-    spaced_b85 = alphabet(R'[-!+*()#-&^-~0-9;-Z\s]')
+    b85s = alphabet(R'[-!+*()#-&^-~0-9;-Z\s]')
     "Base85 encoded string, separated by whitespace"
-    spaced_a85 = alphabet(R'[!-u\s]')
+    a85s = alphabet(R'[!-u\s]')
     "Ascii85 encoded string, separated by whitespace"
-    spaced_z85 = alphabet(R'[-\s0-9a-zA-Z.:+=^!/*?&<>()\[\]{}@%$#]')
+    z85s = alphabet(R'[-\s0-9a-zA-Z.:+=^!/*?&<>()\[\]{}@%$#]')
     "Z85 encoded string, separated by whitespace"
     utf8 = pattern(_pattern_utf8)
     "A sequence of bytes that can be decoded as UTF8."
@@ -446,8 +526,17 @@ class formats(PatternEnum):
     uuencode = pattern(_pattern_uuencode)
     "UUEncoded data"
 
+    # shortcuts
+    url = urlquote
+    int = integer
+    str = string
+    num = number
+    mls = multiline_string
+
     @classmethod
-    def from_dashname(cls, key):
+    def from_dashname(cls, key: str):
+        if key.startswith('[') and key.endswith(']'):
+            key = key[1:-1] + 'array'
         return getattr(cls, normalize_to_identifier(key))
 
 
@@ -492,11 +581,11 @@ class indicators(PatternEnum):
     "String representations of IPv4 addresses"
     ipv6 = pattern(_pattern_ipv6)
     "String representations of IPv6 addresses"
-    md5 = alphabet('[0-9A-Fa-f]', at_least=32, at_most=32)
+    md5 = alphabet('[0-9A-Fa-f]', lower=32, upper=32)
     "Hexadecimal strings of length 32"
-    sha1 = alphabet('[0-9A-Fa-f]', at_least=40, at_most=40)
+    sha1 = alphabet('[0-9A-Fa-f]', lower=40, upper=40)
     "Hexadecimal strings of length 40"
-    sha256 = alphabet('[0-9A-Fa-f]', at_least=64, at_most=64)
+    sha256 = alphabet('[0-9A-Fa-f]', lower=64, upper=64)
     "Hexadecimal strings of length 64"
     hostname = pattern(_pattern_serrated_hostname)
     "Any domain name or IPv4 address, optionally followed by a colon and a port number."
@@ -537,3 +626,34 @@ class defanged(PatternEnum):
     "A defanged `refinery.lib.patterns.indicators.hostname`."
     url = pattern(_pattern_defanged_url)
     "A defanged `refinery.lib.patterns.indicators.url`."
+
+
+def pattern_with_size_limits(p: pattern, lower: int | None, upper: int | None) -> pattern:
+    """
+    This attempts to construct a pattern from a given format that includes the given lower and
+    upper bounds on total match size. This is not always possible.
+    """
+    lower = max(0, lower or 0)
+    upper = max(0, upper or 0)
+    handlers = {
+        formats.int.value     : _sized_pattern_integer,
+        formats.cmdstr.value  : _sized_pattern_cmdstr,
+        formats.string.value  : _sized_pattern_string,
+    }
+    if isinstance(p, alphabet):
+        return alphabet(
+            p.repeat,
+            p.prefix,
+            p.suffix,
+            lower,
+            upper,
+            p.prefix_min,
+            p.prefix_max,
+            p.suffix_min,
+            p.suffix_max,
+            p.token_size,
+            flags=p.flags,
+        )
+    elif h := handlers.get(p):
+        return pattern(h(lower, upper), formats.int.value.flags)
+    return p
