@@ -3,50 +3,60 @@ Interfaces and classes to read structured data.
 """
 from __future__ import annotations
 
-import contextlib
+import abc
 import codecs
-import itertools
+import contextlib
 import enum
 import functools
+import inspect
 import io
+import itertools
 import re
 import struct
+import sys
 import weakref
 
-from uuid import UUID
-
 from typing import (
-    overload,
-    cast,
+    TYPE_CHECKING,
     Any,
-    Dict,
     Generic,
-    Sized,
     Iterable,
-    List,
-    Optional,
-    Tuple,
-    Type,
+    NamedTuple,
+    Sized,
     TypeVar,
     Union,
-    TYPE_CHECKING,
+    cast,
+    get_origin,
+    overload,
 )
+from uuid import UUID
+
+from refinery.lib.id import buffer_offset
 
 if TYPE_CHECKING:
     from collections.abc import Buffer
-    from refinery.lib.types import ByteStr
+    from typing import Generator, Protocol, Self
+
+    from refinery.lib.types import JSON, buf
 
     T = TypeVar('T', bound=Union[bytearray, bytes, memoryview])
     B = TypeVar('B', bound=Union[bytearray, bytes, memoryview], default=T)
     C = TypeVar('C', bound=Union[bytearray, bytes, memoryview])
     R = TypeVar('R', bound=io.IOBase)
 else:
+    Protocol = abc.ABC
     T = TypeVar('T')
     B = TypeVar('B')
     C = TypeVar('C')
     R = TypeVar('R')
 
 
+class ToJSON(Protocol):
+    @abc.abstractmethod
+    def __json__(self) -> JSON:
+        raise NotImplementedError
+
+
 UnpackType = Union[int, bool, float, bytes]
 
 
@@ -64,8 +74,21 @@ class EOF(EOFError):
     While reading from a `refinery.lib.structures.MemoryFile`, less bytes were available than
     requested. The exception contains the data from the incomplete read.
     """
-    def __init__(self, rest: ByteStr = B''):
-        super().__init__('Unexpected end of buffer.')
+    def __init__(self, size: int, rest: buf = B''):
+        super().__init__(F'Unexpected end of buffer; attempted to read {size} bytes, but got only {len(rest)}.')
+        self.rest = rest
+        self.size = size
+
+    def __bytes__(self):
+        return bytes(self.rest)
+
+
+class LimitExceeded(EOFError):
+    """
+    While writing to a `refinery.lib.structures.MemoryFile`, the buffer limit was exceeded.
+    """
+    def __init__(self, rest: buf = B''):
+        super().__init__(F'Unable to write {len(rest)} data to stream due to limit.')
         self.rest = rest
 
     def __bytes__(self):
@@ -77,7 +100,7 @@ class StreamDetour(Generic[R]):
     A stream detour is used as a context manager to temporarily read from a different location
     in the stream and then return to the original offset when the context ends.
     """
-    def __init__(self, stream: R, offset: Optional[int] = None, whence: int = io.SEEK_SET):
+    def __init__(self, stream: R, offset: int | None = None, whence: int = io.SEEK_SET):
         self.stream = stream
         self.offset = offset
         self.whence = whence
@@ -112,30 +135,37 @@ class MemoryFileMethods(Generic[T, B]):
 
     def __init__(
         self,
-        data: Optional[T] = None,
-        output: Optional[type[B]] = None,
-        fileno: Optional[int] = None,
-        size_limit: Optional[int] = None,
+        data: T | MemoryFileMethods[T, B] | None = None,
+        output: type[B] | None = None,
+        fileno: int | None = None,
+        size_limit: int | None = None,
     ) -> None:
-        if data is None:
-            if TYPE_CHECKING:
-                data = cast(T, bytearray())
-            else:
-                data = bytearray()
-        if output is None:
-            if TYPE_CHECKING:
-                output = cast(type[B], type(data))
-            else:
-                output = type(data)
-        if size_limit is not None and len(data) > size_limit:
-            raise ValueError('Initial data exceeds size limit')
-        self._output = output
-        self._cursor = 0
-        self._closed = False
-        self._fileno = fileno
-        self._quicksave = 0
-        self._size_limit = size_limit
-        self._data = data
+        if data is not self and isinstance(data, MemoryFileMethods):
+            self._output = output or data._output
+            self._cursor = data._cursor
+            self._closed = data._closed
+            self._fileno = fileno or data._fileno
+            self._size_limit = size_limit or data._size_limit
+            self._data = data._data
+        else:
+            if data is None:
+                if TYPE_CHECKING:
+                    data = cast(T, bytearray())
+                else:
+                    data = bytearray()
+            if output is None:
+                if TYPE_CHECKING:
+                    output = cast(type[B], type(data))
+                else:
+                    output = type(data)
+            if size_limit is not None and len(data) > size_limit:
+                raise ValueError('Initial data exceeds size limit')
+            self._output = output
+            self._cursor = 0
+            self._closed = False
+            self._fileno = fileno
+            self._size_limit = size_limit
+            self._data = data
 
     def close(self) -> None:
         self._closed = True
@@ -187,16 +217,16 @@ class MemoryFileMethods(Generic[T, B]):
     def remaining_bytes(self) -> int:
         return len(self._data) - self.tell()
 
-    def detour(self, offset: Optional[int] = None, whence: int = io.SEEK_SET):
+    def detour(self, offset: int | None = None, whence: int = io.SEEK_SET):
         return StreamDetour(cast(io.IOBase, self), offset, whence=whence)
 
-    def detour_absolute(self, offset: Optional[int] = None):
+    def detour_absolute(self, offset: int | None = None):
         return self.detour(offset, io.SEEK_SET)
 
-    def detour_relative(self, offset: Optional[int] = None):
+    def detour_relative(self, offset: int | None = None):
         return self.detour(offset, io.SEEK_CUR)
 
-    def detour_from_end(self, offset: Optional[int] = None):
+    def detour_from_end(self, offset: int | None = None):
         return self.detour(offset, io.SEEK_END)
 
     def writable(self) -> bool:
@@ -206,13 +236,13 @@ class MemoryFileMethods(Generic[T, B]):
             return not self._data.readonly
         return isinstance(self._data, bytearray)
 
-    def read_as(self, cast: Type[C], size: int = -1, peek: bool = False) -> C:
+    def read_as(self, cast: type[C], size: int = -1, peek: bool = False) -> C:
         out = self.read(size, peek)
         if not isinstance(out, cast):
             out = cast(out)
         return out
 
-    def read(self, size: Optional[int] = None, peek: bool = False) -> B:
+    def read(self, size: int | None = None, peek: bool = False) -> B:
         beginning = self._cursor
         if size is None or size < 0:
             end = len(self._data)
@@ -233,35 +263,28 @@ class MemoryFileMethods(Generic[T, B]):
             self._cursor = stop
         return match
 
-    def peek(self, size: Optional[int] = None) -> memoryview:
+    def peek(self, size: int | None = None) -> memoryview:
         cursor = self._cursor
         mv = memoryview(self._data)
         if size is None or size < 0:
             return mv[cursor:]
         return mv[cursor:cursor + size]
 
-    def read1(self, size: Optional[int] = None, peek: bool = False) -> B:
+    def read1(self, size: int | None = None, peek: bool = False) -> B:
         return self.read(size, peek)
 
-    def _find_linebreak(self, beginning: int, end: int) -> int:
-        if not isinstance(self._data, memoryview):
-            return self._data.find(B'\n', beginning, end)
-        for k in range(beginning, end):
-            if self._data[k] == 0xA: return k
-        return -1
-
-    def readline(self, size: Optional[int] = None) -> B:
+    def readline(self, size: int | None = None) -> B:
         beginning, end = self._cursor, len(self._data)
         if size is not None and size >= 0:
             end = beginning + size
-        p = self._find_linebreak(beginning, end)
+        p = buffer_offset(self._data, B'\n', beginning, end)
         self._cursor = end if p < 0 else p + 1
         result = self._data[beginning:self._cursor]
         if not isinstance(result, t := self._output):
             result = t(result)
         return result
 
-    def readlines_iter(self, hint: Optional[int] = None) -> Iterable[B]:
+    def readlines_iter(self, hint: int | None = None) -> Iterable[B]:
         if hint is None or hint < 0:
             yield from self
         else:
@@ -271,7 +294,7 @@ class MemoryFileMethods(Generic[T, B]):
                 total += len(line)
                 yield line
 
-    def readlines(self, hint: Optional[int] = None) -> list[bytes]:
+    def readlines(self, hint: int | None = None) -> list[bytes]:
         it = self.readlines_iter(hint)
         if issubclass(self._output, bytes):
             return list(it)
@@ -320,11 +343,11 @@ class MemoryFileMethods(Generic[T, B]):
         self._cursor = min(self._cursor, len(self._data))
         return self._cursor
 
-    def writelines(self, lines: Union[Iterable[Iterable[int]], Iterable[Buffer]]) -> None:
+    def writelines(self, lines: Iterable[Iterable[int]] | Iterable[Buffer]) -> None:
         for line in lines:
             self.write(line)
 
-    def truncate(self, size: Optional[int] = None) -> int:
+    def truncate(self, size: int | None = None) -> int:
         if not isinstance(self._data, bytearray):
             raise TypeError
         if size is not None:
@@ -343,7 +366,7 @@ class MemoryFileMethods(Generic[T, B]):
         cc = self._cursor
         nc = cc + 1
         if limit and nc > limit:
-            raise EOF(bytes((byte,)))
+            raise LimitExceeded(bytes((byte,)))
         try:
             if cc < len(self._data):
                 self._data[cc] = byte
@@ -354,7 +377,7 @@ class MemoryFileMethods(Generic[T, B]):
         else:
             self._cursor = nc
 
-    def write(self, _data: Union[Buffer, Iterable[int]]) -> int:
+    def write(self, _data: Buffer | Iterable[int]) -> int:
         out = self._data
         end = len(out)
 
@@ -404,10 +427,10 @@ class MemoryFileMethods(Generic[T, B]):
                 else:
                     rest = bytearray((b,))
                     rest[1:] = it
-                    raise EOF(rest)
+                    raise LimitExceeded(rest)
         else:
             if limit and size + beginning > limit:
-                raise EOF(bytes(data))
+                raise LimitExceeded(bytes(data))
             self._cursor += size
             try:
                 out[beginning:self._cursor] = data
@@ -455,10 +478,13 @@ class StructReader(MemoryFile[T, T]):
     class Unaligned(RuntimeError):
         pass
 
-    def __init__(self, data: T, bigendian: bool = False):
+    def __init__(self, data: T | StructReader[T], bigendian: bool | None = None):
         super().__init__(data)
-        self._bbits = 0
-        self._nbits = 0
+        if bigendian is None:
+            if isinstance(data, StructReader):
+                bigendian = data.bigendian
+            else:
+                bigendian = False
         self.bigendian = bigendian
 
     def __enter__(self) -> StructReader:
@@ -481,12 +507,7 @@ class StructReader(MemoryFile[T, T]):
     def byteorder_name(self):
         return 'big' if self.bigendian else 'little'
 
-    def seek(self, offset, whence=io.SEEK_SET) -> int:
-        self._bbits = 0
-        self._nbits = 0
-        return super().seek(offset, whence)
-
-    def read_exactly(self, size: Optional[int] = None, peek: bool = False) -> B:
+    def read_exactly(self, size: int | None = None, peek: bool = False) -> T:
         """
         Read bytes from the underlying stream. Raises a `RuntimeError` when the stream is not currently
         byte-aligned, i.e. when `refinery.lib.structures.StructReader.byte_aligned` is `False`. Raises
@@ -495,140 +516,67 @@ class StructReader(MemoryFile[T, T]):
         Use `refinery.lib.structures.StructReader.read_bytes` to read bytes from the stream when it is
         not byte-aligned.
         """
-        if not self.byte_aligned:
-            raise StructReader.Unaligned('buffer is not byte-aligned')
-        data = self.read1(size, peek)
+        data = self.read(size, peek)
         if size and len(data) < size:
-            raise EOF(data)
+            raise EOF(size, data)
         return data
 
-    @property
-    def byte_aligned(self) -> bool:
+    def read_bit_field(self, *sizes: int, peek: bool = False):
         """
-        This property is `True` if and only if there are currently no bits still waiting in the internal
-        bit buffer.
+        Read multiple integers that form a bit field. This method can be used to read bit fields without
+        having to use a `refinery.lib.structures.StructReaderBits` when the bit count sums to a multiple
+        of 8.
         """
-        return not self._nbits
+        data = self.read_integer(sum(sizes))
+        if self.bigendian:
+            sizes = sizes[::-1]
+        fields = []
+        for size in sizes:
+            fields.append(data & ~(-1 << size))
+            data >>= size
+        if self.bigendian:
+            fields.reverse()
+        return fields
 
-    def byte_align(self, blocksize: int = 1) -> Tuple[int, int]:
+    def read_integer(
+        self,
+        size: int,
+        peek: bool = False,
+        signed: bool = False
+    ) -> int:
         """
-        This method clears the internal bit buffer and moves the cursor to the next byte. It returns a
-        tuple containing the size and contents of the bit buffer.
+        Read an integer of the given size (in bytes) from the stream.
         """
-        nbits = self._nbits
-        bbits = self._bbits
-        self._nbits = 0
-        self._bbits = 0
-        mod = self._cursor % blocksize
-        if mod:
-            self.seekrel(blocksize - mod)
-        return nbits, bbits
+        nbytes, rest = divmod(size, 8)
+        if rest > 0:
+            raise ValueError(
+                F'A {self.__class__.__name__} cannot read {size} bit{"s" * (size > 1)}, only multiples of 8 are possible.')
+        data = self.read(nbytes, peek)
+        if len(data) < nbytes:
+            raise EOF(nbytes, data)
+        return int.from_bytes(data, self.byteorder_name, signed=signed)
 
-    @property
-    def remaining_bits(self) -> int:
-        return 8 * self.remaining_bytes + self._nbits
-
-    def read_integer(self, length: Optional[int] = None, peek: bool = False, bigendian: Optional[bool] = None) -> int:
+    def byte_align(self, blocksize: int = 1):
         """
-        Read `length` many bits from the underlying stream as an integer.
+        Align the cursor at the given block size boundary.
         """
-        if length is None:
-            length = self.remaining_bits
-        if bigendian is None:
-            bigendian = self.bigendian
-        if length < self._nbits:
-            new_count = self._nbits - length
-            if bigendian:
-                result = self._bbits >> new_count
-                if not peek:
-                    self._bbits ^= result << new_count
-            else:
-                result = self._bbits & 2 ** length - 1
-                if not peek:
-                    self._bbits >>= length
-            if not peek:
-                self._nbits = new_count
-            return result
-
-        nbits, bbits = self._nbits, self._bbits
-        number_of_missing_bits = length - nbits
-        bytecount, rest = divmod(number_of_missing_bits, 8)
-        if rest:
-            bytecount += 1
-            rest = 8 - rest
-        bb = self.read1(bytecount, True)
-        if len(bb) != bytecount:
-            raise EOFError
-        if not peek:
-            self.seekrel(bytecount)
-        if bytecount == 1:
-            result, = bb
-        else:
-            result = int.from_bytes(bb, self.byteorder_name)
-        if not nbits and not rest:
-            return result
-        if bigendian:
-            rbmask   = 2 ** rest - 1       # noqa
-            excess   = result & rbmask     # noqa
-            result >>= rest                # noqa
-            result  ^= bbits << number_of_missing_bits   # noqa
-        else:
-            excess   = result >> number_of_missing_bits  # noqa
-            result  ^= excess << number_of_missing_bits  # noqa
-            result <<= nbits               # noqa
-            result  |= bbits               # noqa
-        assert excess.bit_length() <= rest
-        if not peek:
-            self._nbits = rest
-            self._bbits = excess
-        return result
+        if mod := -self._cursor % blocksize:
+            self.seekrel(mod)
 
     def read_bytes(self, size: int, peek: bool = False) -> bytes:
         """
         The method reads `size` many bytes from the underlying stream starting at the current bit.
         """
-        if self.byte_aligned:
-            data = self.read_exactly(size, peek)
-            if not isinstance(data, bytes):
-                data = bytes(data)
-            return data
-        else:
-            return self.read_integer(size * 8, peek).to_bytes(size, self.byteorder_name)
-
-    def read_bit(self) -> int:
-        """
-        This function is a shortcut for calling `refinery.lib.structures.StructReader.read_integer` with
-        an argument of `1`, i.e. this reads the next bit from the stream. The bits of any byte in the stream
-        are read from least significant to most significant.
-        """
-        return self.read_integer(1)
-
-    def read_bits(self, nbits: int, bigendian: Optional[bool] = None) -> Iterable[int]:
-        """
-        This method returns the bits of `refinery.lib.structures.StructReader.read_integer` one by one.
-        """
-        if bigendian is None:
-            bigendian = self.bigendian
-        chunk = self.read_integer(nbits, bigendian=bigendian)
-        it = range(nbits - 1, -1, -1) if bigendian else range(nbits)
-        for k in it:
-            yield chunk >> k & 1
-
-    def read_flags(self, nbits: int, reverse=False) -> Iterable[bool]:
-        """
-        Identical to `refinery.lib.structures.StructReader.read_bits` with every bit value cast to a boolean.
-        """
-        bits = list(self.read_bits(nbits))
-        if reverse:
-            bits.reverse()
-        for bit in bits:
-            yield bool(bit)
+        data = self.read_exactly(size, peek)
+        if not isinstance(data, bytes):
+            data = bytes(data)
+        return data
 
     def read_one_struct(self, spec: str, peek=False) -> UnpackType:
         item, = self.read_struct(spec, peek=peek)
         return item
 
-    def read_struct(self, spec: str, peek=False) -> List[UnpackType]:
+    def read_struct(self, spec: str, peek=False) -> list[UnpackType]:
         """
         Read structured data from the stream in any format supported by the `struct` module. The `format`
         argument can be used to override the current byte ordering. If the `unwrap` parameter is `True`, a
@@ -653,7 +601,7 @@ class StructReader(MemoryFile[T, T]):
                     if part == 'a':
                         data.append(self.read_c_string())
                     elif part == 'g':
-                        data.append(self.read_guid())
+                        data.append(str(self.read_guid()))
                     elif part == 'u':
                         data.append(self.read_w_string())
                     elif part == 'w':
@@ -668,69 +616,90 @@ class StructReader(MemoryFile[T, T]):
             self.seekset(current_cursor)
         return data
 
-    def read_nibble(self, peek: bool = False) -> int:
-        """
-        Calls `refinery.lib.structures.StructReader.read_integer` with an argument of `4`.
-        """
-        return self.read_integer(4, peek)
+    def read_bool_byte(self, strict=False):
+        value = self.u8()
+        if strict and value not in (0, 1):
+            raise ValueError(F'Invalid boolean byte value {value:#02x}.')
+        return bool(value)
 
-    def u8(self, peek: bool = False) -> int: return self.read_integer(8, peek)
-    def i8(self, peek: bool = False) -> int: return signed(self.read_integer(8, peek), 8)
+    def read_regex(
+        self,
+        pattern: bytes | bytearray | memoryview | re.Pattern[bytes],
+        dotall: bool = True
+    ):
+        if isinstance(pattern, (bytes, bytearray, memoryview)):
+            flags = re.DOTALL if dotall else re.NOFLAG
+            pattern = re.compile(bytes(pattern), flags=flags)
+        elif dotall and not re.DOTALL | pattern.flags:
+            pattern = re.compile(pattern.pattern, flags=re.DOTALL)
+        data = self._data
+        if isinstance(data, memoryview) and not data.contiguous:
+            raise NotImplementedError('Cannot perform regex-based read on non-contiguous buffer.')
+        if result := pattern.match(data, self._cursor):
+            self._cursor = result.end()
+            return result
 
-    def u16(self, peek: bool = False) -> int: return self.read_integer(16, peek)
-    def u32(self, peek: bool = False) -> int: return self.read_integer(32, peek)
-    def u64(self, peek: bool = False) -> int: return self.read_integer(64, peek)
-    def i16(self, peek: bool = False) -> int: return signed(self.read_integer(16, peek), 16)
-    def i32(self, peek: bool = False) -> int: return signed(self.read_integer(32, peek), 32)
-    def i64(self, peek: bool = False) -> int: return signed(self.read_integer(64, peek), 64)
-
-    def f32(self, peek: bool = False) -> float: return cast(float, self.read_one_struct('f', peek=peek))
-    def f64(self, peek: bool = False) -> float: return cast(float, self.read_one_struct('d', peek=peek))
-
-    def u8fast(self):
+    def read_byte(self, peek: bool = False) -> int:
         try:
             b = self._data[self._cursor]
         except IndexError:
-            raise EOFError
-        else:
+            raise EOF(1)
+        if not peek:
             self._cursor += 1
-            return b
+        return b
 
-    def read_byte(self, peek: bool = False) -> int: return self.read_integer(8, peek)
-    def read_char(self, peek: bool = False) -> int: return signed(self.read_integer(8, peek), 8)
+    def read_char(self, peek: bool = False) -> str:
+        try:
+            b = self._data[self._cursor]
+        except IndexError:
+            raise EOF(1)
+        if not peek:
+            self._cursor += 1
+        return chr(b)
 
-    def read_terminated_array(self, terminator: bytes, alignment: int = 1) -> bytearray:
+    u8fast = u8 = read_byte
+
+    def i8(self, peek: bool = False) -> int:
+        return signed(self.u8(peek), 8)
+
+    def u16(self, peek: bool = False) -> int:
+        return self.read_integer(16, peek, signed=False)
+
+    def u32(self, peek: bool = False) -> int:
+        return self.read_integer(32, peek, signed=False)
+
+    def u64(self, peek: bool = False) -> int:
+        return self.read_integer(64, peek, signed=False)
+
+    def i16(self, peek: bool = False) -> int:
+        return self.read_integer(16, peek, signed=True)
+
+    def i32(self, peek: bool = False) -> int:
+        return self.read_integer(32, peek, signed=True)
+
+    def i64(self, peek: bool = False) -> int:
+        return self.read_integer(64, peek, signed=True)
+
+    def f32(self, peek: bool = False) -> float:
+        return cast(float, self.read_one_struct('f', peek=peek))
+
+    def f64(self, peek: bool = False) -> float:
+        return cast(float, self.read_one_struct('d', peek=peek))
+
+    def read_terminated_array(self, terminator: bytes, alignment: int = 1) -> T:
         buf = self.getvalue()
         pos = self.tell()
-
-        if isinstance(buf, memoryview):
-            def find(whence: int):
-                n = len(terminator)
-                for k in range(whence, len(buf)):
-                    if buf[k:k + n] == terminator:
-                        return k
-                return -1
-        else:
-            def find(whence: int):
-                return buf.find(terminator, whence)
-        try:
-            end = pos - 1
-            while True:
-                end = find(end + 1)
-                if end < 0 or not (end - pos) % alignment:
-                    break
-        except AttributeError:
-            result = bytearray()
-            while not self.eof:
-                result.extend(self.read_bytes(alignment))
-                if result.endswith(terminator):
-                    return result[:-len(terminator)]
-            self.seek(pos)
-            raise EOF
-        else:
-            data = self.read_exactly(end - pos)
-            self.seekrel(len(terminator))
-            return bytearray(data)
+        end = pos - 1
+        n = len(terminator)
+        while True:
+            end = buffer_offset(buf, terminator, end + 1)
+            if end < 0 or not (end - pos) % alignment:
+                break
+        if end >= pos:
+            result = self.read_exactly(end - pos)
+            self.skip(n)
+            return result
+        raise EOF(len(buf) - pos + n)
 
     def read_guid(self) -> UUID:
         return UUID(bytes_le=self.read_bytes(16))
@@ -739,28 +708,28 @@ class StructReader(MemoryFile[T, T]):
         return UUID(bytes=self.read_bytes(16))
 
     @overload
-    def read_c_string(self) -> bytearray:
+    def read_c_string(self) -> T:
         ...
 
     @overload
     def read_c_string(self, encoding: str) -> str:
         ...
 
-    def read_c_string(self, encoding=None) -> Union[str, bytearray]:
+    def read_c_string(self, encoding=None) -> str | T:
         data = self.read_terminated_array(B'\0')
         if encoding is not None:
             data = codecs.decode(data, encoding)
         return data
 
     @overload
-    def read_w_string(self) -> bytearray:
+    def read_w_string(self) -> T:
         ...
 
     @overload
     def read_w_string(self, encoding: str) -> str:
         ...
 
-    def read_w_string(self, encoding=None) -> Union[str, bytearray]:
+    def read_w_string(self, encoding=None) -> str | T:
         data = self.read_terminated_array(B'\0\0', 2)
         if encoding is not None:
             data = codecs.decode(data, encoding)
@@ -792,7 +761,7 @@ class StructReader(MemoryFile[T, T]):
     def read_length_prefixed(self, prefix_size: int, *, block_size: int = 1) -> T:
         ...
 
-    def read_length_prefixed(self, prefix_size: int = 32, encoding: Optional[str] = None, block_size: int = 1) -> Union[T, str]:
+    def read_length_prefixed(self, prefix_size: int = 32, encoding: str | None = None, block_size: int = 1) -> T | str:
         prefix = self.read_integer(prefix_size) * block_size
         data = self.read(prefix)
         if encoding is not None:
@@ -805,7 +774,7 @@ class StructReader(MemoryFile[T, T]):
         if bigendian is None:
             bigendian = self.bigendian
         while True:
-            b = self.u8fast()
+            b = self.u8()
             if bigendian:
                 value <<= 7
                 value |= (b & 0x7F)
@@ -816,26 +785,194 @@ class StructReader(MemoryFile[T, T]):
             if (shift := shift + 7) > max_bits > 0:
                 raise OverflowError('Maximum bits were exceeded by encoded integer.')
 
+    def read_bits(self, nbits: int) -> Iterable[int]:
+        """
+        This method returns the bits of `refinery.lib.structures.StructReader.read_integer` one by one.
+        """
+        chunk = self.read_integer(nbits)
+        it = range(nbits - 1, -1, -1) if self.bigendian else range(nbits)
+        for k in it:
+            yield chunk >> k & 1
 
-class StructMeta(type):
+    def read_flags(self, nbits: int, reverse=False) -> Iterable[bool]:
+        """
+        Identical to `refinery.lib.structures.StructReader.read_bits` with every bit value cast to a boolean.
+        """
+        bits = self.read_bits(nbits)
+        if reverse:
+            bits = list(bits)
+            bits.reverse()
+        for bit in bits:
+            yield bool(bit)
+
+
+class StructReaderBits(StructReader[T]):
+
+    def __init__(self, data: T | StructReader[T], bigendian: bool | None = None):
+        super().__init__(data, bigendian)
+        if isinstance(data, StructReaderBits):
+            self._bbits = data._bbits
+            self._nbits = data._nbits
+        else:
+            self._bbits = 0
+            self._nbits = 0
+
+    @property
+    def remaining_bits(self) -> int:
+        return 8 * self.remaining_bytes + self._nbits
+
+    @property
+    def byte_aligned(self) -> bool:
+        """
+        This property is `True` if and only if there are currently no bits still waiting in the internal
+        bit buffer.
+        """
+        return not self._nbits
+
+    def read_bytes(self, size: int, peek: bool = False) -> bytes:
+        """
+        The method reads `size` many bytes from the underlying stream starting at the current bit.
+        """
+        if not self.byte_aligned:
+            return self.read_integer(size * 8, peek).to_bytes(size, self.byteorder_name)
+        return super().read_bytes(size, peek)
+
+    def byte_align(self, blocksize: int = 1):
+        """
+        This method clears the internal bit buffer and moves the cursor to the next byte. It returns a
+        tuple containing the size and contents of the bit buffer.
+        """
+        self._nbits = 0
+        self._bbits = 0
+        super().byte_align(blocksize)
+
+    def read_exactly(self, size: int | None = None, peek: bool = False) -> T:
+        if not self.byte_aligned:
+            raise StructReader.Unaligned('buffer is not byte-aligned')
+        return super().read_exactly(size, peek)
+
+    def seek(self, offset, whence=io.SEEK_SET) -> int:
+        self._bbits = 0
+        self._nbits = 0
+        return super().seek(offset, whence)
+
+    def read_integer(
+        self,
+        size: int | None = None,
+        peek: bool = False,
+        signed: bool = False,
+    ) -> int:
+        """
+        Read `size` many bits from the underlying stream as an integer.
+        """
+        if size is None:
+            size = self.remaining_bits
+        if size < self._nbits:
+            new_count = self._nbits - size
+            if self.bigendian:
+                result = self._bbits >> new_count
+                if not peek:
+                    self._bbits ^= result << new_count
+            else:
+                result = self._bbits & 2 ** size - 1
+                if not peek:
+                    self._bbits >>= size
+            if not peek:
+                self._nbits = new_count
+        else:
+            nbits, bbits = self._nbits, self._bbits
+            number_of_missing_bits = size - nbits
+            bytecount, rest = divmod(number_of_missing_bits, 8)
+            if rest:
+                bytecount += 1
+                rest = 8 - rest
+            bb = self.read(bytecount, True)
+            if len(bb) != bytecount:
+                raise EOF(bytecount, bb)
+            if not peek:
+                self.seekrel(bytecount)
+            if bytecount == 1:
+                result, = bb
+            else:
+                result = int.from_bytes(bb, self.byteorder_name)
+            if nbits or rest:
+                if self.bigendian:
+                    rbmask   = 2 ** rest - 1       # noqa
+                    excess   = result & rbmask     # noqa
+                    result >>= rest                # noqa
+                    result  ^= bbits << number_of_missing_bits   # noqa
+                else:
+                    excess   = result >> number_of_missing_bits  # noqa
+                    result  ^= excess << number_of_missing_bits  # noqa
+                    result <<= nbits               # noqa
+                    result  |= bbits               # noqa
+                assert excess.bit_length() <= rest
+                if not peek:
+                    self._nbits = rest
+                    self._bbits = excess
+        if signed and (result & (msb := 1 << (size - 1))):
+            result &= msb - 1
+            result -= msb
+        return result
+
+    def read_bit(self) -> int:
+        """
+        This function is a shortcut for calling `refinery.lib.structures.StructReader.read_integer` with
+        an argument of `1`, i.e. this reads the next bit from the stream. The bits of any byte in the stream
+        are read from least significant to most significant.
+        """
+        return self.read_integer(1)
+
+    def read_nibble(self, peek: bool = False) -> int:
+        """
+        Calls `refinery.lib.structures.StructReader.read_integer` with an argument of `4`.
+        """
+        return self.read_integer(4, peek)
+
+    def read_byte(self, peek: bool = False) -> int:
+        return self.read_integer(8, peek)
+
+    def read_char(self, peek: bool = False) -> str:
+        return chr(self.read_integer(8, peek))
+
+
+class StructMeta(abc.ABCMeta):
     """
     A metaclass to facilitate the behavior outlined for `refinery.lib.structures.Struct`.
     """
-    def __new__(mcls, name, bases, nmspc, parser=StructReader):
-        return type.__new__(mcls, name, bases, nmspc)
+    def __new__(mcls, name, bases, namespace: dict, interface: type[StructReader] | None = None):
+        if interface is None:
+            if init := namespace.get('__init__'):
+                args = iter(inspect.signature(init).parameters.values())
+                next(args)
+                interface = next(args).annotation
+                if isinstance(interface, str):
+                    try:
+                        module = sys.modules[namespace['__module__']]
+                        interface = eval(interface, module.__dict__)
+                    except Exception:
+                        interface = None
+                if not isinstance(interface, type):
+                    interface = get_origin(interface)
+                if not isinstance(interface, type) or not issubclass(interface, StructReader):
+                    raise RuntimeError
+            else:
+                interface = StructReader
 
-    def __init__(cls, name, bases, nmspc, parser=StructReader):
-        super(StructMeta, cls).__init__(name, bases, nmspc)
+        def parse(cls, reader: T | StructReader[T], *args, **kwargs):
+            if not isinstance(reader, interface):
+                reader = interface(reader)
+            return cls(reader, *args, **kwargs)
+
+        namespace.update(Parse=classmethod(parse))
+        return super().__new__(mcls, name, bases, namespace)
+
+    def __init__(cls, name, bases, nmspc, **_):
+        super().__init__(name, bases, nmspc)
         original__init__ = cls.__init__
 
         @functools.wraps(original__init__)
-        def wrapped__init__(self: Struct, reader, *args, **kwargs):
-            if not isinstance(reader, parser):
-                if issubclass(parser, reader.__class__):
-                    raise ValueError(
-                        F'A reader of type {reader.__class__.__name__} was passed to {cls.__name__}, '
-                        F'but a {parser.__name__} is required.')
-                reader = parser(reader)
+        def wrapped__init__(self: Struct, reader: StructReader, *args, **kwargs):
             start = reader.tell()
             view = reader.getbuffer()
             original__init__(self, reader, *args, **kwargs)
@@ -845,7 +982,7 @@ class StructMeta(type):
         setattr(cls, '__init__', wrapped__init__)
 
 
-class Struct(metaclass=StructMeta):
+class Struct(Generic[T], metaclass=StructMeta):
     """
     A class to parse structured data. A `refinery.lib.structures.Struct` class can be instantiated
     as follows:
@@ -857,7 +994,11 @@ class Struct(metaclass=StructMeta):
     as `reader`. Otherwise, the argument will be wrapped in a `refinery.lib.structures.StructReader`.
     Additional arguments to the struct are passed through.
     """
-    _data: Union[memoryview, bytearray]
+    _data: memoryview | bytearray
+
+    @classmethod
+    def Parse(cls, reader: T | StructReader[T], *args, **kwargs) -> Self:
+        ...
 
     def __len__(self):
         return len(self._data)
@@ -870,7 +1011,7 @@ class Struct(metaclass=StructMeta):
             self._data = bytearray(self._data)
         return self._data
 
-    def __init__(self, reader: StructReader, *args, **kwargs):
+    def __init__(self, reader: StructReader[T], *args, **kwargs):
         pass
 
 
@@ -882,8 +1023,8 @@ class PerInstanceAttribute(Generic[AttrType]):
         return value
 
     def __init__(self):
-        self.__set: Dict[int, Any] = {}
-        self.__get: Dict[int, AttrType] = {}
+        self.__set: dict[int, Any] = {}
+        self.__get: dict[int, AttrType] = {}
 
     def __set__(self, parent: Any, value: Any) -> None:
         pid = id(parent)
@@ -903,3 +1044,81 @@ class PerInstanceAttribute(Generic[AttrType]):
                 raise AttributeError from K
             self.__get[pid] = self.resolve(parent, seed)
         return self.__get[pid]
+
+
+def struct_to_json(v: dict | list | enum.IntFlag | enum.IntEnum | Struct | ToJSON | NamedTuple | None, codec: str | None = None) -> JSON:
+    """
+    Attempt to convert a `refinery.lib.structures.Struct` to a JSON representation.
+    """
+    if v is None:
+        return v
+    if isinstance(v, Struct):
+        v = {k: v for k, v in v.__dict__.items() if not k.startswith('_')}
+    if isinstance(v, list):
+        return [struct_to_json(x, codec) for x in v]
+    if isinstance(v, tuple):
+        v = v._asdict()
+    if isinstance(v, dict):
+        return {x: struct_to_json(y, codec) for x, y in v.items()}
+    if isinstance(v, enum.IntFlag):
+        return [option.name for option in v.__class__ if v & option == option]
+    if isinstance(v, enum.IntEnum):
+        return v.name
+    if codec is not None:
+        if isinstance(v, (memoryview, bytes, bytearray)):
+            return codecs.decode(v, codec)
+    try:
+        return v.__json__()
+    except AttributeError:
+        return cast('JSON', v)
+
+
+class FlagAccessMixin:
+    """
+    This class can be mixed into an `enum.IntFlag` for some quality of life improvements. Firstly,
+    you can now access flags as follows:
+
+        class Flags(FlagAccessMixin, enum.IntFlag):
+            IsBinary = 1
+            IsCompressed = 2
+
+        flag = Flags(3)
+
+        if flag.IsCompressed:
+            decompress()
+
+    Furthermore, flag values can be enumerated:
+
+        >>> list(flag)
+        [IsBinary, IsCompressed]
+        >>> flag
+        IsBinary|IsCompressed
+
+    And finally, as visible from the above output, flag values are represented by their name by
+    default.
+    """
+    def __getattribute__(self, name: str):
+        if not isinstance(self, enum.IntFlag):
+            raise RuntimeError
+        if not name.startswith('_'):
+            try:
+                flag = self.__class__[name]
+            except KeyError:
+                pass
+            else:
+                return flag in self
+        return super().__getattribute__(name)
+
+    def __iter__(self) -> Generator[Self]:
+        if not isinstance(self, enum.IntFlag):
+            raise RuntimeError
+        for flag in self.__class__:
+            if flag in self:
+                yield flag
+
+    def __repr__(self):
+        if not isinstance(self, enum.IntFlag):
+            raise RuntimeError
+        if name := self.name:
+            return name
+        return super().__repr__()
