@@ -5,24 +5,44 @@ from __future__ import annotations
 
 import datetime
 import inspect
+import io
 import itertools
 import logging
 import os
-import sys
-import io
-import warnings
 import re
+import sys
+import warnings
 
-from typing import Callable, Generator, Iterable, Optional, Tuple, TypeVar, Any, Type
+from enum import Enum, IntFlag
 from math import log
-from enum import IntFlag, Enum
+from typing import Any, Callable, Generator, Iterable, TypeVar
 
-from refinery.lib.types import ByteStr
+from refinery.lib.types import INF, buf
 
 _T = TypeVar('_T')
 
 
-def lookahead(iterator: Iterable[_T]) -> Generator[Tuple[bool, _T], None, None]:
+try:
+    import ctypes
+except ImportError:
+    def _meminfo_d(v: memoryview) -> slice | None:
+        return None
+    meminfo = _meminfo_d
+else:
+    def _meminfo_c(v: memoryview):
+        if not (n := len(v)):
+            return None
+        if v.readonly or not v.contiguous:
+            return None
+        base = memoryview(v.obj)
+        offset, base_addr = (
+            ctypes.addressof(ctypes.c_char.from_buffer(t)) for t in (v, base))
+        start = offset - base_addr
+        return slice(start, min(start + n, len(base)), 1)
+    meminfo = _meminfo_c
+
+
+def lookahead(iterator: Iterable[_T]) -> Generator[tuple[bool, _T]]:
     """
     Implements a new iterator from a given one which returns elements `(last, item)` where each
     `item` is taken from the original iterator and `last` is a boolean indicating whether this is
@@ -70,8 +90,8 @@ def terminalfit(text: str, delta: int = 0, width: int = 0, parsep: str = '\n\n',
     """
     Reformats text to fit the given width while not mangling bullet point lists.
     """
-    import textwrap
     import re
+    import textwrap
 
     width = width or get_terminal_size()
     width = width - delta
@@ -85,7 +105,7 @@ def terminalfit(text: str, delta: int = 0, width: int = 0, parsep: str = '\n\n',
     def bulletpoint(line):
         wrapped = textwrap.wrap(line, width - 2, **kw)
         indent = '  ' if isul(line) else '   '
-        wrapped[1:] = ['{}{}'.format(indent, line) for line in wrapped[1:]]
+        wrapped[1:] = [f'{indent}{line}' for line in wrapped[1:]]
         return '\n'.join(wrapped)
 
     def fitted(paragraphs):
@@ -124,7 +144,7 @@ def documentation(unit):
     return docs.replace('`', '')
 
 
-def begin(iterable: Iterable[_T]) -> Optional[Tuple[_T, Iterable[_T]]]:
+def begin(iterable: Iterable[_T]) -> tuple[_T, Iterable[_T]] | None:
     """
     Iterates the first element of an iterator and returns None if this fails. Otherwise, it returns
     both the first element and a new iterable which will return the same elements as the input.
@@ -141,7 +161,7 @@ def begin(iterable: Iterable[_T]) -> Optional[Tuple[_T, Iterable[_T]]]:
         return head, _fused()
 
 
-def skipfirst(iterable: Iterable[_T]) -> Generator[_T, None, None]:
+def skipfirst(iterable: Iterable[_T]) -> Generator[_T]:
     """
     Returns an interable where the first element of the input iterable was skipped.
     """
@@ -189,7 +209,7 @@ def autoinvoke(method: Callable[..., _T], keywords: dict) -> _T:
     return method(*posargs, *varargs, **kwdargs)
 
 
-def entropy_fallback(data: ByteStr) -> float:
+def entropy_fallback(data: buf) -> float:
     """
     This method is called by `refinery.lib.tools.entropy` when the `numpy` module is not available.
     It computes the shannon entropy of the input byte string and is written in pure Python.
@@ -202,7 +222,7 @@ def entropy_fallback(data: ByteStr) -> float:
     return 0.0 + -sum(p * log(p, 2) for p in S if p) / 8.0
 
 
-def entropy(data: ByteStr) -> float:
+def entropy(data: buf) -> float:
     """
     Computes the entropy of `data` over the alphabet of all bytes.
     """
@@ -218,7 +238,7 @@ def entropy(data: ByteStr) -> float:
     return 0.0 - (numpy.log2(prob) * prob).sum() / 8.0
 
 
-def index_of_coincidence(data: ByteStr) -> float:
+def index_of_coincidence(data: buf) -> float:
     """
     Computes the index of coincidence of `data` over the alphabet of all bytes.
     """
@@ -260,7 +280,7 @@ def isbuffer(obj) -> bool:
         return False
 
 
-def asbuffer(obj) -> Optional[memoryview]:
+def asbuffer(obj) -> memoryview | None:
     """
     Attempts to acquire a memoryview of the given object. This works for bytes and bytearrays, or
     memoryview objects themselves. The return value is `None` for objects that do not support the
@@ -273,11 +293,11 @@ def asbuffer(obj) -> Optional[memoryview]:
 
 
 def splitchunks(
-    data: ByteStr,
+    data: buf,
     size: int,
-    step: Optional[int] = None,
+    step: int | None = None,
     truncate: bool = False
-) -> Iterable[ByteStr]:
+) -> Iterable[buf]:
     """
     Split `data` into chunks of size `size`. The cursor advances by `step` bytes after extracting a
     block, the default value for `step` is equal to `size`. The boolean parameter `truncate`
@@ -299,7 +319,7 @@ def splitchunks(
         yield chunk
 
 
-def make_buffer_mutable(data: ByteStr):
+def make_buffer_mutable(data: buf):
     """
     Returns a mutable version of the input data. Already mutable inputs are returned
     as themselves, i.e. no copy operation occurs in these cases.
@@ -311,16 +331,18 @@ def make_buffer_mutable(data: ByteStr):
     return bytearray(data)
 
 
-def infinitize(it):
-    if not isinstance(it, (
+def infinitize(it: _T | Iterable[_T]) -> Iterable[_T]:
+    if isinstance(it, (
         itertools.cycle,
         itertools.repeat,
-        itertools.count
+        itertools.count,
     )):
-        if not hasattr(it, '__iter__') and not hasattr(it, '__next__'):
-            it = (it, )
-        return itertools.cycle(it)
-    return it
+        return it
+    try:
+        it = iter(it)           # type:ignore
+    except TypeError:
+        it = (it,)              # type:ignore
+    return itertools.cycle(it)  # type:ignore
 
 
 class NoLogging:
@@ -425,7 +447,7 @@ class NoLoggingProxy:
             else:
                 proxy_class = cls
             cls.__proxy_cache__[wrap_type] = proxy_class
-        return super().__new__(proxy_class)
+        return super().__new__(proxy_class) # type:ignore
 
     def __init__(self, wrap, mode: NoLogging.Mode = NoLogging.Mode.ALL):
         self.__wrapped__ = wrap
@@ -454,6 +476,12 @@ class NoLoggingProxy:
             attr = getattr(wrap, name)
         return NoLoggingProxy(attr, mode)
 
+    def __getitem__(self, k):
+        mode = self.__nl_mode__
+        with NoLogging(mode):
+            item = self.__wrapped__[k]
+        return NoLoggingProxy(item, mode)
+
     def __iter__(self):
         mode = self.__nl_mode__
         with NoLogging(mode):
@@ -474,6 +502,13 @@ class NoLoggingProxy:
         return NoLoggingProxy(rv, mode)
 
 
+def unwrap(t: _T) -> _T:
+    """
+    Unwrap an object that is potentially wrapped, say, as a `refinery.lib.tools.NoLoggingProxy`.
+    """
+    return getattr(t, '__wrapped__', t)
+
+
 class NotOne(LookupError):
     """
     A custom exception raised by `refinery.lib.tools.one` if the input iterator does not yield
@@ -504,7 +539,7 @@ def one(iterable: Iterable[_T]) -> _T:
         raise NotOne(False)
 
 
-def isodate(iso: str) -> Optional[datetime.datetime]:
+def isodate(iso: str) -> datetime.datetime | None:
     """
     Convert an input date string in ISO format to a `datetime` object. Contains fallbacks for early
     Python versions.
@@ -521,7 +556,7 @@ def isodate(iso: str) -> Optional[datetime.datetime]:
         return None
 
 
-def date_from_timestamp(ts: int):
+def date_from_timestamp(ts: int | float):
     """
     Convert a UTC timestamp to a datetime object.
     """
@@ -605,8 +640,68 @@ def nopdoc(obj: object):
     return obj
 
 
-def convert(x: _T | Any, t: Type[_T]) -> _T:
+def convert(x: _T | Any, t: type[_T]) -> _T:
     """
     Convert the given object `x` to the type `t`.
     """
     return x if isinstance(x, t) else t(x) # type:ignore
+
+
+class BoundsType:
+    """
+    Can be used to specify certain upper and lower bounds. For example, the following is `True`:
+
+        5 in bounds[3:5]
+
+    This is notably different from how a `range` object functions since the upper bound is included
+    in the valid range, and it is also permitted to be `None` for an unbounded range.
+    """
+    __name__ = 'bounds'
+
+    min: int
+    max: int | INF
+    inc: int
+
+    def __getitem__(self, k: slice):
+        return BoundsType(k)
+
+    def __init__(self, bounds: int | slice[int, int | None | INF, int | None] | None):
+        if bounds is None:
+            self.min = 0
+            self.max = INF
+            self.inc = 1
+        elif isinstance(bounds, int):
+            self.min = self.max = bounds
+            self.inc = 1
+        else:
+            _min, _max, _inc = bounds.start, bounds.stop, bounds.step
+            self.min = _min or 0
+            self.max = _max or INF
+            self.inc = _inc or 1
+            if _max and _max < self.min:
+                raise ValueError(F'The maximum {self.max} is lesser than the minimum {self.min}.')
+            if self.inc < 0:
+                raise ValueError('Negative step size not supported for range expressions.')
+
+    def __iter__(self):
+        k = self.min
+        i = self.inc
+        if (m := self.max) is INF:
+            yield from itertools.count(k, i)
+        else:
+            while k <= m:
+                yield k
+                k += i
+
+    def __repr__(self):
+        return F'[{self.min}:{self.max}:{self.inc}]'
+
+    def __contains__(self, value: int):
+        if value < self.min:
+            return False
+        if (m := self.max) and value > m:
+            return False
+        return (value - self.min) % self.inc == 0
+
+
+bounds = BoundsType(slice(None, None))
