@@ -8,17 +8,16 @@ standard library functions `builtins.open`, `os.stat`, and `mmap.mmap`.
 from __future__ import annotations
 
 import builtins
+import io
+import mmap
 import os
 import os.path
-import io
 import stat
-import uuid
 import threading
-import mmap
+import uuid
 
-from typing import Dict, Optional
-from refinery.lib.types import ByteStr
 from refinery.lib.structures import MemoryFile, MemoryFileMethods
+from refinery.lib.types import buf
 
 
 class VirtualFile:
@@ -32,11 +31,9 @@ class VirtualFile:
     extension to deduce a mode of operation.
     """
     name: str
-    path: str
-    node: int
-    data: Optional[ByteStr]
+    data: buf | None
 
-    def __init__(self, fs: VirtualFileSystem, data: Optional[ByteStr] = None, extension: Optional[str] = None):
+    def __init__(self, fs: VirtualFileSystem, data: buf | None = None, extension: str | None = None):
         extension = extension and F'.{extension}' or ''
         self.uuid = uuid.uuid4()
         self.name = F'{self.uuid!s}{extension}'
@@ -50,11 +47,13 @@ class VirtualFile:
     def node(self) -> int:
         return self.uuid.fields[1]
 
-    def mmap(self, length: int = 0, offset: int = 0) -> MemoryFile:
+    def mmap(self, length: int = 0, offset: int = 0) -> MemoryFileMethods:
         """
         Emulate the result of an `mmap` call to the virtual file.
         """
-        view = memoryview(self.data)
+        if (data := self.data) is None:
+            raise ValueError('Attempt to map a virtual file with no associated data.')
+        view = memoryview(data)
         node = self.node
         if length:
             view = view[offset:offset + length]
@@ -65,7 +64,6 @@ class VirtualFile:
 
         mapped = _MappedView()
         mapped[:] = view
-
         return mapped
 
     def open(self, mode: str) -> MemoryFile:
@@ -139,10 +137,10 @@ class VirtualFileSystem:
 
     def __init__(self):
         self._lock: threading.RLock = threading.RLock()
-        self._by_name: Dict[str, VirtualFile] = {}
-        self._by_node: Dict[int, VirtualFile] = {}
+        self._by_name: dict[str, VirtualFile] = {}
+        self._by_node: dict[int, VirtualFile] = {}
 
-    def new(self, data: Optional[ByteStr] = None, extension: Optional[str] = None):
+    def new(self, data: buf | None = None, extension: str | None = None):
         return VirtualFile(self, data, extension)
 
     def install(self, file: VirtualFile):
