@@ -3,15 +3,16 @@ Custom XML parser that is intended to be less strict than the standard library o
 """
 from __future__ import annotations
 
+import collections
+import re
 import uuid
 import weakref
-import re
-import defusedxml.ElementTree as et
-import collections
 
-from typing import Any, Dict, Iterable, List, Optional, TYPE_CHECKING
-from xml.parsers import expat
+from typing import TYPE_CHECKING, Any, Iterable
 from xml.etree.ElementTree import Element, ElementTree
+from xml.parsers import expat
+
+import defusedxml.ElementTree as et
 
 from refinery.lib.structures import MemoryFile
 from refinery.lib.tools import exception_to_string
@@ -20,53 +21,6 @@ if TYPE_CHECKING:
     from typing import Self
 
 
-def is_xml(data: bytearray, default=True):
-    """
-    Implements a heuristic check for whether the input is likely XML data.
-    """
-    _be = data[0:120:2]
-    _le = data[1:120:2]
-    if isinstance(data, memoryview):
-        _le = bytearray(_le)
-        _be = bytearray(_be)
-    if _le.count(0) > 100:
-        data = _be
-    if _be.count(0) > 100:
-        data = _le
-    if tag_match := re.search(BR'''(?x)
-        ^(?:            # at the very start of the document
-         \xef\xbb\xbf   # utf-8 byte order mark
-        |\xfe           # (utf-16be)
-        |\xff)?         # (utf-16le)
-        \s{0,10}        # allow for some leading white space
-        <               # a tag opens
-        ([?!]?          # allow for question or exclamation mark
-         [-:\w]{3,64})  # the tag name
-        \s{1,20}        # white space after tag name
-        (/?>            # the tag may end here, or:
-        |[-:\w]{3,32})  # we have an attribute.
-    ''', data):
-        tag = tag_match[1].lower()
-        end = tag_match[2].lower()
-        # <?xml...
-        if tag == B'?xml':
-            return True
-        # <HTML>
-        # <BODY>
-        if tag in (B'html', B'body'):
-            return False
-        # <!DOCTYPE html
-        if tag == '!doctype' and end == B'html':
-            return False
-        # <project xmlns:xsi=...
-        if end.startswith(b'xml'):
-            return True
-        # inconclusive, return default
-        return default
-    else:
-        return False
-
-
 def ForgivingParse(data: bytes, entities=None) -> ElementTree:
     """
     Uses the `refinery.lib.xml.ForgivingXMLParser` to parse the input data.
@@ -129,21 +83,21 @@ class XMLNodeBase:
 
     __slots__ = 'tag', 'index', 'children', 'empty', 'attributes', 'content', '_parent', '__weakref__'
 
-    attributes: Dict[str, Any]
-    children: List[Self]
-    content: Optional[str]
-    _parent: Optional[weakref.ReferenceType[Self]]
+    attributes: dict[str, Any]
+    children: list[Self]
+    content: str
+    _parent: weakref.ReferenceType[Self] | None
     empty: bool
-    tag: Optional[str]
+    tag: str | None
 
     def __init__(
         self,
-        tag: Optional[str],
-        index: Optional[int] = None,
-        parent: Optional[Self] = None,
-        content: Optional[str] = None,
+        tag: str | None,
+        index: int | None = None,
+        parent: Self | None = None,
+        content: str | None = None,
         empty: bool = False,
-        attributes: Optional[Dict[str, Any]] = None,
+        attributes: dict[str, Any] | None = None,
     ):
         if parent is None and index is not None:
             raise ValueError('Cannot set index for XML node without parent.')
@@ -151,14 +105,14 @@ class XMLNodeBase:
             attributes = {}
         self.tag = tag
         self.index = index
-        self.content = content
+        self.content = content or ''
         self.empty = empty
         self.children = []
         self.attributes = attributes
         self.parent = parent
 
     @property
-    def parent(self) -> Optional[XMLNodeBase]:
+    def parent(self) -> Self | None:
         parent = self._parent
         if parent is not None:
             parent = parent()
@@ -248,9 +202,9 @@ class XMLNode(XMLNodeBase):
     """
     __slots__ = 'source',
 
-    source: Optional[Element]
+    source: Element | None
 
-    def __init__(self, tag: str, parent: Optional[Self] = None, source: Optional[Element] = None):
+    def __init__(self, tag: str, parent: Self | None = None, source: Element | None = None):
         super().__init__(tag, parent=parent)
         self.source = source
 
@@ -261,7 +215,7 @@ class XMLNode(XMLNodeBase):
         return ElementTree(self.source).write(stream)
 
 
-def parse(data) -> Optional[XMLNode]:
+def parse(data) -> XMLNode | None:
     """
     This function is the primary export of the `refinery.lib.xml` module. It accepts raw XML data
     as input and returns an `refinery.lib.xml.XMLNode` representing the document root node as
