@@ -7,20 +7,19 @@ from __future__ import annotations
 import abc
 import itertools
 
-from typing import TYPE_CHECKING
 from functools import cached_property
+from typing import TYPE_CHECKING
 
-from refinery.units import Arg, Unit
-from refinery.lib.argformats import numseq
 from refinery.lib import chunks
-from refinery.lib.tools import infinitize
 from refinery.lib.inline import iterspread
-from refinery.lib.types import NoMask, INF
+from refinery.lib.tools import asbuffer, infinitize
+from refinery.lib.types import INF, NoMask, Param, buf, isq
+from refinery.units import Arg, Unit
 
 if TYPE_CHECKING:
+    from typing import Generator, Iterable, Literal, Union
+
     from numpy import ndarray
-    from typing import TypeVar, Union, Iterable, Generator, Optional, Tuple
-    _T = TypeVar('_T')
     _I = Union[Iterable[int], int]
 
 
@@ -32,18 +31,16 @@ class BlockTransformationBase(Unit, abstract=True):
 
     def __init__(
         self,
-        bigendian: Arg.Switch('-E', help='Read chunks in big endian.') = False,
-        blocksize: Arg.Number('-B', help=(
-            'The size of each block in bytes, default is 1.'
-        )) = None,
-        precision: Arg.Number('-P', help=(
+        bigendian: Param[bool, Arg.Switch('-E', help='Read chunks in big endian.')] = False,
+        blocksize: Param[int, Arg.Number('-B', help=(
+            'The size of each block in bytes. The default is {default}.'
+        ))] = 1,
+        precision: Param[int, Arg.Number('-P', help=(
             'The size of the variables used for computing the result. By default, this is equal to the block size. '
-            'The value may be zero, indicating that arbitrary precision is required.')) = None,
+            'The value may be zero, indicating that arbitrary precision is required.'))] = -1,
         _truncate: Arg.Delete() = 0,
         **keywords
     ):
-        if precision is None:
-            precision = blocksize
         self._truncate = _truncate
         super().__init__(bigendian=bigendian, blocksize=blocksize, precision=precision, **keywords)
 
@@ -55,11 +52,8 @@ class BlockTransformationBase(Unit, abstract=True):
             return '<'
 
     @cached_property
-    def _byte_order_adjective(self):
-        if self.args.bigendian:
-            return 'big'
-        else:
-            return 'little'
+    def _byte_order_adjective(self) -> Literal['big', 'little']:
+        return 'big' if self.args.bigendian else 'little'
 
     @property
     def bytestream(self):
@@ -70,13 +64,13 @@ class BlockTransformationBase(Unit, abstract=True):
         return self.blocksize == 1
 
     @property
-    def blocksize(self):
+    def blocksize(self) -> int:
         return self.args.blocksize or 1
 
     @property
-    def precision(self):
+    def precision(self) -> int | Literal[INF]:
         precision = self.args.precision
-        if precision is None:
+        if precision < 0:
             return self.blocksize
         if precision == 0:
             return INF
@@ -87,9 +81,9 @@ class BlockTransformationBase(Unit, abstract=True):
         return 8 * self.precision
 
     @property
-    def fmask(self):
+    def fmask(self) -> int | Literal[NoMask]:
         fbits = self.fbits
-        if fbits is INF:
+        if fbits == INF:
             return NoMask
         return (1 << fbits) - 1
 
@@ -103,7 +97,7 @@ class BlockTransformationBase(Unit, abstract=True):
         end = self.blocksize * (len(data) // self.blocksize)
         return data[end:]
 
-    def chunk_into_bytes(self, data: _T) -> Generator[_T | bytearray, None, None]:
+    def chunk_into_bytes(self, data: buf) -> Generator[buf]:
         """
         Returns an iterator over the blocks of the input data according to the current operational block
         size. The blocks are returned as slices of the input data. Note that zero bytes may be appended if
@@ -164,19 +158,19 @@ class BlockTransformation(BlockTransformationBase, abstract=True):
 
 class ArithmeticUnit(BlockTransformation, abstract=True):
 
-    def __init__(self, *argument: Arg(type=numseq, help=(
+    def __init__(self, *argument: Param[isq, Arg.NumSeq(help=(
         'A single numeric expression which provides the right argument to the operation, '
         'where the left argument is each block in the input data. This argument can also '
         'contain a sequence of bytes which is then split into blocks of the same size as '
-        'the input data and used cyclically.')),
-        bigendian=False, blocksize=None, precision=None, **kw
+        'the input data and used cyclically.'))],
+        bigendian=False, blocksize=1, precision=-1, **kw
     ):
         super().__init__(bigendian=bigendian, blocksize=blocksize, precision=precision, argument=argument, **kw)
 
-    def _argument_parse_hook(self, it: _I) -> Tuple[bool, _I]:
+    def _argument_parse_hook(self, it: _I) -> tuple[_I, bool]:
         return it, False
 
-    def _infinitize_argument(self, it: _I, masked=False) -> Iterable[int]:
+    def _infinitize_argument(self, min_size: int, it: _I, masked=False) -> Iterable[int]:
         def _mask(it):
             warnings = 3
             for block in it:
@@ -187,6 +181,13 @@ class ArithmeticUnit(BlockTransformation, abstract=True):
                     if not warnings:
                         self.log_warn('additional warnings are suppressed')
                 yield out
+        if isinstance(it, (bytes, bytearray)) and (n := len(it)) > 0x400:
+            quotient, remainder = divmod(min_size, n)
+            if remainder > 0:
+                quotient += 1
+            if quotient > 1:
+                it = it * quotient
+            return it
         if isinstance(it, int):
             it = (it,)
         if not masked:
@@ -198,7 +199,7 @@ class ArithmeticUnit(BlockTransformation, abstract=True):
         raise NotImplementedError
 
     @abc.abstractmethod
-    def inplace(self, block: ndarray, *args) -> Optional[ndarray]:
+    def inplace(self, block: ndarray, *args) -> ndarray | None:
         tmp: ndarray = self.operate(block, *args)
         if tmp.dtype != block.dtype:
             tmp = tmp.astype(block.dtype)
@@ -209,7 +210,7 @@ class ArithmeticUnit(BlockTransformation, abstract=True):
         import numpy
         return numpy
 
-    def _fastblock(self, data):
+    def _fastblock(self, data) -> bytes | bytearray:
         """
         Attempts to perform the operation more quickly by using numpy arrays.
         """
@@ -218,11 +219,22 @@ class ArithmeticUnit(BlockTransformation, abstract=True):
         except ImportError as IE:
             raise FastBlockError from IE
 
+        self.log_debug('fastblock: parsing and extending arguments')
+
+        def _execute_hooks():
+            for a in self.args.argument:
+                it, masked = self._argument_parse_hook(a)
+                na = self._infinitize_argument(len(data), it, masked)
+                yield it, masked, na
+
+        _hooks_executed = list(_execute_hooks())
+
         byte_order = self._byte_order_symbol
         num_blocks = len(data) // self.blocksize
 
         try:
-            if self.precision is None:
+            if self.precision is INF:
+                # NumPy type for Python Objects
                 dtype = numpy.dtype('O')
             else:
                 dtype = numpy.dtype(F'{byte_order}u{self.precision!s}')
@@ -232,9 +244,7 @@ class ArithmeticUnit(BlockTransformation, abstract=True):
         br_args = []
         np_args = []
 
-        for a in self.args.argument:
-            it, masked = self._argument_parse_hook(a)
-            na = self._infinitize_argument(it, masked)
+        for it, masked, na in _hooks_executed:
             br_args.append(na)
             if isinstance(it, int):
                 if not masked:
@@ -242,35 +252,63 @@ class ArithmeticUnit(BlockTransformation, abstract=True):
                 npa = int(it)
             elif self.precision is INF:
                 npa = numpy.array(list(itertools.islice(na, num_blocks)), dtype=dtype)
+            elif nb := asbuffer(na):
+                npa = numpy.frombuffer(nb, dtype, num_blocks)
             else:
                 npa = numpy.fromiter(na, dtype, num_blocks)
             np_args.append(npa)
 
-        overlap = len(data) - num_blocks * self.blocksize
+        if (_truncate := self._truncate) >= 2 or (_t := len(data) - num_blocks * self.blocksize) <= 0:
+            rest = ()
+        else:
+            rest = data[-_t:]
+            if _truncate < 1:
+                last_ops = [next(a) for a in br_args]
+                last_int = int.from_bytes(rest, self._byte_order_adjective)
+                dst_tail = self.operate(last_int, *last_ops) & self.fmask
+                dst_tail = dst_tail.to_bytes(self.blocksize, self._byte_order_adjective)
+                rest = dst_tail[:_t]
 
         try:
             stype = numpy.dtype(F'{byte_order}u{self.blocksize}')
         except TypeError as T:
             raise FastBlockError from T
 
-        src = numpy.frombuffer(memoryview(data), stype, num_blocks)
+        if not isinstance(data, bytearray):
+            data = bytearray(data)
+
+        self.log_debug(F'fastblock: loading {num_blocks} blocks of type {stype} from source data')
+
+        dst = data
+        src = numpy.frombuffer(dst, stype, num_blocks)
         if stype != dtype:
             src = src.astype(dtype)
+
         tmp = self.inplace(src, *np_args)
+
         if tmp is not None:
             src = tmp
+            dst = None
         if stype != dtype:
             src = src.astype(stype)
-        dst = bytearray(memoryview(src))
-        if overlap and self._truncate < 2:
-            rest = self.rest(data)
-            if self._truncate < 1:
-                last_ops = [next(a) for a in br_args]
-                last_int = int.from_bytes(rest, self._byte_order_adjective)
-                dst_tail = self.operate(last_int, *last_ops) & self.fmask
-                dst_tail = dst_tail.to_bytes(self.blocksize, self._byte_order_adjective)
-                rest = dst_tail[:overlap]
-            dst.extend(rest)
+            dst = None
+
+        mem = memoryview(src)
+
+        if dst is None:
+            dst = bytearray(mem)
+        elif (n := mem.nbytes) < len(dst):
+            del tmp
+            del mem
+            del src
+            try:
+                del dst[n:]
+            except BufferError:
+                import gc
+                gc.collect()
+                del dst[n:]
+
+        dst.extend(rest)
         return dst
 
     def process(self, data):
@@ -280,12 +318,14 @@ class ArithmeticUnit(BlockTransformation, abstract=True):
         except FastBlockError:
             pass
         except Exception as error:
+            if self.log_debug():
+                raise
             self.log_warn('falling back to default method after fast block failed with error:', error)
         else:
             self.log_debug('fast block method successful')
             return result
         arguments = [
-            self._infinitize_argument(*self._argument_parse_hook(a))
+            self._infinitize_argument(len(data), *self._argument_parse_hook(a))
             for a in self.args.argument]
         try:
             mask = self.fmask
@@ -310,7 +350,7 @@ class ArithmeticUnit(BlockTransformation, abstract=True):
 
 
 class UnaryOperation(ArithmeticUnit, abstract=True):
-    def __init__(self, bigendian=False, blocksize=None, **kw):
+    def __init__(self, bigendian=False, blocksize=1, **kw):
         super().__init__(
             bigendian=bigendian, blocksize=blocksize, **kw)
 
@@ -319,7 +359,7 @@ class UnaryOperation(ArithmeticUnit, abstract=True):
 
 
 class BinaryOperation(ArithmeticUnit, abstract=True):
-    def __init__(self, *argument, bigendian=False, blocksize=None):
+    def __init__(self, *argument, bigendian=False, blocksize=1):
         super().__init__(*argument, bigendian=bigendian, blocksize=blocksize)
 
     def inplace(self, block, argument) -> None:
@@ -329,16 +369,16 @@ class BinaryOperation(ArithmeticUnit, abstract=True):
 class BinaryOperationWithAutoBlockAdjustment(BinaryOperation, abstract=True):
     def __init__(
         self, *argument, bigendian=False,
-        blocksize: Arg.Number(help=(
+        blocksize: Param[int, Arg.Number(help=(
             'The size of each block in bytes. It is chosen, by default, to be the smallest size that can '
             'hold the provided argument without loss of precision. For example, passing the value 0x1234 '
             'will result in a default block size of 2, while passing the value 12 will mean that the '
             'default block size is 1.'
-        )) = None
+        ))] = 0
     ):
         super().__init__(*argument, bigendian=bigendian, blocksize=blocksize)
 
-    def _argument_parse_hook(self, it: _I) -> Tuple[bool, _I]:
+    def _argument_parse_hook(self, it: _I) -> tuple[_I, bool]:
         try:
             n = len(it)
         except TypeError:
@@ -347,7 +387,7 @@ class BinaryOperationWithAutoBlockAdjustment(BinaryOperation, abstract=True):
             if n == 1:
                 it = it[0]
         if masked := isinstance(it, int):
-            if self.args.blocksize is None:
+            if not self.args.blocksize:
                 self.log_debug('detected numeric argument with no specified block size')
                 bits = it.bit_length()
                 if bits > self.blocksize * 8:
@@ -364,11 +404,11 @@ class BinaryOperationWithAutoBlockAdjustment(BinaryOperation, abstract=True):
         try:
             blocksize = self._blocksize
         except AttributeError:
-            blocksize = None
+            blocksize = 0
         return blocksize or super().blocksize
 
     def process(self, data):
         try:
             return super().process(data)
         finally:
-            self._blocksize = None
+            self._blocksize = 0
