@@ -1,7 +1,8 @@
 from __future__ import annotations
+
 from typing import TYPE_CHECKING
 
-from refinery.units.blockwise import UnaryOperation, FastBlockError
+from refinery.units.blockwise import FastBlockError, UnaryOperation
 
 if TYPE_CHECKING:
     from numpy import ndarray
@@ -9,11 +10,12 @@ if TYPE_CHECKING:
 
 class rev(UnaryOperation):
     """
-    The blocks of the input data are output in reverse order. If the length of
-    the input data is not a multiple of the block size, the data is truncated.
+    By default, this reverses the input string. For different block sizes, the blocks of the
+    input data are output in reverse order. If the length of the input data is not a multiple
+    of the block size, the data is truncated.
     """
 
-    def __init__(self, blocksize=None):
+    def __init__(self, blocksize=1):
         super().__init__(blocksize=blocksize, _truncate=2)
 
     def inplace(self, block: ndarray):
