@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from io import BytesIO, SEEK_END
+from io import SEEK_END, BytesIO
 
 from refinery.units import Unit
 
@@ -64,53 +64,67 @@ class _bits_compress(BytesIO):
 
 
 class _bits_decompress(BytesIO):
+
+    __slots__ = 'bitcount', 'bitbuffer', 'bytebuf', 'decompressed'
+
     def __init__(self, data):
         super().__init__(data)
-        self.__bitcount = 0
-        self.__bitbuffer = 0
+        self.bitcount = 0
+        self.bitbuffer = 0
+        self.bytebuf = bytearray(1)
         self.decompressed = bytearray()
 
-    def read_bit(self):
-        return self.read_bits(1)
-
     def read_byte(self):
-        buffer = self.read(1)
-        if len(buffer) != 1:
-            raise BufferError('received zero bytes from read')
-        return buffer[0]
+        bb = self.bytebuf
+        if self.readinto(bb) != 1:
+            raise EOFError
+        return bb[0]
 
-    def read_bits(self, nbits):
-        while self.__bitcount < nbits:
-            self.__bitbuffer = (self.__bitbuffer << 8) | self.read_byte()
-            self.__bitcount += 8
-        self.__bitcount -= nbits
-        value, self.__bitbuffer = divmod(self.__bitbuffer, (1 << self.__bitcount))
+    def read_bits(self, nbits=1):
+        r1 = self.bytebuf
+        bc = self.bitcount
+        bb = self.bitbuffer
+        while bc < nbits:
+            if self.readinto(r1) != 1:
+                raise EOFError
+            bb <<= 8
+            bb |= r1[0]
+            bc += 8
+        bc -= nbits
+        value, bb = divmod(bb, (1 << bc))
+        self.bitbuffer = bb
+        self.bitcount = bc
         return value
 
     def read_variablenumber(self):
+        bit = self.read_bits
         result = 1
-        result = (result << 1) + self.read_bit()
-        while self.read_bit():
-            result = (result << 1) + self.read_bit()
-        return result
-
-    def read_setbits(self, max_, set_=1):
-        result = 0
-        while result < max_ and self.read_bit() == set_:
-            result += 1
+        result = (result << 1) + bit()
+        while bit():
+            result = (result << 1) + bit()
         return result
 
     def back_copy(self, offset, length=1):
-        for _ in range(length):
-            self.decompressed.append(self.decompressed[-offset])
-        return
-
-    def read_literal(self, value=None):
-        if value is None:
-            self.decompressed.append(self.read_byte())
+        buffer = self.decompressed
+        if offset == 0:
+            end = len(buffer)
+            buffer[end:end + length] = (buffer[0] for _ in range(length))
+        elif 1 <= length <= 8:
+            append = buffer.append
+            for _ in range(length):
+                append(buffer[-offset])
         else:
-            self.decompressed.append(value)
-        return False
+            write = buffer.extend
+            rep, rest = divmod(length, offset)
+            offset = len(buffer) - offset
+            if offset < 0:
+                raise IndexError
+            if rep > 0:
+                head = buffer[offset:]
+                for _ in range(rep):
+                    write(head)
+            if rest > 0:
+                write(buffer[offset:offset + rest])
 
 
 def lengthdelta(offset):
@@ -217,63 +231,70 @@ class compressor(_bits_compress):
 
 
 class decompressor(_bits_decompress):
+
+    __slots__ = 'pair', 'lastoffset'
+
     def __init__(self, data):
         super().__init__(data)
-        self.__pair = True
-        self.__lastoffset = 0
-        self.__functions = [
-            self.__literal,
-            self.__block,
-            self.__shortblock,
-            self.__singlebyte
-        ]
+        self.pair = True
+        self.lastoffset = 0
 
-    def __literal(self):
-        self.read_literal()
-        self.__pair = True
+    def literal(self):
+        self.decompressed.append(self.read_byte())
+        self.pair = True
         return False
 
-    def __block(self):
+    def block(self):
         b = self.read_variablenumber()
-        if b == 2 and self.__pair:
-            offset = self.__lastoffset
+        if b == 2 and self.pair:
+            offset = self.lastoffset
             length = self.read_variablenumber()
         else:
             high = b - 2
-            if self.__pair:
+            if self.pair:
                 high -= 1
             offset = (high << 8) + self.read_byte()
             length = self.read_variablenumber()
             length += lengthdelta(offset)
-        self.__lastoffset = offset
+        self.lastoffset = offset
         self.back_copy(offset, length)
-        self.__pair = False
+        self.pair = False
         return False
 
-    def __shortblock(self):
+    def shortblock(self):
         b = self.read_byte()
-        if b <= 1:  # likely 0
+        if b <= 1:
             return True
-        length = 2 + (b & 0x01)  # 2-3
-        offset = b >> 1  # 1-127
+        length = 2 + (b & 1)
+        offset = b >> 1
         self.back_copy(offset, length)
-        self.__lastoffset = offset
-        self.__pair = False
+        self.lastoffset = offset
+        self.pair = False
         return False
 
-    def __singlebyte(self):
-        offset = self.read_bits(4)  # 0-15
+    def singlebyte(self):
+        offset = self.read_bits(4)
         if offset:
             self.back_copy(offset)
         else:
-            self.read_literal(0)
-        self.__pair = True
+            self.decompressed.append(0)
+        self.pair = True
         return False
 
+    def read_sequence(self):
+        if not self.read_bits(1):
+            return self.literal()
+        if not self.read_bits(1):
+            return self.block()
+        if not self.read_bits(1):
+            return self.shortblock()
+        else:
+            return self.singlebyte()
+
     def decompress(self):
         self.seek(0)
-        self.read_literal()
-        while not self.__functions[self.read_setbits(3)]():
+        self.decompressed.append(self.read_byte())
+        while not self.read_sequence():
             continue
         return self.decompressed
 
@@ -298,7 +319,9 @@ class aplib(Unit):
         return decompressor(view[size:]).decompress()
 
     @classmethod
-    def handles(cls, data: bytearray):
+    def handles(cls, data):
+        if len(data) < 2:
+            return False
         if data[:4] == B'AP32':
             return True
         return None
