@@ -1,8 +1,7 @@
 from __future__ import annotations
 
 from enum import IntFlag
-from itertools import repeat, product
-
+from itertools import product, repeat
 from lzma import (
     FILTER_DELTA,
     FILTER_LZMA1,
@@ -10,15 +9,16 @@ from lzma import (
     FORMAT_ALONE,
     FORMAT_RAW,
     FORMAT_XZ,
+    PRESET_EXTREME,
     LZMACompressor,
     LZMADecompressor,
     LZMAError,
-    PRESET_EXTREME,
 )
 
-from refinery.units import Arg, Unit, RefineryPartialResult
-from refinery.lib.structures import MemoryFile
 from refinery.lib.decompression import parse_lzma_properties
+from refinery.lib.structures import MemoryFile
+from refinery.lib.types import Param
+from refinery.units import Arg, RefineryPartialResult, Unit
 
 __all__ = ['lzma', '_auto_decompress_lzma']
 
@@ -43,11 +43,11 @@ class lzma(Unit):
 
     def __init__(
         self,
-        raw   : Arg.Switch('-r', group='MODE', help='Use raw (no container) format.') = False,
-        alone : Arg.Switch('-a', group='MODE', help='Use the lzma container format.') = False,
-        xz    : Arg.Switch('-x', group='MODE', help='Use the default xz format.') = False,
-        level : Arg.Number('-l', bound=(0, 9), help='The compression level preset; between 0 and 9.') = 9,
-        delta : Arg.Number('-d', help='Add a delta filter when compressing.') = None,
+        raw: Param[bool, Arg.Switch('-r', group='MODE', help='Use raw (no container) format.')] = False,
+        alone: Param[bool, Arg.Switch('-a', group='MODE', help='Use the lzma container format.')] = False,
+        xz: Param[bool, Arg.Switch('-x', group='MODE', help='Use the default xz format.')] = False,
+        level: Param[int, Arg.Number('-l', bound=(0, 9), help='The compression level preset; between 0 and 9.')] = 9,
+        delta: Param[int, Arg.Number('-d', help='Add a delta filter when compressing.')] = 0,
     ):
         if (raw, alone, xz).count(True) > 1:
             raise ValueError('Only one container format can be enabled.')
@@ -58,7 +58,7 @@ class lzma(Unit):
 
     def reverse(self, data):
         filters = []
-        if self.args.delta is not None:
+        if self.args.delta > 0:
             self.log_debug('adding delta filter')
             filters.append({'id': FILTER_DELTA, 'dist': self.args.delta})
         if self.args.alone:
@@ -159,7 +159,7 @@ class lzma(Unit):
         return self._process(data, partial=True)
 
     @classmethod
-    def handles(cls, data: bytearray):
+    def handles(cls, data):
         if data[:4] == B'\x5D\0\0\0':
             return True
         if data[:5] == B'\xFD7zXZ':
