@@ -1,10 +1,11 @@
 from __future__ import annotations
 
+import copy
 import io
 import struct
-import copy
 
-from refinery.units import Arg, Unit, RefineryPartialResult
+from refinery.lib.types import Param
+from refinery.units import Arg, RefineryPartialResult, Unit
 
 
 class lznt1(Unit):
@@ -144,5 +145,10 @@ class lznt1(Unit):
             out.write(chunk)
         return out.getvalue()
 
-    def __init__(self, chunk_size: Arg.Number('-c', help='Optionally specify the chunk size for compression, default is 0x1000.') = 0x1000):
+    def __init__(
+        self,
+        chunk_size: Param[int, Arg.Number('-c', help=(
+            'Optionally specify the chunk size for compression, default is 0x1000.')
+        )] = 0x1000
+    ):
         super().__init__(chunk_size=chunk_size)
