@@ -1,7 +1,7 @@
 from __future__ import annotations
 
-from refinery.units import Unit, RefineryPartialResult
-from refinery.lib.structures import StructReader, MemoryFile
+from refinery.lib.structures import MemoryFile, StructReaderBits
+from refinery.units import RefineryPartialResult, Unit
 
 
 class pkw(Unit):
@@ -26,9 +26,9 @@ class pkw(Unit):
             raise ValueError(
                 'Failed to decode a symbol in the compressed data stream.')
 
-        reader = StructReader(data)
-        codelit = reader.u8()  # First byte is 0 if literals are uncoded, otherwise 1
-        maxdict = reader.u8()  # Second byte is 4, 5, or 6 (max size of dictionary)
+        reader = StructReaderBits(data)
+        codelit = reader.u8fast()  # First byte is 0 if literals are uncoded, otherwise 1
+        maxdict = reader.u8fast()  # Second byte is 4, 5, or 6 (max size of dictionary)
 
         if not 0 <= codelit <= 1:
             raise ValueError(F'Invalid literal encoding value {codelit}.')
@@ -65,7 +65,7 @@ class pkw(Unit):
         return output.getvalue()
 
     @classmethod
-    def handles(cls, data: bytearray) -> bool:
+    def handles(cls, data) -> bool:
         return (len(data) > 2) and (0 <= data[0] <= 1) and (4 <= data[1] <= 6)
 
 
