@@ -1,10 +1,11 @@
 from __future__ import annotations
 
-import zlib
 import itertools
+import zlib
 
-from refinery.units import Arg, Unit, RefineryPartialResult
 from refinery.lib.tools import exception_to_string
+from refinery.lib.types import Param
+from refinery.units import Arg, RefineryPartialResult, Unit
 
 
 class zl(Unit):
@@ -14,10 +15,10 @@ class zl(Unit):
 
     def __init__(
         self,
-        level  : Arg.Number('-l', bound=(0, 0X9), help='Specify a compression level between 0 and 9.') = 9,
-        window : Arg.Number('-w', bound=(8, 0XF), help='Manually specify the window size between 8 and 15.') = 15,
-        zlib_header: Arg.Switch('-z', group='MODE', help='Use a ZLIB header.') = False,
-        gzip_header: Arg.Switch('-g', group='MODE', help='Use a GZIP header.') = False
+        level: Param[int, Arg.Number('-l', bound=(0, 0X9), help='Specify a compression level between 0 and 9.')] = 9,
+        window: Param[int, Arg.Number('-w', bound=(8, 0XF), help='Manually specify the window size between 8 and 15.')] = 15,
+        zlib_header: Param[bool, Arg.Switch('-z', group='MODE', help='Use a ZLIB header.')] = False,
+        gzip_header: Param[bool, Arg.Switch('-g', group='MODE', help='Use a GZIP header.')] = False
     ):
         if zlib_header and gzip_header:
             raise ValueError('You can only specify one header type (ZLIB or GZIP).')
@@ -32,6 +33,8 @@ class zl(Unit):
             try:
                 chunk = zl.decompress(memory[:read])
             except zlib.error as e:
+                if not result:
+                    raise
                 raise RefineryPartialResult(exception_to_string(e), result) from e
             else:
                 result.extend(chunk)
@@ -48,32 +51,37 @@ class zl(Unit):
             modes = [-self.args.window, self.args.window | 0x20]
         modes.extend([0x10 | self.args.window, 0])
         view = memoryview(data)
+        rest = view
         step = 32 if self.leniency > 0 else len(data)
+        count = 0
+        error = None
         for k in itertools.count(1):
             error = None
-            rest = view
             for mode in modes:
+                msg = F'decompressing chunk {k} with mode {mode & 0xFF:02X}'
                 try:
                     out, rest = self._decompress_data(view, mode, step)
+                    yield out
                 except Exception as e:
+                    self.log_info(F'{msg} failed: {e!s}')
                     error = error or e
                 else:
-                    self.log_info(F'used mode {mode} to decompress chunk {k}')
-                    yield out
+                    self.log_info(F'{msg} ok, remaining data:', rest, clip=True)
+                    count += 1
                     error = None
+                    modes = [mode]
                     break
-            if error:
-                raise error
-            if not rest:
-                break
-            if len(rest) == len(view):
+            if error or not rest or len(rest) == len(view):
                 break
             if len(rest) > len(view):
                 raise RuntimeError('Decompressor returned more tail data than input data.')
-            yield out
             view = rest
-        if k <= 0:
-            raise ValueError('Could not detect any zlib stream.')
+        if count <= 0:
+            raise error or ValueError('Could not detect any zlib stream.')
+        if rest:
+            from refinery.lib.meta import SizeInt
+            size = SizeInt(len(rest))
+            raise RefineryPartialResult(F'{size!r} excess data after compressed stream', rest)
 
     def reverse(self, data):
         mode = -self.args.window
@@ -87,7 +95,7 @@ class zl(Unit):
         return zz + zl.flush(zlib.Z_FINISH)
 
     @classmethod
-    def handles(cls, data: bytearray):
+    def handles(cls, data):
         for sig in (
             B'\x1F\x8B',  # gzip header
             B'\x78\x01',  # zlib low compression
