@@ -5,57 +5,46 @@ from __future__ import annotations
 
 import abc
 
-from typing import (
-    Any,
-    ByteString,
-    ClassVar,
-    Iterable,
-    Optional,
-    Collection,
-    Type,
-)
-
-from refinery.lib.tools import isbuffer
-from refinery.lib.types import ByteStr
-
-from refinery.lib.crypto import (
-    pad,
-    unpad,
-    CipherObjectFactory,
-    CipherInterface,
-)
 from refinery.lib.argformats import (
     Option,
-    extract_options,
     OptionFactory,
+    extract_options,
 )
+from refinery.lib.crypto import (
+    CipherInterface,
+    CipherObjectFactory,
+    pad,
+    unpad,
+)
+from refinery.lib.tools import isbuffer
+from refinery.lib.types import Any, ClassVar, Collection, Iterable, Param, buf, isq
 from refinery.units import (
     Arg,
+    Chunk,
     Executable,
     RefineryCriticalException,
     RefineryPartialResult,
     Unit,
-    Chunk,
 )
 
 
 class CipherUnit(Unit, abstract=True):
 
-    key_size: Optional[Collection[int]] = None
+    key_size: Collection[int] | None = None
     block_size: int
 
-    def __init__(self, key: Arg(help='The encryption key.'), **keywords):
+    def __init__(self, key: Param[buf, Arg(help='The encryption key.')], **keywords):
         super().__init__(key=key, **keywords)
 
     @abc.abstractmethod
-    def decrypt(self, data: Chunk) -> ByteStr:
+    def decrypt(self, data: Chunk) -> buf:
         raise NotImplementedError
 
     @abc.abstractmethod
-    def encrypt(self, data: Chunk) -> ByteStr:
+    def encrypt(self, data: Chunk) -> buf:
         raise NotImplementedError
 
-    def process(self, data: Chunk) -> ByteStr:
+    def process(self, data: Chunk) -> buf:
         ks = self.key_size
         if ks and len(self.args.key) not in ks:
             import itertools
@@ -77,7 +66,7 @@ class CipherUnit(Unit, abstract=True):
             raise ValueError(F'the given key has an invalid length of {len(self.args.key)} bytes; {msg}{pt}')
         return self.decrypt(data)
 
-    def reverse(self, data: Chunk) -> ByteStr:
+    def reverse(self, data: Chunk) -> buf:
         return self.encrypt(data)
 
 
@@ -87,8 +76,8 @@ class StreamCipherUnit(CipherUnit, abstract=True):
 
     def __init__(
         self, key,
-        discard: Arg.Number('-d', help='Discard the first {varname} bytes of the keystream, {default} by default.') = 0,
-        stateful: Arg.Switch('-s', help='Do not reset the key stream while processing the chunks of one frame.') = False,
+        discard: Param[int, Arg.Number('-d', help='Discard the first {varname} bytes of the keystream, {default} by default.')] = 0,
+        stateful: Param[bool, Arg.Switch('-s', help='Do not reset the key stream while processing the chunks of one frame.')] = False,
         **keywords
     ):
         super().__init__(key=key, stateful=stateful, discard=discard, **keywords)
@@ -136,13 +125,19 @@ PADDINGS_ALL = PADDINGS_LIB + [PADDING_NONE]
 
 class BlockCipherUnitBase(CipherUnit, abstract=True):
     def __init__(
-        self, key, iv: Arg('-i', '--iv', help=(
-            'Specifies the initialization vector. If none is specified, then a block of zero bytes is used.')) = None,
-        padding: Arg.Choice('-p', type=str.lower, choices=PADDINGS_ALL, metavar='P', help=(
-            'Choose a padding algorithm ({choices}). The raw algorithm does nothing. By default, all other algorithms '
-            'are attempted. In most cases, the data was not correctly decrypted if none of these work.')
-        ) = None,
-        raw: Arg.Switch('-r', '--raw', help='Set the padding to raw; ignored when a padding is specified.') = False,
+        self, key,
+        iv: Param[buf, Arg('-i', '--iv', help=(
+            'Specifies the initialization vector. If none is specified, then a block of zero bytes '
+            'is used.')
+        )] = B'',
+        padding: Param[str | None, Arg.Choice('-p', choices=PADDINGS_ALL, metavar='P', help=(
+            'Choose a padding algorithm ({choices}). The raw algorithm does nothing. By default, '
+            'all other algorithms are attempted. In most cases, the data was not correctly '
+            'decrypted if none of these work.')
+        )] = None,
+        raw: Param[bool, Arg.Switch('-r', '--raw', help=(
+            'Set the padding to raw; ignored when a padding is specified.')
+        )] = False,
         **keywords
     ):
         if not padding and raw:
@@ -155,13 +150,13 @@ class BlockCipherUnitBase(CipherUnit, abstract=True):
         raise NotImplementedError
 
     @property
-    def iv(self) -> ByteString:
+    def iv(self) -> buf:
         return self.args.iv or bytes(self.block_size)
 
-    def _default_padding(self) -> Optional[str]:
+    def _default_padding(self) -> str | None:
         return self.args.padding
 
-    def reverse(self, data: Chunk) -> ByteStr:
+    def reverse(self, data: Chunk) -> buf:
         padding = self._default_padding()
         if padding is not None:
             self.log_info('padding method:', padding)
@@ -169,7 +164,7 @@ class BlockCipherUnitBase(CipherUnit, abstract=True):
                 pad(data, self.block_size, padding)
         return super().reverse(data)
 
-    def process(self, data: Chunk) -> ByteStr:
+    def process(self, data: Chunk) -> buf:
         padding = self._default_padding()
         result = self.labelled(super().process(data))
         if padding is None:
@@ -193,16 +188,16 @@ class BlockCipherUnitBase(CipherUnit, abstract=True):
 
 class StandardCipherExecutable(Executable):
 
-    _available_block_cipher_modes: Type[Option]
-    _cipher_factory: Optional[CipherObjectFactory]
+    _available_block_cipher_modes: type[Option]
+    _cipher_factory: CipherObjectFactory | None
 
-    def __new__(mcs, name, bases, nmspc, cipher: Optional[CipherObjectFactory] = None):
+    def __new__(mcs, name, bases, nmspc, cipher: CipherObjectFactory | None = None):
         keywords: dict = dict(abstract=(cipher is None))
-        return super(StandardCipherExecutable, mcs).__new__(mcs, name, bases, nmspc, **keywords)
+        return super().__new__(mcs, name, bases, nmspc, **keywords)
 
-    def __init__(_class, name, bases, nmspc, cipher: Optional[CipherObjectFactory] = None):
+    def __init__(_class, name, bases, nmspc, cipher: CipherObjectFactory | None = None):
         abstract = cipher is None
-        super(StandardCipherExecutable, _class).__init__(name, bases, nmspc, abstract=abstract)
+        super().__init__(name, bases, nmspc, abstract=abstract)
         _class._cipher_factory = cipher
         if abstract:
             return
@@ -241,9 +236,9 @@ class StandardCipherExecutable(Executable):
 
 class StandardCipherUnit(CipherUnit, metaclass=StandardCipherExecutable):
 
-    _available_block_cipher_modes: ClassVar[Type[Option]]
+    _available_block_cipher_modes: ClassVar[type[Option]]
     _cipher_factory: ClassVar[CipherObjectFactory]
-    _cipher_interface: Optional[CipherInterface] = None
+    _cipher_interface: CipherInterface | None = None
 
     def _new_cipher(self, **optionals) -> CipherInterface:
         self.log_info(lambda: F'encryption key: {self.args.key.hex()}')
@@ -261,26 +256,24 @@ class StandardCipherUnit(CipherUnit, metaclass=StandardCipherExecutable):
         return self._get_cipher().block_size
 
     @property
-    def key_size(self) -> Optional[Collection[int]]:
+    def key_size(self) -> Collection[int] | None:
         return self._get_cipher().key_size
 
-    def encrypt(self, data: Chunk) -> ByteStr:
+    def encrypt(self, data: Chunk) -> buf:
         cipher = self._get_cipher(True)
         assert cipher.block_size == self.block_size
         return cipher.encrypt(data)
 
-    def decrypt(self, data: Chunk) -> ByteStr:
-        cipher = self._get_cipher(True)
-        assert cipher.block_size == self.block_size
+    def decrypt(self, data: Chunk) -> buf:
         try:
-            return cipher.decrypt(data)
+            return self._get_cipher(True).decrypt(data)
         except ValueError:
             overlap = len(data) % self.block_size
             if not overlap:
                 raise
             del data[-overlap:]
             self.log_warn(F'removing {overlap} bytes from the input to make it a multiple of the {self.block_size}-byte block size')
-            return cipher.decrypt(data)
+            return self._get_cipher(True).decrypt(data)
 
 
 class StandardBlockCipherUnit(BlockCipherUnitBase, StandardCipherUnit):
@@ -289,21 +282,21 @@ class StandardBlockCipherUnit(BlockCipherUnitBase, StandardCipherUnit):
         self, key, *,
         iv=B'',
         padding=None, mode=None, raw=False,
-        little_endian: Arg.Switch('-e', '--little-endian', help=(
+        little_endian: Param[bool, Arg.Switch('-e', '--little-endian', help=(
             'Only for CTR: Use a little endian counter instead of the default big endian.'
-        )) = False,
-        segment_size: Arg.Number('-S', '--segment-size', help=(
+        ))] = False,
+        segment_size: Param[int, Arg.Number('-S', '--segment-size', help=(
             'Only for CFB: Number of segmentation bits. It must be a multiple of 8. The default '
             'of {default} means that the block size will be used as the segment size.'
-        )) = 0,
-        tag: Arg.NumSeq('-t', '--tag', metavar='TAG', help=(
+        ))] = 0,
+        tag: Param[isq, Arg.NumSeq('-t', '--tag', metavar='TAG', help=(
             'Only for EAX, GCM, OCB, and CCM: An authentication tag to verify the message. For '
             'encryption, this parameter specifies the tag length, and the tag is provided as a '
             'meta variable named "tag".'
-        )) = None,
-        aad: Arg.Binary('-a', '--aad', metavar='AAD', help=(
+        ))] = (),
+        aad: Param[buf, Arg.Binary('-a', '--aad', metavar='AAD', help=(
             'Only for EAX, GCM, OCB, and CCM: Set additional authenticated data.'
-        )) = None,
+        ))] = B'',
         **keywords
     ):
         mode = self._available_block_cipher_modes(mode or iv and 'CBC' or 'ECB')
@@ -322,7 +315,7 @@ class StandardBlockCipherUnit(BlockCipherUnitBase, StandardCipherUnit):
             **keywords
         )
 
-    def _default_padding(self) -> Optional[str]:
+    def _default_padding(self) -> str | None:
         padding = super()._default_padding()
         if padding is not None:
             return padding
@@ -357,7 +350,7 @@ class StandardBlockCipherUnit(BlockCipherUnitBase, StandardCipherUnit):
         return self._get_cipher().block_size
 
     @property
-    def key_size(self) -> Optional[Collection[int]]:
+    def key_size(self) -> Collection[int] | None:
         return self._get_cipher().key_size
 
     def _new_cipher(self, **optionals) -> CipherInterface:
@@ -415,10 +408,10 @@ class LatinCipherUnit(StreamCipherUnit, abstract=True):
 
     def __init__(
         self, key, stateful=False, discard=0,
-        nonce: Arg(help='The nonce. Default is the string {default}.') = B'REFINERY',
-        magic: Arg('-m', help='The magic constant; depends on the key size by default.') = B'',
-        offset: Arg.Number('-x', help='Optionally specify the stream index, default is {default}.') = 0,
-        rounds: Arg.Number('-r', help='The number of rounds. Has to be an even number. Default is {default}.') = 20,
+        nonce: Param[buf, Arg(help='The nonce. Default is the string {default}.')] = B'REFINERY',
+        magic: Param[buf, Arg('-m', help='The magic constant; depends on the key size by default.')] = B'',
+        offset: Param[int, Arg.Number('-x', help='Optionally specify the stream index, default is {default}.')] = 0,
+        rounds: Param[int, Arg.Number('-r', help='The number of rounds. Has to be an even number. Default is {default}.')] = 20,
     ):
         super().__init__(
             key=key,
@@ -432,7 +425,7 @@ class LatinCipherUnit(StreamCipherUnit, abstract=True):
 
 
 class LatinCipherStandardUnit(StandardCipherUnit):
-    def __init__(self, key, nonce: Arg(help='The nonce. Default is the string {default}.') = B'REFINERY'):
+    def __init__(self, key, nonce: Param[buf, Arg(help='The nonce. Default is the string {default}.')] = B'REFINERY'):
         super().__init__(key, nonce=nonce)
 
     def _new_cipher(self, **optionals) -> Any:
