@@ -3,8 +3,9 @@ from __future__ import annotations
 from Cryptodome.Cipher import ARC2
 
 from refinery.lib.crypto import PyCryptoFactoryWrapper
+from refinery.lib.types import Param
 from refinery.units import Arg
-from refinery.units.crypto.cipher import StandardBlockCipherUnit, CipherInterface
+from refinery.units.crypto.cipher import CipherInterface, StandardBlockCipherUnit
 
 
 class rc2(StandardBlockCipherUnit, cipher=PyCryptoFactoryWrapper(ARC2)):
@@ -15,10 +16,10 @@ class rc2(StandardBlockCipherUnit, cipher=PyCryptoFactoryWrapper(ARC2)):
     def __init__(
         self, key, *,
         iv=b'',
-        eks: Arg.Number('-k', '--eks', group='EKS',
-            help='Set the effective key size. Default is {default}.') = 1024,
-        derive_eks: Arg.Switch('-d', '--dks', group='EKS',
-            help='Act as .NET and derive the effective key size from the key length.') = False,
+        eks: Param[int, Arg.Number('-k', '--eks', group='EKS',
+            help='Set the effective key size. Default is {default}.')] = 1024,
+        derive_eks: Param[bool, Arg.Switch('-d', '--dks', group='EKS',
+            help='Act as .NET and derive the effective key size from the key length.')] = False,
         padding=None,
         mode=None,
         raw=False,
