@@ -2,8 +2,9 @@ from __future__ import annotations
 
 from Cryptodome.Cipher import ARC4
 
-from refinery.units.crypto.cipher import StandardCipherUnit, Arg
 from refinery.lib.crypto import PyCryptoFactoryWrapper
+from refinery.lib.types import Param
+from refinery.units.crypto.cipher import Arg, StandardCipherUnit
 
 ARC4.key_size = range(1, 257)
 
@@ -14,7 +15,7 @@ class rc4(StandardCipherUnit, cipher=PyCryptoFactoryWrapper(ARC4)):
     """
     def __init__(
         self, key,
-        discard: Arg.Number('-d', help='Discard the first {varname} bytes of the keystream, {default} by default.') = 0,
+        discard: Param[int, Arg.Number('-d', help='Discard the first {varname} bytes of the keystream, {default} by default.')] = 0,
     ):
         super().__init__(key, discard=discard)
 
