@@ -2,13 +2,15 @@ from __future__ import annotations
 
 import struct
 
-from Cryptodome.Cipher import Salsa20
 from abc import ABC, abstractmethod
-from typing import List, Union, Sequence, Optional, Iterable, Tuple, Type, TypeVar
+from typing import Iterable, Sequence, TypeVar
 
-from refinery.units.crypto.cipher import LatinCipherUnit, LatinCipherStandardUnit
-from refinery.lib.crypto import rotl32, PyCryptoFactoryWrapper
-from refinery.lib.types import ByteStr
+from Cryptodome.Cipher import Salsa20
+
+from refinery.lib.crypto import PyCryptoFactoryWrapper, rotl32
+from refinery.lib.tools import asbuffer
+from refinery.lib.types import buf
+from refinery.units.crypto.cipher import LatinCipherStandardUnit, LatinCipherUnit
 
 
 class LatinCipher(ABC):
@@ -17,15 +19,23 @@ class LatinCipher(ABC):
     _idx_key32: slice
     _idx_nonce: slice
     _idx_count: slice
-    _round_access_pattern: Tuple[Tuple[int, int, int, int], ...]
+    _round_access_pattern: tuple[
+        tuple[int, int, int, int],
+        tuple[int, int, int, int],
+        tuple[int, int, int, int],
+        tuple[int, int, int, int],
+        tuple[int, int, int, int],
+        tuple[int, int, int, int],
+        tuple[int, int, int, int],
+        tuple[int, int, int, int],
+    ]
 
     @classmethod
-    def FromState(cls, state: Union[Sequence[int], ByteStr]):
-        try:
-            state = struct.unpack('<16L', state)
-        except TypeError:
-            pass
-        state: List[int] = list(state)
+    def FromState(cls, state: Sequence[int] | buf):
+        if b := asbuffer(state):
+            state = struct.unpack('<16L', b)
+        else:
+            state = list(state)
         if len(state) != 16:
             raise ValueError('State must contain 16 DWORDs')
         key = struct.pack(
@@ -38,9 +48,9 @@ class LatinCipher(ABC):
             '<2L', *state[cls._idx_count]), 'little')
         return cls(key, nonce, magic, counter=count)
 
-    def __init__(self, key: ByteStr, nonce: ByteStr, magic: Optional[ByteStr] = None, rounds: int = 20, counter: int = 0):
+    def __init__(self, key: buf, nonce: buf, magic: buf | None = None, rounds: int = 20, counter: int = 0):
         if len(key) == 16:
-            key = 2 * key
+            key = 2 * bytes(key)
         elif len(key) != 32:
             raise ValueError('The key must be of length 16 or 32.')
         if rounds % 2:
@@ -49,19 +59,16 @@ class LatinCipher(ABC):
             nonce = bytearray(8)
         elif len(nonce) != 8:
             raise ValueError('The nonce must be of length 8.')
-        if magic:
-            if len(magic) != 16:
-                raise ValueError('The initialization magic must be 16 bytes in length.')
-        elif len(key) == 16:
-            magic = B'expand 16-byte k'
-        elif len(key) == 32:
-            magic = B'expand 32-byte k'
-        key = struct.unpack('<8L', key)
-        self.key16 = key[:4]
-        self.key32 = key[4:]
+        if not magic:
+            magic = B'expand %d-byte k' % len(key)
+        elif len(magic) != 16:
+            raise ValueError('The initialization magic must be 16 bytes in length.')
+        _key = struct.unpack('<8L', key)
+        self.key16 = _key[:4]
+        self.key32 = _key[4:]
         self.magic = struct.unpack('<4L', magic)
         self.nonce = struct.unpack('<2L', nonce)
-        self.state: List[int] = [0] * 4 * 4
+        self.state: list[int] = [0] * 4 * 4
         self.rounds = rounds // 2
         self.reset(counter)
 
@@ -82,11 +89,12 @@ class LatinCipher(ABC):
             hi = hi + 1 & 0xFFFFFFFF
         self.state[self._idx_count] = self.counter = lo, hi
 
+    @staticmethod
     @abstractmethod
-    def quarter(self, x: List[int], a: int, b: int, c: int, d: int):
+    def quarter(x: list[int], a: int, b: int, c: int, d: int):
         raise NotImplementedError
 
-    def permute(self, x: List[int]):
+    def permute(self, x: list[int]):
         for a, b, c, d in self.rounds * self._round_access_pattern:
             self.quarter(x, a, b, c, d)
 
@@ -114,11 +122,11 @@ class SalsaCipher(LatinCipher):
         (0x0, 0x1, 0x2, 0x3),
         (0x5, 0x6, 0x7, 0x4),
         (0xA, 0xB, 0x8, 0x9),
-        (0xF, 0xC, 0xD, 0xE)
+        (0xF, 0xC, 0xD, 0xE),
     )
 
     @staticmethod
-    def quarter(x: List[int], a: int, b: int, c: int, d: int) -> None:
+    def quarter(x: list[int], a: int, b: int, c: int, d: int) -> None:
         x[b] ^= rotl32(x[a] + x[d] & 0xFFFFFFFF, 0x07)
         x[c] ^= rotl32(x[b] + x[a] & 0xFFFFFFFF, 0x09)
         x[d] ^= rotl32(x[c] + x[b] & 0xFFFFFFFF, 0x0D)
@@ -129,13 +137,13 @@ _X = TypeVar('_X', bound=LatinCipher)
 
 
 def LatinX(
-    cipher: Type[_X],
+    cipher: type[_X],
     blocks: Iterable[int],
-    key: ByteStr,
-    kdn: ByteStr,
-    kdp: ByteStr,
-    nonce: ByteStr,
-    magic: ByteStr,
+    key: buf,
+    kdn: buf,
+    kdp: int,
+    nonce: buf,
+    magic: buf,
     rounds: int,
     offset: int,
 ) -> _X:
@@ -192,4 +200,3 @@ class salsa20(LatinCipherStandardUnit, cipher=PyCryptoFactoryWrapper(Salsa20)):
     with 20 rounds, but it uses the PyCryptodome library C implementation rather than the pure
     Python implementation used by `refinery.salsa`.
     """
-    pass
