@@ -3,19 +3,20 @@ Implements various hashing algorithms.
 """
 from __future__ import annotations
 
-from refinery.units import Arg, Unit, ByteStr, abc
+from refinery.lib.types import Param, buf
+from refinery.units import Arg, Unit, abc
 
 
 class HashUnit(Unit, abstract=True):
 
     @abc.abstractmethod
-    def _algorithm(self, data: ByteStr) -> bytes:
+    def _algorithm(self, data: buf) -> bytes:
         raise NotImplementedError
 
     def __init__(
         self,
-        reps: Arg.Number('-r', help='Optionally specify a number of times to apply the hash to its own output.') = 1,
-        text: Arg.Switch('-t', help='Output a hexadecimal representation of the hash.') = False,
+        reps: Param[int, Arg.Number('-r', help='Optionally specify a number of times to apply the hash to its own output.')] = 1,
+        text: Param[bool, Arg.Switch('-t', help='Output a hexadecimal representation of the hash.')] = False,
         **kwargs
     ):
         super().__init__(text=text, reps=reps, **kwargs)
