@@ -7,19 +7,18 @@ from __future__ import annotations
 
 import importlib
 
-from refinery.units import Arg, Unit
-from refinery.lib.argformats import number
-from refinery.lib.types import ByteStr
-
 from enum import Enum
-from typing import Callable, TYPE_CHECKING, cast
+from typing import TYPE_CHECKING, Callable, cast
+
+from refinery.lib.types import Param, buf
+from refinery.units import Arg, Unit
 
 if TYPE_CHECKING:
     from typing import Protocol
 
     class _Hash(Protocol):
-        def update(self, data: ByteStr): ...
-        def digest(self) -> ByteStr: ...
+        def update(self, data: buf): ...
+        def digest(self) -> buf: ...
         def hexdigest(self) -> str: ...
 
     class _HashModule(Protocol):
@@ -40,7 +39,7 @@ class HASH(str, Enum):
     SHA384 = 'SHA384'
 
 
-def multidecode(data: ByteStr, function: Callable[[str], ByteStr]) -> ByteStr:
+def multidecode(data: buf, function: Callable[[str], buf]) -> buf:
     for codec in ['utf8', 'latin1', 'cp1252']:
         try:
             return function(data.decode(codec))
@@ -54,11 +53,12 @@ class KeyDerivation(Unit, abstract=True):
 
     def __init__(
         self,
-        size: Arg(help='The number of bytes to generate.', type=number),
-        salt: Arg(help='Salt for the derivation.'),
-        hash: Arg.Option(choices=HASH, metavar='hash',
-            help='Specify one of these algorithms (default is {default}): {choices}') = None,
-        iter: Arg.Number(metavar='iter', help='Number of iterations; default is {default}.') = None,
+        size: Param[int, Arg.Number(help='The number of bytes to generate.')],
+        salt: Param[buf, Arg.Binary(help='Salt for the derivation.')],
+        hash: Param[str, Arg.Option(choices=HASH, metavar='hash',
+            help='Specify one of these algorithms (default is {default}): {choices}')] = HASH.SHA1,
+        iter: Param[int, Arg.Number(metavar='iter',
+            help='Number of iterations; default is {default}.')] = 0,
         **kw
     ):
         if hash is not None:
