@@ -1,6 +1,7 @@
 from __future__ import annotations
 
-from refinery.units import Unit, Arg
+from refinery.lib.types import Param, buf
+from refinery.units import Arg, Unit
 
 
 class argon2id(Unit):
@@ -10,13 +11,13 @@ class argon2id(Unit):
 
     def __init__(
         self,
-        size: Arg.Number(metavar='n', help='number of bytes to generate'),
-        salt: Arg.Binary(metavar='S', help='salt bytes'),
-        iter: Arg.Number(metavar='t', help='number of iterations, defaults to {default}') = 1,
-        jobs: Arg.Number(metavar='p', help='parallelism, defaults to {default}') = 1,
-        cost: Arg.Number(metavar='m', help='memory cost in kibibytes, defaults to the minimum of 8192 per job.') = None,
-        skey: Arg.Binary(metavar='K', help='optional secret key') = None,
-        more: Arg.Binary(metavar='X', help='optional additional data') = None,
+        size: Param[int, Arg.Number(metavar='n', help='number of bytes to generate')],
+        salt: Param[buf, Arg.Binary(metavar='S', help='salt bytes')],
+        iter: Param[int, Arg.Number(metavar='t', help='number of iterations, defaults to {default}')] = 1,
+        jobs: Param[int, Arg.Number(metavar='p', help='parallelism, defaults to {default}')] = 1,
+        cost: Param[int, Arg.Number(metavar='m', help='memory cost in kibibytes, defaults to the minimum of 8192 per job.')] = None,
+        skey: Param[buf, Arg.Binary(metavar='K', help='optional secret key')] = None,
+        more: Param[buf, Arg.Binary(metavar='X', help='optional additional data')] = None,
     ):
         super().__init__(size=size, salt=salt, iter=iter, skey=skey, jobs=jobs, cost=cost, more=more)
 
