@@ -1,9 +1,9 @@
 from __future__ import annotations
 
+from refinery.lib.types import Param
 from refinery.units import RefineryPartialResult
 from refinery.units.crypto.keyderive import Arg, KeyDerivation
 
-
 _DES_PARITYTABLE = bytearray((
     0x01, 0x01, 0x02, 0x02, 0x04, 0x04, 0x07, 0x07,
     0x08, 0x08, 0x0B, 0x0B, 0x0D, 0x0D, 0x0E, 0x0E,
@@ -50,7 +50,7 @@ class deskd(KeyDerivation):
     converts a string to an 8 byte DES key with odd byte parity, per FIPS specification. This is not a modern
     key derivation function.
     """
-    def __init__(self, size: Arg(help='The number of bytes to generate, default is the maximum of 8.') = 8):
+    def __init__(self, size: Param[int, Arg(help='The number of bytes to generate, default is the maximum of 8.')] = 8):
         super().__init__(size=size, salt=None)
 
     def process(self, password):
