@@ -1,5 +1,5 @@
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 import base64
 
 from refinery.units import Unit
@@ -30,6 +30,6 @@ class b32(Unit):
     @classmethod
     def handles(cls, data):
         from refinery.lib.patterns import formats
-        if not formats.b32.value.fullmatch(data):
+        if not formats.b32.value.bin.fullmatch(data):
             return False
-        return not formats.hex.value.fullmatch(data)
+        return not formats.hex.value.bin.fullmatch(data)
