@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import base64
 
+from refinery.lib.types import Param
 from refinery.units import Arg, Unit
 
 
@@ -9,7 +10,7 @@ class b64(Unit):
     """
     Base64 encoding and decoding.
     """
-    def __init__(self, urlsafe: Arg.Switch('-u', help='use URL-safe alphabet') = False):
+    def __init__(self, urlsafe: Param[bool, Arg.Switch('-u', help='use URL-safe alphabet')] = False):
         super().__init__(urlsafe=urlsafe)
 
     def reverse(self, data):
@@ -35,9 +36,9 @@ class b64(Unit):
         return numpy
 
     @classmethod
-    def handles(cls, data: bytearray) -> bool:
+    def handles(cls, data) -> bool:
         from refinery.lib.patterns import formats
-        if not formats.spaced_b64.value.bin.fullmatch(data):
+        if not formats.b64s.value.bin.fullmatch(data):
             return False
         try:
             np = cls._numpy
