@@ -19,6 +19,6 @@ class b85(Unit):
         return base64.b85decode(data)
 
     @classmethod
-    def handles(cls, data: bytearray):
+    def handles(cls, data):
         from refinery.lib.patterns import formats
-        return formats.spaced_b85.value.bin.fullmatch(data) is not None
+        return formats.b85s.value.bin.fullmatch(data) is not None
