@@ -1,7 +1,7 @@
 from __future__ import annotations
 
-from refinery.units import Unit, RefineryPartialResult
-from refinery.lib.structures import StructReader, MemoryFile
+from refinery.lib.structures import MemoryFile, StructReaderBits
+from refinery.units import RefineryPartialResult, Unit
 
 _B92_ALPHABET = (
     RB"!#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_abcdefghijklmnopqrstuvwxyz{|}"
@@ -19,7 +19,7 @@ class b92(Unit):
         if not data:
             return B'~'
 
-        reader = StructReader(data, bigendian=True)
+        reader = StructReaderBits(data, bigendian=True)
         output = MemoryFile()
         while reader.remaining_bits > 0:
             try:
@@ -101,6 +101,6 @@ class b92(Unit):
         return output.getvalue()
 
     @classmethod
-    def handles(cls, data: bytearray):
+    def handles(cls, data):
         from refinery.lib.patterns import formats
         return formats.b92.value.bin.fullmatch(data) is not None
