@@ -3,9 +3,8 @@ from __future__ import annotations
 import math
 import re
 
+from refinery.lib.types import Param, isq
 from refinery.units import Arg, Unit
-from refinery.lib.argformats import numseq
-
 
 _DEFAULT_ALPH_STR = R'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
 _DEFAULT_ALPHABET = B'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
@@ -23,13 +22,13 @@ class base(Unit):
     """
     def __init__(
         self,
-        base: Arg(type=numseq, metavar='base|alphabet', help=(
+        base: Param[isq, Arg.NumSeq(metavar='base|alphabet', help=(
             R'Either the base to be used or an alphabet. If an explicit alphabet is given, its length '
             R'determines the base. The default base 0 treats the input as a Python integer literal. If '
-            F'a numeric base is given, digits from the alphabet "{_DEFAULT_ALPH_STR}" are used. ')) = 0,
-        strip_padding: Arg.Switch('-s', help='Do not add leading zeros to the output.') = False,
-        little_endian: Arg.Switch('-e', help='Use little endian byte order instead of big endian.') = False,
-        strict_digits: Arg.Switch('-d', help='Check that all input digits are part of the alphabet.') = False,
+            F'a numeric base is given, digits from the alphabet "{_DEFAULT_ALPH_STR}" are used. '))] = 0,
+        strip_padding: Param[bool, Arg.Switch('-s', help='Do not add leading zeros to the output.')] = False,
+        little_endian: Param[bool, Arg.Switch('-e', help='Use little endian byte order instead of big endian.')] = False,
+        strict_digits: Param[bool, Arg.Switch('-d', help='Check that all input digits are part of the alphabet.')] = False,
     ):
         super().__init__(
             base=base,
@@ -84,7 +83,10 @@ class base(Unit):
         return result
 
     def process(self, data: bytearray):
+        if not data:
+            return data
         base, alphabet = self._args
+        self.log_debug(F'decoding data using base {base}; alphabet {alphabet!r}')
         be_lenient = not self.args.strict_digits
         if be_lenient and alphabet.upper() == alphabet:
             lcased = (c + 0x20 if 0x41 <= c <= 0x5a else c for c in data)
@@ -105,20 +107,21 @@ class base(Unit):
                 result = int(data_translated, base)
             else:
                 result = int(data, base)
-        elif len(alphabet) == 64:
+        elif len(alphabet) == 64 and len(data) >= 4:
             import base64
             _b64_alphabet = _LARGER_ALPHABETS[64]
             if alphabet != _b64_alphabet:
                 data = data.translate(bytes.maketrans(alphabet, _b64_alphabet))
             return base64.b64decode(data + b'===', validate=self.args.strict_digits)
-        elif len(alphabet) == 85:
+        elif len(alphabet) == 85 and len(data) >= 5:
             import base64
             _b85_alphabet = _LARGER_ALPHABETS[85]
             if alphabet != _b85_alphabet:
                 data = data.translate(bytes.maketrans(alphabet, _b85_alphabet))
             return base64.b85decode(data)
         else:
-            self.log_warn('very long alphabet, unable to use built-ins; reverting to (slow) fallback.')
+            if len(data) > 100_000:
+                self.log_warn('long alphabet & unable to use built-ins; reverting to (slow) fallback.')
             result = 0
             lookup = {digit: k for k, digit in enumerate(alphabet)}
             for digit in data:
