@@ -15,15 +15,15 @@ class hex(Unit):
         return base64.b16encode(data)
 
     def process(self, data):
-        import re
         import base64
+        import re
         data = re.sub(B'[^A-Fa-f0-9]+', B'', data)
         if len(data) % 2:
             data = data[:-1]
         return base64.b16decode(data, casefold=True)
 
     @classmethod
-    def handles(cls, data: bytearray):
+    def handles(cls, data):
         from refinery.lib.patterns import formats
-        if formats.spaced_hex.fullmatch(data):
+        if formats.b16s.value.bin.fullmatch(data):
             return True
