@@ -4,10 +4,10 @@ import itertools
 import pathlib
 import re
 
-from refinery.units import Unit, RefineryPartialResult
-from refinery.lib.structures import MemoryFile
 from refinery.lib import chunks
 from refinery.lib.meta import metavars
+from refinery.lib.structures import MemoryFile
+from refinery.units import RefineryPartialResult, Unit
 
 
 class uuenc(Unit):
@@ -74,4 +74,4 @@ class uuenc(Unit):
         if len(data) < 16:
             return False
         if data[:6] == B'begin ':
-            return set(data[6:9]) <= set(B'01234567')
+            return re.fullmatch(B'[0-7]{3}', data[6:9]) is not None
