@@ -2,8 +2,10 @@ from __future__ import annotations
 
 import re
 
-from refinery.units import Arg, Unit
+from refinery.lib.id import is_likely_vbe
 from refinery.lib.patterns import formats
+from refinery.lib.types import Param
+from refinery.units import Arg, Unit
 
 
 class wshenc(Unit):
@@ -102,10 +104,10 @@ class wshenc(Unit):
 
     def __init__(
         self,
-        marker: Arg.Switch('-m', '--no-marker', off=True, help=(
+        marker: Param[bool, Arg.Switch('-m', '--no-marker', off=True, help=(
             'Do not require magic marker when encoding and do not search for '
             'marker when decoding.')
-        ) = True
+        )] = True
     ):
         super().__init__(marker=marker)
 
@@ -163,10 +165,5 @@ class wshenc(Unit):
         return bytearray(self._decoded(self._unescape(data)))
 
     @classmethod
-    def handles(cls, data: bytearray):
-        view = memoryview(data)
-        if not re.search(BR'#@~\^[!-~]{6}==', view[:+64]):
-            return None
-        if not re.search(BR'[!-~]{6}==\^#~@', view[-64:]):
-            return None
-        return True
+    def handles(cls, data):
+        return is_likely_vbe(data)
