@@ -1,16 +1,16 @@
 from __future__ import annotations
 
-from typing import NamedTuple, Optional, Union, List, Dict, Iterable, Tuple, Generator, Set
-from enum import Enum
-
 import io
 import math
 import struct
 
-from refinery.lib.structures import Struct, StructReader, MemoryFile
-from refinery.units.formats import PathExtractorUnit, UnpackResult
-from refinery.lib.tools import date_from_timestamp
+from enum import Enum
+from typing import Generator, Iterable, NamedTuple
 
+from refinery.lib.id import buffer_contains
+from refinery.lib.structures import MemoryFile, Struct, StructReader, StructReaderBits
+from refinery.lib.tools import date_from_timestamp
+from refinery.units.formats import PathExtractorUnit, UnpackResult
 
 A3X_KEYWORDS = [
     '',
@@ -510,12 +510,12 @@ A3X_OPCODES = {
     0x58: ':',
 }
 
-_PRETTY: Dict[str, str] = {}
+_PRETTY: dict[str, str] = {}
 _PRETTY.update((name.lower(), name) for name in A3X_APICALLS)
 _PRETTY.update((name.lower(), name) for name in A3X_KEYWORDS)
 
 
-def a3x_number_representation(value: Union[float, int]) -> str:
+def a3x_number_representation(value: float | int) -> str:
     """
     The AutoIt3 compiler will emit constants for -inf/inf/nan. However, AutoIt3 itself has no
     keywords for these values; The output of this function is valid AutoIt3 code that will
@@ -526,13 +526,13 @@ def a3x_number_representation(value: Union[float, int]) -> str:
     if isinstance(value, int):
         return str(value)
     elif math.isnan(value) or math.isinf(value):
-        value = struct.pack('>d', value).hex().upper()
-        return RF'Dec("{value}", 3)'
+        rep = struct.pack('>d', value).hex().upper()
+        return RF'Dec("{rep}", 3)'
     else:
         return RF'{value!s}'
 
 
-def a3x_decompile(bytecode: bytearray) -> Generator[Tuple[int, str]]:
+def a3x_decompile(bytecode: bytes | bytearray) -> Generator[tuple[int, str]]:
     class _decompiler(dict):
         def __missing__(self, key):
             if key == 's':
@@ -559,10 +559,10 @@ def a3x_decompile(bytecode: bytearray) -> Generator[Tuple[int, str]]:
             return a3x_number_representation(number)
 
     decompiler = _decompiler()
-    reader = A3xReader(bytecode)
+    reader = A3xReader(memoryview(bytecode))
     num_lines = reader.u32()
-    tokens: List[str] = []
-    expected_terminators: List[str] = []
+    tokens: list[str] = []
+    expected_terminators: list[str] = []
     line = 0
 
     while line < num_lines and not reader.eof:
@@ -631,7 +631,7 @@ def a3x_decompress(data: bytearray, is_current: bool) -> bytearray:
     cursor = 0
     view = memoryview(data)
     size = int.from_bytes(view[4:8], 'big')
-    bit_reader = StructReader(view[8:], bigendian=True)
+    bit_reader = StructReaderBits(view[8:], bigendian=True)
     bits = bit_reader.read_integer
     while cursor < size:
         check = bits(1)
@@ -793,30 +793,8 @@ class A3xEncryptionConstants(NamedTuple):
 
 
 class A3xEncryptionType(A3xEncryptionConstants, Enum):
-    EA06 = A3xEncryptionConstants(
-        b'AU3!EA06',
-        tag_size=0xADBC,
-        tag=0xB33F,
-        path_size=0xF820,
-        path=0xF479,
-        compressed_size=0x87BC,
-        decompressed_size=0x87BC,
-        checksum=0xA685,
-        data=0x2477,
-        is_current=True
-    )
-    EA05 = A3xEncryptionConstants(
-        b'AU3!EA05',
-        tag_size=0x29BC,
-        tag=0xA25E,
-        path_size=0x29AC,
-        path=0xF25E,
-        compressed_size=0x45AA,
-        decompressed_size=0x45AA,
-        checksum=0xC3D2,
-        data=0x22AF,
-        is_current=False
-    )
+    EA06 = (b'AU3!EA06', 0xADBC, 0xB33F, 0xF820, 0xF479, 0x87BC, 0x87BC, 0xA685, 0x2477, True)
+    EA05 = (b'AU3!EA05', 0x29BC, 0xA25E, 0x29AC, 0xF25E, 0x45AA, 0x45AA, 0xC3D2, 0x22AF, False)
 
 
 class A3xReader(StructReader[memoryview]):
@@ -847,7 +825,7 @@ class A3xReader(StructReader[memoryview]):
         return ''.join(chr((self.u16() ^ length) & 0xFFFF) for _ in range(length))
 
 
-class A3xRecord(Struct, parser=A3xReader):
+class A3xRecord(Struct):
     MAGIC = b'\x6B\x43\xCA\x52'
 
     def __init__(self, reader: A3xReader, encryption_type: A3xEncryptionType):
@@ -926,7 +904,7 @@ class A3xRecord(Struct, parser=A3xReader):
             return 'script.ahk'
 
 
-class A3xScript(Struct, parser=A3xReader):
+class A3xScript(Struct):
     MAGIC = b'\xA3\x48\x4B\xBE\x98\x6C\x4A\xA9\x99\x4C\x53\x0A\x86\xD6\x48\x7D'
     WIDTH = 0x28
 
@@ -941,7 +919,7 @@ class A3xScript(Struct, parser=A3xReader):
         if not self.type.startswith(B'AU3!'):
             self.body = []
             return
-        self.body: List[A3xRecord] = []
+        self.body: list[A3xRecord] = []
         last_known_good_position = reader.tell()
         types = {at.version: at for at in A3xEncryptionType}
         while not reader.eof:
@@ -970,12 +948,12 @@ class a3x(PathExtractorUnit):
     def unpack(self, data: bytearray):
         view = memoryview(data)
         cursor = 0
-        errors: Dict[int, Exception] = {}
+        errors: dict[int, Exception] = {}
         script_count = 0
-        truncated: Set[A3xRecord] = set()
-        intact: Set[A3xRecord] = set()
+        truncated: set[A3xRecord] = set()
+        intact: set[A3xRecord] = set()
 
-        def _package(records: Iterable[A3xRecord]) -> Generator[UnpackResult, None, None]:
+        def _package(records: Iterable[A3xRecord]) -> Generator[UnpackResult]:
             for k, record in enumerate(records, 1):
                 self.log_info(F'record {k} type:', record.type)
                 self.log_info(F'record {k} path:', record.src_path)
@@ -1000,7 +978,7 @@ class a3x(PathExtractorUnit):
                     break
                 cursor = rp
             try:
-                script = A3xScript(view[cursor:])
+                script = A3xScript.Parse(view[cursor:])
             except Exception as E:
                 errors[cursor] = E
                 cursor += 1
@@ -1052,5 +1030,5 @@ class a3x(PathExtractorUnit):
                 raise error
 
     @classmethod
-    def handles(cls, data: bytearray) -> Optional[bool]:
-        return A3xScript.MAGIC in data or A3xRecord.MAGIC in data
+    def handles(cls, data) -> bool | None:
+        return buffer_contains(data, A3xScript.MAGIC) or buffer_contains(data, A3xRecord.MAGIC)
