@@ -1,10 +1,9 @@
 from __future__ import annotations
 
-from refinery.units import Unit
-
-from refinery.lib.inno.archive import InnoArchive
+from refinery.lib.inno.archive import InnoArchive, is_inno_setup
 from refinery.lib.inno.emulator import InnoSetupEmulator, NewPassword
 from refinery.lib.inno.ifps import IFPSFile
+from refinery.units import Unit
 
 
 class innopwd(Unit):
@@ -15,7 +14,7 @@ class innopwd(Unit):
     """
     def process(self, data: bytearray):
         if data.startswith(IFPSFile.Magic):
-            inno = IFPSFile(data)
+            inno = IFPSFile.Parse(data)
             self.log_info('running in script-only mode; cannot check passwords')
             file = None
         else:
@@ -33,7 +32,7 @@ class innopwd(Unit):
         for password in emulator.emulate_installation():
             if not isinstance(password, NewPassword):
                 continue
-            if file and not inno.check_password(file, password):
+            if isinstance(inno, InnoArchive) and file and not inno.check_password(file, password):
                 self.log_info('discarding password:', password)
                 continue
             yield password.encode(self.codec)
@@ -43,10 +42,4 @@ class innopwd(Unit):
 
     @classmethod
     def handles(cls, data):
-        import re
-        if data[:2] != B'MZ':
-            return False
-        if re.search(re.escape(InnoArchive.ChunkPrefix), data) is None:
-            return False
-        return bool(
-            re.search(BR'Inno Setup Setup Data \(\d+\.\d+\.', data))
+        return is_inno_setup(data)
