@@ -1,8 +1,8 @@
 from __future__ import annotations
 
-from refinery.units.formats.archive import ArchiveUnit
-from refinery.lib.thirdparty import acefile
 from refinery.lib.structures import MemoryFile
+from refinery.lib.thirdparty import acefile
+from refinery.units.formats.archive import ArchiveUnit
 
 
 class xtace(ArchiveUnit, docs='{0}{s}{PathExtractorUnit}'):
@@ -22,5 +22,5 @@ class xtace(ArchiveUnit, docs='{0}{s}{PathExtractorUnit}'):
             )
 
     @classmethod
-    def handles(cls, data: bytearray) -> bool:
-        return b'**ACE**' in data[:0x100]
+    def handles(cls, data) -> bool:
+        return data[7:14] == b'**ACE**'
