@@ -1,8 +1,8 @@
 from __future__ import annotations
 
-from refinery.units.formats.archive import ArchiveUnit
+from refinery.lib.cab import CabDisk, Cabinet
 from refinery.units import Chunk
-from refinery.lib.cab import Cabinet, CabDisk
+from refinery.units.formats.archive import ArchiveUnit
 
 
 class xtcab(ArchiveUnit, docs='{0}{p}{PathExtractorUnit}'):
@@ -40,5 +40,5 @@ class xtcab(ArchiveUnit, docs='{0}{p}{PathExtractorUnit}'):
             yield box
 
     @classmethod
-    def handles(cls, data: bytearray):
-        return data.startswith(CabDisk.MAGIC)
+    def handles(cls, data):
+        return data[:4] == CabDisk.MAGIC
