@@ -1,12 +1,12 @@
 from __future__ import annotations
 
-from gzip import GzipFile, FEXTRA, FNAME
 from datetime import datetime
+from gzip import FEXTRA, FNAME, GzipFile
 from pathlib import Path
 
-from refinery.units.formats.archive import ArchiveUnit
-from refinery.lib.structures import Struct, StructReader, StreamDetour
 from refinery.lib.meta import metavars
+from refinery.lib.structures import StreamDetour, Struct, StructReader
+from refinery.units.formats.archive import ArchiveUnit
 
 
 class GzipHeader(Struct):
@@ -16,13 +16,16 @@ class GzipHeader(Struct):
         unpacker = GzipFile(fileobj=reader)
         with StreamDetour(reader, 0):
             self.magic = reader.read(2)
-            self.method, self.flags, self.mtime = reader.read_struct('BBIxx')
+            self.method = reader.u8()
+            self.flags = reader.u8()
+            self.mtime = reader.u32()
+            reader.skip(2)
             self.extra = None
             self.name = None
             if self.flags & FEXTRA:
                 self.extra = reader.read(reader.u16())
             if self.flags & FNAME:
-                self.name = reader.read_c_string().decode('latin1')
+                self.name = reader.read_c_string('latin1')
         self.data = unpacker.read()
 
 
@@ -31,7 +34,7 @@ class xtgz(ArchiveUnit):
     Extract a file from a GZip archive.
     """
     def unpack(self, data: bytearray):
-        archive = GzipHeader(data)
+        archive = GzipHeader.Parse(data)
         path = archive.name
         date = archive.mtime
         date = date and datetime.fromtimestamp(date) or None
@@ -52,5 +55,5 @@ class xtgz(ArchiveUnit):
         yield self._pack(path, date, archive.data)
 
     @classmethod
-    def handles(cls, data: bytearray) -> bool:
-        return data.startswith(B'\x1F\x8B')
+    def handles(cls, data) -> bool:
+        return data[:2] == B'\x1F\x8B'
