@@ -1,12 +1,12 @@
 from __future__ import annotations
 
-from typing import Optional
-from itertools import cycle, islice
-
 import zlib
 
-from refinery.units.formats.archive import ArchiveUnit
+from itertools import cycle, islice
+
+from refinery.lib.id import buffer_contains
 from refinery.lib.structures import StructReader
+from refinery.units.formats.archive import ArchiveUnit
 
 
 class ISSReader(StructReader[bytearray]):
@@ -88,5 +88,7 @@ class xtiss(ArchiveUnit, docs='{0}{s}{PathExtractorUnit}'):
             yield self._pack(name, None, data)
 
     @classmethod
-    def handles(cls, data: bytearray) -> Optional[bool]:
-        return data.startswith(B'MZ') and any(data.find(m) > 0 for m in ISSReader.MAGIC)
+    def handles(cls, data) -> bool | None:
+        if data[:2] != B'MZ':
+            return False
+        return any(buffer_contains(data, m) for m in ISSReader.MAGIC)
