@@ -2,8 +2,8 @@ from __future__ import annotations
 
 from enum import IntFlag
 
+from refinery.lib.structures import StreamDetour, Struct, StructReader
 from refinery.units.formats.archive import ArchiveUnit
-from refinery.lib.structures import Struct, StructReader, StreamDetour
 
 
 class CPUType(IntFlag):
@@ -65,7 +65,7 @@ class xtmacho(ArchiveUnit):
             count -= 1
 
     @classmethod
-    def handles(cls, data: bytearray):
+    def handles(cls, data):
         return data[:4] in (
             cls._SIGNATURE_BE,
             cls._SIGNATURE_LE,
