@@ -1,20 +1,21 @@
 from __future__ import annotations
-from typing import Iterable, Optional, Dict
 
-import re
 import json
+import re
+
+from typing import Iterable
 
-from refinery.units.formats.archive import Arg, ArchiveUnit, UnpackResult
-from refinery.units.encoding.esc import esc
-from refinery.lib.structures import StructReader
 from refinery.lib.patterns import formats
-from refinery.lib.types import ByteStr, JSON
+from refinery.lib.structures import StructReader
+from refinery.lib.types import JSON, Param, buf
+from refinery.units.encoding.esc import esc
+from refinery.units.formats.archive import ArchiveUnit, Arg, UnpackResult
 from refinery.units.pattern.carve_json import JSONCarver
 
 
 class JSONReader(StructReader):
 
-    def read_string(self) -> Optional[str]:
+    def read_string(self) -> str | None:
         quote = self.u8()
         value = bytearray()
         if quote not in B'\"\'':
@@ -31,7 +32,7 @@ class JSONReader(StructReader):
             value.append(char)
         return value | esc | str
 
-    def read_json(self) -> Optional[JSON]:
+    def read_json(self) -> JSON:
         while self.u8() not in b'[{':
             pass
         self.seekrel(-1)
@@ -65,7 +66,7 @@ class xtnode(ArchiveUnit, docs='{0}{p}{PathExtractorUnit}'):
     _PKG_COMMON_JS = B'sourceMappingURL=common.js.map'
 
     def __init__(
-        self, *paths, entry: Arg.Switch('-u', help='Only extract the entry point.') = False,
+        self, *paths, entry: Param[bool, Arg.Switch('-u', help='Only extract the entry point.')] = False,
         list=False, join_path=False, drop_path=False, fuzzy=0, exact=False, regex=False,
         path=b'path', date=b'date',
     ):
@@ -73,7 +74,7 @@ class xtnode(ArchiveUnit, docs='{0}{p}{PathExtractorUnit}'):
             list=list, join_path=join_path, drop_path=drop_path, fuzzy=fuzzy, exact=exact, regex=regex,
             path=path, date=date)
 
-    def unpack(self, data: ByteStr) -> Iterable[UnpackResult]:
+    def unpack(self, data: buf) -> Iterable[UnpackResult]:
         if self._is_nexe(data):
             self.log_info('unpacking as nexe')
             yield from self._unpack_nexe(data)
@@ -83,7 +84,7 @@ class xtnode(ArchiveUnit, docs='{0}{p}{PathExtractorUnit}'):
             yield from self._unpack_pkg(data)
             return
 
-    def _unpack_nexe(self, data: ByteStr):
+    def _unpack_nexe(self, data: buf):
         try:
             ep = re.compile(
                 RB"entry\s*=\s*path\.resolve\(path\.dirname\(process\.execPath\),\s*(%s)\)" % formats.string)
@@ -124,7 +125,7 @@ class xtnode(ArchiveUnit, docs='{0}{p}{PathExtractorUnit}'):
                         continue
                     yield UnpackResult(path, blob[offset:end])
 
-    def _unpack_pkg(self, data: ByteStr):
+    def _unpack_pkg(self, data: buf):
         def _extract_coordinates(*v: bytes):
             for name in v:
                 pattern = name + BR'''\s{0,3}=\s{0,3}(['"])([\s\d]+)\1'''
@@ -150,7 +151,7 @@ class xtnode(ArchiveUnit, docs='{0}{p}{PathExtractorUnit}'):
 
         reader = JSONReader(prelude[mapping.end() - 1:])
 
-        files: Dict[str, dict] = reader.read_json()
+        files: dict[str, dict] = reader.read_json()
 
         if files is None:
             raise ValueError('failed to read file list')
@@ -204,11 +205,11 @@ class xtnode(ArchiveUnit, docs='{0}{p}{PathExtractorUnit}'):
                 yield UnpackResult(path, data)
 
     @classmethod
-    def _is_nexe(cls, data: ByteStr) -> bool:
+    def _is_nexe(cls, data: buf) -> bool:
         return cls._NEXE_SENTINEL in data
 
     @classmethod
-    def _is_pkg(cls, data: ByteStr) -> bool:
+    def _is_pkg(cls, data: buf) -> bool:
         if cls._PKG_PAYLOAD_P not in data:
             return False
         if cls._PKG_PAYLOAD_S not in data:
@@ -222,5 +223,5 @@ class xtnode(ArchiveUnit, docs='{0}{p}{PathExtractorUnit}'):
         return True
 
     @classmethod
-    def handles(cls, data: ByteStr) -> Optional[bool]:
+    def handles(cls, data: buf) -> bool | None:
         return cls._is_nexe(data) or cls._is_pkg(data)
