@@ -1,32 +1,27 @@
 from __future__ import annotations
-from typing import TYPE_CHECKING, NamedTuple
 
-import marshal
+import codecs
+import contextlib
+import dataclasses
 import enum
-import io
+import marshal
+import os
+import os.path
 import re
 import uuid
 import zlib
-import os
-import os.path
-import contextlib
-import codecs
-import dataclasses
 
-from refinery.units.formats.archive import Arg, ArchiveUnit
-from refinery.units.pattern.carve import carve
-from refinery.units.formats.pym import Marshal
-from refinery.lib.structures import MemoryFile, StreamDetour, Struct, StructReader
-from refinery.lib.tools import NoLogging, normalize_word_separators
-from refinery.lib.shared import xdis, decompyle3, uncompyle6
+from typing import TYPE_CHECKING, Callable, cast
 
 from Cryptodome.Cipher import AES
 
-if TYPE_CHECKING:
-    from types import CodeType
-    from typing import cast, Callable, Dict, List, Tuple, Optional, Set, Union, Generator, Iterable
-    from xdis import Instruction
-    from refinery.lib.types import ByteStr
+from refinery.lib.py import decompile_buffer, version2tuple
+from refinery.lib.shared import xdis
+from refinery.lib.structures import StreamDetour, Struct, StructReader
+from refinery.lib.types import Param, buf
+from refinery.units.formats.archive import ArchiveUnit, Arg
+from refinery.units.formats.pym import Marshal
+from refinery.units.pattern.carve import carve
 
 
 class Unmarshal(enum.IntEnum):
@@ -35,129 +30,13 @@ class Unmarshal(enum.IntEnum):
     YesAndDecompile = 2
 
 
-def version2tuple(version: str):
-    if m := re.fullmatch(R'^(\d+\.\d+(?:\.\d+)?)(.*)$', version):
-        return tuple(int(k, 10) for k in m.group(1).split('.'))
-    raise ValueError(version)
-
-
-def decompress_peek(buffer, size=512) -> Optional[bytes]:
+def decompress_peek(buffer, size=512) -> bytes | None:
     try:
         return zlib.decompressobj().decompress(buffer[:size])
     except zlib.error:
         return None
 
 
-class Code(NamedTuple):
-    version: Tuple[int]
-    timestamp: int
-    magic: int
-    container: CodeType
-    is_pypi: bool
-    code_objects: dict
-
-
-def extract_code_from_buffer(buffer: ByteStr, file_name: Optional[str] = None) -> Generator[Code, None, None]:
-    code_objects = {}
-    file_name = file_name or '<unknown>'
-    load = xdis.load.load_module_from_file_object
-    with NoLogging(NoLogging.Mode.STD_ERR):
-        version, timestamp, magic_int, codes, is_pypy, _, _ = load(MemoryFile(buffer), file_name, code_objects)
-    if not isinstance(codes, list):
-        codes = [codes]
-    for code in codes:
-        yield Code(version, timestamp, magic_int, code, is_pypy, code_objects)
-
-
-def disassemble_code(code: CodeType, version=None) -> Iterable[Instruction]:
-    opc = None
-    if version is not None:
-        if isinstance(version, float):
-            version = str(version)
-        if not isinstance(version, str):
-            version = xdis.version_info.version_tuple_to_str(version)
-        with contextlib.suppress(KeyError):
-            opc = xdis.op_imports.op_imports[version]
-    return xdis.std.Bytecode(code, opc=opc)
-
-
-def decompile_buffer(buffer: Union[Code, ByteStr], file_name: Optional[str] = None) -> ByteStr:
-    errors = ''
-    python = ''
-
-    if not isinstance(buffer, Code):
-        codes = list(extract_code_from_buffer(buffer, file_name))
-    else:
-        codes = [buffer]
-
-    def _engines():
-        nonlocal errors
-        try:
-            dc = decompyle3.main.decompile
-        except ImportError:
-            errors += '# The decompiler decompyle3 is not installed.\n'
-        else:
-            yield 'decompyle3', dc
-        try:
-            dc = uncompyle6.main.decompile
-        except ImportError:
-            errors += '# The decompiler decompyle3 is not installed.\n'
-        else:
-            yield 'uncompyle6', dc
-
-    engines = dict(_engines())
-
-    if not engines:
-        errors += '# (all missing, install one of the above to enable decompilation)'
-
-    for code in codes:
-        for name, decompile in engines.items():
-            with io.StringIO(newline='') as output, NoLogging(NoLogging.Mode.ALL):
-                try:
-                    decompile(
-                        co=code.container,
-                        bytecode_version=code.version,
-                        out=output,
-                        timestamp=code.timestamp,
-                        code_objects=code.code_objects,
-                        is_pypy=code.is_pypi,
-                        magic_int=code.magic,
-                    )
-                except Exception as E:
-                    errors += '\n'.join(F'# {line}' for line in (
-                        F'Error while decompiling with {name}:', *str(E).splitlines(True)))
-                    errors += '\n'
-                else:
-                    python = output.getvalue()
-                    break
-    if python:
-        # removes leading comments
-        python = python.splitlines(True)
-        python.reverse()
-        while python[-1].strip().startswith('#'):
-            python.pop()
-        python.reverse()
-        python = ''.join(python)
-        return python.encode('utf8')
-    if not isinstance(buffer, Code):
-        embedded = bytes(buffer | carve('printable', single=True))
-        if len(buffer) - len(embedded) < 0x20:
-            return embedded
-    disassembly = MemoryFile()
-    with io.TextIOWrapper(disassembly, 'utf8', newline='\n') as output:
-        output.write(errors)
-        output.write('# Generating Disassembly:\n\n')
-        for code in codes:
-            instructions = list(disassemble_code(code.container, code.version))
-            width_offset = max(len(str(i.offset)) for i in instructions)
-            for i in instructions:
-                opname = normalize_word_separators(i.opname, '.').lower()
-                offset = F'{i.offset:0{width_offset}d}'
-                output.write(F'# {offset:>5} {opname:<25} {i.argrepr}\n')
-        output.write('\n')
-    return disassembly.getvalue()
-
-
 class PiType(bytes, enum.Enum):
     BINARY          = B'b'  # noqa / binary
     DEPENDENCY      = B'd'  # noqa / runtime option
@@ -184,15 +63,15 @@ class PzType(enum.IntEnum):
 class PiMeta:
     type: PiType
     name: str
-    data: Union[Callable[[], ByteStr], ByteStr]
+    data: Callable[[], buf] | buf
 
-    def unpack(self) -> ByteStr:
+    def unpack(self) -> buf:
         if callable(self.data):
             self.data = self.data()
         return self.data
 
 
-def make_decompiled_item(name: str, data: ByteStr, *magics) -> PiMeta:
+def make_decompiled_item(name: str, data: buf, *magics) -> PiMeta:
 
     def extract(data=data, magics=magics):
         error = None
@@ -232,9 +111,9 @@ class PYZ(Struct):
         self.magic = magic + padding_size * b'\0'
         self.toc_offset = reader.i32()
         self.reader = reader
-        self.entries: List[PiMeta] = []
+        self.entries: list[PiMeta] = []
 
-    def unpack(self, decompile: bool, key: Optional[bytes] = None) -> bool:
+    def unpack(self, decompile: bool, key: bytes | None = None) -> bool:
         with StreamDetour(self.reader, self.base + self.toc_offset):
             toc_data = self.reader.read()
         try:
@@ -346,7 +225,7 @@ class PyInstallerArchiveEpilogue(Struct):
 
     MagicSignature = bytes.fromhex('4D45490C0B0A0B0E')
 
-    def _read_libname(self, reader: StructReader) -> Optional[str]:
+    def _read_libname(self, reader: StructReader) -> str | None:
         position = reader.tell()
         try:
             libname, t, rest = reader.read_bytes(64).partition(B'\0')
@@ -380,7 +259,7 @@ class PyInstallerArchiveEpilogue(Struct):
         self.py_libname = self._read_libname(reader)
         self.offset = reader.tell() - self.size
 
-        self.toc: Dict[str, PiTOCEntry] = {}
+        self.toc: dict[str, PiTOCEntry] = {}
         toc_end = self.offset + toc_offset + toc_length
         reader.seekset(self.offset + toc_offset)
         while reader.tell() < toc_end:
@@ -396,9 +275,9 @@ class PyInstallerArchiveEpilogue(Struct):
                 raise KeyError(F'duplicate name {entry.name}')
             self.toc[entry.name] = entry
 
-        self.files: Dict[str, PiMeta] = {}
+        self.files: dict[str, PiMeta] = {}
         no_pyz_found = True
-        pyz_entries: Dict[str, PYZ] = {}
+        pyz_entries: dict[str, PYZ] = {}
 
         for entry in list(self.toc.values()):
             if entry.type is not PiType.PYZ:
@@ -415,7 +294,7 @@ class PyInstallerArchiveEpilogue(Struct):
                 data = self.extract(entry.name).unpack()
             else:
                 data = reader
-            pyz_entries[name] = PYZ(data, self.py_version)
+            pyz_entries[name] = PYZ.Parse(data, self.py_version)
 
         magics = {pyz.magic for pyz in pyz_entries.values()}
 
@@ -430,7 +309,7 @@ class PyInstallerArchiveEpilogue(Struct):
             xtpyi.logger.warning('more than one magic signature was recovered; this is unusual.')
 
         magics = list(magics)
-        keys: Set[bytes] = set()
+        keys: set[bytes] = set()
 
         for entry in self.toc.values():
             extracted = self.extract(entry.name)
@@ -501,15 +380,15 @@ class xtpyi(ArchiveUnit, docs='{0}{s}{PathExtractorUnit}'):
     def __init__(
         self, *paths, list=False, join_path=False, drop_path=False, fuzzy=0, exact=False, regex=False,
         path=b'path', date=b'date',
-        decompile: Arg.Switch('-c', help='Attempt to decompile PYC files.') = False,
-        user_code: Arg.Switch('-u', group='FILTER', help=(
+        decompile: Param[bool, Arg.Switch('-c', help='Attempt to decompile PYC files.')] = False,
+        user_code: Param[bool, Arg.Switch('-u', group='FILTER', help=(
             'Extract only source code files from the root of the archive. These usually implement '
-            'the actual domain logic. This implies the --decompile option.')) = False,
-        unmarshal: Arg('-y', action='count', group='FILTER', help=(
+            'the actual domain logic. This implies the --decompile option.'))] = False,
+        unmarshal: Param[int, Arg('-y', action='count', group='FILTER', help=(
             '(DANGEROUS) Unmarshal embedded PYZ archives. Warning: Maliciously crafted packages can '
             'potentially exploit this to execute code. It is advised to only use this option inside '
             'an isolated environment. Specify twice to decompile unmarshalled Python bytecode.'
-        )) = 0
+        ))] = 0
     ):
         super().__init__(
             *paths,
@@ -541,7 +420,7 @@ class xtpyi(ArchiveUnit, docs='{0}{s}{PathExtractorUnit}'):
             self.log_warn(F'found {len(positions) - 1} potential PyInstaller epilogue markers; using last one.')
         decompile = self.args.decompile
         uc_target = PiType.USERCODE if decompile else PiType.SOURCE
-        archive = PyInstallerArchiveEpilogue(view, positions[-1], mode, decompile)
+        archive = PyInstallerArchiveEpilogue.Parse(view, positions[-1], mode, decompile)
         for name, file in archive.files.items():
             if self.args.user_code:
                 if file.type != uc_target:
@@ -551,5 +430,5 @@ class xtpyi(ArchiveUnit, docs='{0}{s}{PathExtractorUnit}'):
             yield self._pack(name, None, file.data, type=file.type.name)
 
     @classmethod
-    def handles(cls, data: ByteStr) -> Optional[bool]:
+    def handles(cls, data: buf) -> bool | None:
         return PyInstallerArchiveEpilogue.MagicSignature in data
