@@ -1,7 +1,7 @@
 from __future__ import annotations
 
-import tarfile
 import datetime
+import tarfile
 
 from refinery.lib.structures import MemoryFile
 from refinery.units.formats.archive import ArchiveUnit
@@ -21,17 +21,15 @@ class xttar(ArchiveUnit, docs='{0}{s}{PathExtractorUnit}'):
                     raise
                 stream.seek(ustar - 257)
                 archive = tarfile.open(fileobj=stream)
-        for info in archive.getmembers():
-            if not info.isfile():
-                continue
-            extractor = archive.extractfile(info)
-            if extractor is None:
-                continue
-            date = datetime.datetime.fromtimestamp(info.mtime)
-            yield self._pack(info.name, date, lambda e=extractor: e.read())
+            for info in archive.getmembers():
+                if not info.isfile():
+                    continue
+                extractor = archive.extractfile(info)
+                if extractor is None:
+                    continue
+                date = datetime.datetime.fromtimestamp(info.mtime)
+                yield self._pack(info.name, date, lambda e=extractor: e.read())
 
     @classmethod
-    def handles(cls, data: bytearray) -> bool:
-        if data[257:262] != B'ustar':
-            return False
-        return data[5:8] in (B'\x00\x30\x30', B'\x20\x20\x00')
+    def handles(cls, data) -> bool:
+        return data[257:262] == B'ustar'
