@@ -1,52 +1,30 @@
 from __future__ import annotations
 
+import email.utils
 import json
 import re
-import email.utils
 
-from typing import TYPE_CHECKING, Iterable, Tuple
 from email.parser import Parser
+from typing import TYPE_CHECKING, Iterable
 
+from refinery.lib.id import is_likely_email
+from refinery.lib.mime import file_extension
+from refinery.lib.tools import NoLogging, asbuffer, isbuffer
 from refinery.units.formats import PathExtractorUnit, UnpackResult
 from refinery.units.pattern.mimewords import mimewords
-from refinery.lib.mime import file_extension
-from refinery.lib.tools import NoLogging, isbuffer, asbuffer
 
 if TYPE_CHECKING:
     from extract_msg import Message
 
-
 CDFv2_MARKER = B'\xD0\xCF\x11\xE0\xA1\xB1\x1A\xE1'
 
 
-_EMAIL_TXT_MARKERS = [
-    b'\nReceived:\x20from'
-    b'\nSubject:\x20',
-    b'\nTo:\x20',
-    b'\nFrom:\x20',
-    B'\nMessage-ID:\x20',
-    b'\nBcc:\x20',
-    b'\nContent-Transfer-Encoding:\x20',
-    b'\nContent-Type:\x20',
-    b'\nReturn-Path:\x20',
-]
-
-_EMAIL_BIN_MARKERS = [
-    marker.encode('utf-16le') for marker in (
-        "__nameid_version"        # root node
-        "__recip_version"         # recipients
-        "__properties_version"    # properties
-        "__substg1.0_"            # strings
-    )
-]
-
-
 class xtmail(PathExtractorUnit):
     """
     Extract files and body from EMail messages. The unit supports both the Outlook message format
     and regular MIME documents.
     """
-    def _get_headparts(self, head: Iterable[Tuple[str, str]]):
+    def _get_headparts(self, head: Iterable[tuple[str, str]]):
         def normalize_spaces(value: str):
             return ''.join(re.sub(R'\A\s+', '\x20', t) for t in value.splitlines(False))
 
@@ -243,18 +221,5 @@ class xtmail(PathExtractorUnit):
             yield from self._get_parts_regular(data)
 
     @classmethod
-    def handles(cls, data: bytearray) -> bool:
-        counter = 0
-        if data.startswith(CDFv2_MARKER):
-            markers = _EMAIL_BIN_MARKERS
-            threshold = 1
-        else:
-            markers = _EMAIL_TXT_MARKERS
-            threshold = 2
-        for marker in markers:
-            if re.search(re.escape(marker), data, flags=re.IGNORECASE):
-                counter += 1
-            if counter >= threshold:
-                return True
-        else:
-            return False
+    def handles(cls, data) -> bool:
+        return is_likely_email(data)
