@@ -4,15 +4,16 @@ The code is based on the logic implemented in IFPSTools:
 """
 from __future__ import annotations
 
-from refinery.units.formats import Unit
 from refinery.lib.inno.ifps import IFPSFile
+from refinery.lib.types import Param
+from refinery.units.formats import Arg, Unit
 
 
 class IFPSBase(Unit, abstract=True):
     def __init__(
         self,
-        codec: Unit.Arg.String(
-            help='Optionally specify the string encoding. The default is "{default}".') = 'cp1252',
+        codec: Param[str, Arg.String(
+            help='Optionally specify the string encoding. The default is "{default}".')] = 'cp1252',
         **more
     ):
         super().__init__(codec=codec, **more)
@@ -25,13 +26,13 @@ class ifps(IFPSBase):
     """
     def __init__(
         self,
-        bytes: IFPSBase.Arg.Switch('-b', help='Print opcode bytes in the disassembly.'),
+        bytes: Param[bool, Arg.Switch('-b', help='Print opcode bytes in the disassembly.')],
         codec='cp1252'
     ):
         super().__init__(codec=codec, bytes=bytes)
 
     def process(self, data):
-        return IFPSFile(data, self.args.codec).disassembly(self.args.bytes).encode(self.codec)
+        return IFPSFile.Parse(data, self.args.codec).disassembly(self.args.bytes).encode(self.codec)
 
     @classmethod
     def handles(cls, data) -> bool:
