@@ -1,10 +1,11 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING
 from itertools import islice
+from typing import TYPE_CHECKING
 
-from refinery.units import Unit
+from refinery.lib.id import get_image_format
 from refinery.lib.structures import MemoryFile
+from refinery.units import Unit
 
 if TYPE_CHECKING:
     from PIL.Image import Image
@@ -33,9 +34,11 @@ class imgdb(Unit):
             raise ValueError('input could not be parsed as an image')
         test = image.getpixel((0, 0))
         if isinstance(test, int):
+            self.log_info('reading each pixel as an integer')
             for row in self._get_rows(image):
                 yield bytearray(row)
         else:
+            self.log_info('reading each pixel as a color value tuple')
             count = len(test)
             total = count * image.width
             out = bytearray(total)
@@ -43,3 +46,8 @@ class imgdb(Unit):
                 for pixel, offset in zip(row, range(0, total, count)):
                     out[offset:offset + count] = pixel
             yield out
+
+    @classmethod
+    def handles(cls, data) -> bool | None:
+        if get_image_format(data) is not None:
+            return True
