@@ -1,10 +1,12 @@
 from __future__ import annotations
 
-import io
 import enum
+import io
 
-from refinery.units import Unit, Arg
+from refinery.lib.id import get_image_format
 from refinery.lib.structures import MemoryFile
+from refinery.lib.types import Param
+from refinery.units import Arg, Unit
 
 
 class T(enum.IntEnum):
@@ -26,7 +28,7 @@ class imgtp(Unit):
     """
     def __init__(
         self,
-        transformation: Arg.String(help='The transformation sequence; default is {default}.') = 'R'
+        transformation: Param[str, Arg.String(help='The transformation sequence; default is {default}.')] = 'R'
     ):
         transformation = [Arg.AsOption(t, T) for t in transformation]
         super().__init__(transformation=transformation)
@@ -55,3 +57,8 @@ class imgtp(Unit):
         with io.BytesIO() as out:
             image.save(out, format)
             return out.getvalue()
+
+    @classmethod
+    def handles(cls, data) -> bool | None:
+        if get_image_format(data) is not None:
+            return True
