@@ -1,7 +1,7 @@
 from __future__ import annotations
 
-from refinery.units import Unit
 from refinery.lib.java import JvClassFile
+from refinery.units import Unit
 
 
 class jvstr(Unit):
@@ -9,7 +9,7 @@ class jvstr(Unit):
     Extract string constants from Java class files.
     """
     def process(self, data):
-        jc = JvClassFile(data)
+        jc = JvClassFile.Parse(data)
         for string in jc.strings:
             yield string.encode(self.codec)
 
