@@ -1,10 +1,11 @@
 from __future__ import annotations
 
-from refinery.units.formats import Unit
-from refinery.units.sinks.ppjson import ppjson
-from refinery.lib.structures import MemoryFile
 from refinery.lib.json import JSONEncoderEx
+from refinery.lib.structures import MemoryFile
 from refinery.lib.tools import NoLogging
+from refinery.lib.types import Param
+from refinery.units.formats import Arg, Unit
+from refinery.units.sinks.ppjson import ppjson
 
 
 class lnk(Unit):
@@ -26,8 +27,8 @@ class lnk(Unit):
 
     def __init__(
         self,
-        tabular: Unit.Arg('-t', help='Print information in a table rather than as JSON.') = False,
-        details: Unit.Arg('-d', help='Print all details; some properties are hidden by default.') = False,
+        tabular: Param[bool, Arg('-t', help='Print information in a table rather than as JSON.')] = False,
+        details: Param[bool, Arg('-d', help='Print all details; some properties are hidden by default.')] = False,
     ):
         super().__init__(tabular=tabular, details=details)
 
@@ -53,3 +54,7 @@ class lnk(Unit):
             pp = ppjson(tabular=self.args.tabular)
             yield from pp._pretty_output(
                 parsed, indent=4, cls=encoder, ensure_ascii=False)
+
+    @classmethod
+    def handles(cls, data):
+        return data[:20] == B'L\0\0\0\01\x14\02\0\0\0\0\0\xC0\0\0\0\0\0\0F'
