@@ -1,5 +1,4 @@
 from __future__ import annotations
-from typing import List, Dict, NamedTuple, Union, Optional
 
 import codecs
 import collections
@@ -9,16 +8,16 @@ import re
 import struct
 
 from functools import cached_property
+from typing import NamedTuple
 
-from refinery.lib.structures import StructReader
-from refinery.units import Arg
-from refinery.units.formats.office.xtdoc import xtdoc, UnpackResult
 from refinery.lib import chunks
-from refinery.lib.types import ByteStr, JSONDict
-from refinery.lib.mime import FileMagicInfo
 from refinery.lib.cab import Cabinet
-
+from refinery.lib.id import buffer_offset, is_likely_msi, is_likely_pe
+from refinery.lib.structures import StructReader
+from refinery.lib.types import Param, buf
+from refinery.units import Arg
 from refinery.units.formats.csv import csv
+from refinery.units.formats.office.xtdoc import UnpackResult, xtdoc
 
 
 class MsiType(enum.IntEnum):
@@ -87,17 +86,21 @@ class MSITableColumnInfo(NamedTuple):
             return 'H'
 
 
+_NEGATED_MSI_SIGNATURE = B'\x2F\x30\xEE\x1F\x5E\x4E\xE5\x1E'
+
+
 class MSIStringData:
-    def __init__(self, string_data: ByteStr, string_pool: ByteStr):
+    def __init__(self, string_data: buf, string_pool: buf):
         data = StructReader(string_data)
         pool = StructReader(string_pool)
-        self.strings: List[bytes] = []
-        self.provided_ref_count: List[int] = []
-        self.computed_ref_count: List[int] = []
+        self.strings: list[bytes] = []
+        self.provided_ref_count: list[int] = []
+        self.computed_ref_count: list[int] = []
         self.codepage = pool.u16()
         self._unknown = pool.u16()
         while not pool.eof:
-            size, rc = pool.read_struct('<HH')
+            size = pool.u16()
+            rc = pool.u16()
             if size == 0 and rc != 0:
                 size = pool.u32()
             string = data.read_bytes(size)
@@ -122,14 +125,16 @@ class MSIStringData:
     def __contains__(self, index):
         return 0 < index <= len(self)
 
-    def ref(self, index: int, increment=True) -> Union[str, bytes]:
+    def ref(self, index: int, increment=True) -> str:
         assert index > 0
         index -= 1
         if increment:
             self.computed_ref_count[index] += 1
-        data = self.strings[index]
-        data = data.decode(self.codec)
-        return data
+        string = self.strings[index]
+        try:
+            return string.decode(self.codec)
+        except UnicodeDecodeError:
+            return string.decode('latin1')
 
 
 class xtmsi(xtdoc):
@@ -165,9 +170,10 @@ class xtmsi(xtdoc):
     }
 
     def __init__(
-            self, *paths,
-            list=False, path=b'path', join_path=False, drop_path=False, fuzzy=0, exact=False, regex=False,
-            nocab: Arg.Switch('-N', help='Do not list and extract embedded CAB archives.') = False, **kw,
+        self, *paths,
+        list=False, path=b'path', join_path=False, drop_path=False, fuzzy=0, exact=False, regex=False,
+        nocab: Param[bool, Arg.Switch('-N', help='Do not list and extract embedded CAB archives.')] = False,
+        **keywords,
     ):
         super().__init__(
             *paths,
@@ -179,27 +185,30 @@ class xtmsi(xtdoc):
             fuzzy=fuzzy,
             exact=exact,
             regex=regex,
-            **kw,
+            **keywords,
         )
 
-    def unpack(self, data):
-        streams = {result.path: result for result in super().unpack(data)}
+    def unpack(self, data: buf):
+        streams = {
+            result.path: result
+            for result in super().unpack(self.get_msi_from_overlay(data))
+        }
 
         def stream(name: str):
             return streams.pop(name).get_data()
 
-        def column_formats(table: Dict[str, MSITableColumnInfo]) -> str:
+        def column_formats(table: dict[str, MSITableColumnInfo]) -> str:
             return ''.join(v.struct_format for v in table.values())
 
-        def stream_to_rows(data: ByteStr, row_format: str):
+        def stream_to_rows(data: buf, row_format: str):
             row_size = struct.calcsize(F'<{row_format}')
             row_count = int(len(data) / row_size)
             reader = StructReader(data)
             columns = [reader.read_struct(F'<{sc * row_count}') for sc in row_format]
             for i in range(row_count):
-                yield [c[i] for c in columns]
+                yield [int(c[i]) for c in columns]
 
-        tables: Dict[str, Dict[str, MSITableColumnInfo]] = collections.defaultdict(collections.OrderedDict)
+        tables: dict[str, dict[str, MSITableColumnInfo]] = collections.defaultdict(collections.OrderedDict)
         strings = MSIStringData(stream('!_StringData'), stream('!_StringPool'))
 
         for tbl_name_id, col_number, col_name_id, col_attributes in stream_to_rows(stream('!_Columns'), 'HHHH'):
@@ -217,18 +226,19 @@ class xtmsi(xtdoc):
 
         class ScriptItem(NamedTuple):
             row_index: int
-            extension: Optional[str]
+            extension: str | None
 
-        processed_table_data: Dict[str, List[Dict[str, str]]] = {}
-        tbl_properties: Dict[str, str] = {}
-        tbl_files: Dict[str, str] = {}
-        tbl_components: Dict[str, str] = {}
-        postprocessing: List[ScriptItem] = []
+        processed_table_data: dict[str, list[dict[str, str]]] = {}
+        tbl_properties: dict[str, str] = {}
+        tbl_files: dict[str, str] = {}
+        tbl_components: dict[str, str] = {}
+        postprocessing: list[ScriptItem] = []
 
         def format_string(string: str):
             # https://learn.microsoft.com/en-us/windows/win32/msi/formatted
             def _replace(match: re.Match[str]):
-                _replace.done = False
+                nonlocal _replace_done
+                _replace_done = False
                 prefix, name = match.groups()
                 if not prefix:
                     tbl = tbl_properties
@@ -243,14 +253,14 @@ class xtmsi(xtdoc):
                     raise ValueError
                 return tbl.get(name, '')
             while True:
-                _replace.done = True
+                _replace_done = True
                 string = re.sub(R'''(?x)
                     \[             # open square bracket
                       (?![~\\])    # not followed by escapes
                       ([%$!#]?)    # any of the valid prefix characters
                       ([^[\]{}]+)  # no brackets or braces
                     \]''', _replace, string)
-                if _replace.done:
+                if _replace_done:
                     break
             string = re.sub(r'\[\\(.)\]', r'\1', string)
             string = string.replace('[~]', '\0')
@@ -311,38 +321,40 @@ class xtmsi(xtdoc):
             if processed:
                 processed_table_data[table_name] = processed
 
-        ca = processed_table_data.get('CustomAction', None)
-        for item in postprocessing:
-            entry = ca[item.row_index]
-            try:
-                path: str = entry['Action']
-                data: str = entry['Target']
-            except KeyError:
-                continue
-            root = F'Action/{path}'
-            if item.extension:
-                path = F'{root}.{item.extension}'
-                streams[path] = UnpackResult(path, data.encode(self.codec))
-                continue
-            data = format_string(data)
-            parts = [part.partition('\x02') for part in data.split('\x01')]
-            if not all(part[1] == '\x02' for part in parts):
-                continue
-            for name, _, script in parts:
-                if not name.lower().startswith('script'):
+        if ca := processed_table_data.get('CustomAction'):
+            for item in postprocessing:
+                entry = ca[item.row_index]
+                try:
+                    action: str = entry['Action']
+                    target: str = entry['Target']
+                except KeyError:
                     continue
-                if not script:
+                root = F'Action/{action}'
+                if item.extension:
+                    action = F'{root}.{item.extension}'
+                    streams[action] = UnpackResult(action, target.encode(self.codec))
                     continue
-                path = F'{root}.{name}'
-                streams[path] = UnpackResult(path, script.encode(self.codec))
+                target = format_string(target)
+                parts = [part.partition('\x02') for part in target.split('\x01')]
+                if not all(part[1] == '\x02' for part in parts):
+                    continue
+                for name, _, script in parts:
+                    if not name.lower().startswith('script'):
+                        continue
+                    if not script:
+                        continue
+                    action = F'{root}.{name}'
+                    streams[action] = UnpackResult(action, script.encode(self.codec))
 
         for ignored_stream in [
-            '[5]SummaryInformation',
-            '[5]DocumentSummaryInformation',
-            '[5]DigitalSignature',
-            '[5]MsiDigitalSignatureEx'
+            'SummaryInformation',
+            'DocumentSummaryInformation',
+            'DigitalSignature',
+            'MsiDigitalSignatureEx'
         ]:
-            streams.pop(ignored_stream, None)
+            if r := streams.pop(F'[5]{ignored_stream}', None):
+                r.path = F'Meta/{ignored_stream}'
+                yield r
 
         inconsistencies = 0
         w1 = len(str(len(strings)))
@@ -366,13 +378,13 @@ class xtmsi(xtdoc):
         else:
             def _iscab(path):
                 return media_info and any(item.get('Cabinet', '') == F'#{path}' for item in media_info)
-            media_info: List[JSONDict] = processed_table_data.get('Media', [])
-            cabs: Dict[str, UnpackResult] = {
+            media_info = processed_table_data.get('Media', [])
+            cabs: dict[str, UnpackResult] = {
                 path: item for path, item in streams.items() if _iscab(path)}
             for cab in cabs:
                 self.log_info(F'found cab file: {cab}')
         if cabs:
-            file_names: Dict[str, JSONDict] = {}
+            file_names: dict[str, str] = {}
 
             for file_info in processed_table_data.get('File', []):
                 try:
@@ -386,7 +398,7 @@ class xtmsi(xtdoc):
 
             for path, cab in cabs.items():
                 try:
-                    _cabinet = Cabinet(cab.get_data())
+                    _cabinet = Cabinet(memoryview(cab.get_data()))
                     unpacked = _cabinet.process().get_files()
                 except Exception as e:
                     self.log_info(F'unable to extract embedded cab file: {e!s}')
@@ -409,12 +421,12 @@ class xtmsi(xtdoc):
         streams[ds.path] = ds
 
         converter = csv()
-        for key, data in processed_table_data.items():
+        for key, jd in processed_table_data.items():
             sk = key.strip('_')
             if sk not in processed_table_data:
                 key = sk
             try:
-                tbl = UnpackResult(F'{self._SYNTHETIC_STREAMS_TOPLEVEL}/{key}.csv', converter.json_to_csv(data))
+                tbl = UnpackResult(F'{self._SYNTHETIC_STREAMS_TOPLEVEL}/{key}.csv', converter.json_to_csv(jd))
             except Exception:
                 continue
             streams[tbl.path] = tbl
@@ -424,10 +436,36 @@ class xtmsi(xtdoc):
             yield streams[path]
 
     @classmethod
-    def handles(cls, data: bytearray):
-        if not data[:4] == B'\xD0\xCF\x11\xE0':
-            return False
-        return FileMagicInfo(data).extension == 'msi'
+    def get_msi_from_overlay(cls, data: buf) -> buf:
+        if is_likely_pe(data):
+            from refinery.units.formats.pe import get_pe_size
+            view = memoryview(data)
+            overlay = view[get_pe_size(data):]
+            if is_likely_msi(overlay):
+                return overlay
+            if (start := buffer_offset(overlay, _NEGATED_MSI_SIGNATURE, 0, 0x1000)) >= 0:
+                if (nulls := buffer_offset(overlay, bytes(8), start, start + 0x1000)) >= 0:
+                    from refinery.units.blockwise.neg import neg
+                    decoded = overlay[start:nulls] | neg | bytearray
+                    decoded.extend(overlay[nulls:])
+                    if is_likely_msi(decoded):
+                        return decoded
+        return data
+
+    @classmethod
+    def handles(cls, data):
+        if is_likely_msi(data):
+            return True
+        if is_likely_pe(data):
+            from refinery.units.formats.pe import get_pe_size
+            view = memoryview(data)
+            overlay = view[get_pe_size(data):]
+            if is_likely_msi(overlay):
+                return True
+            if overlay[:8] == _NEGATED_MSI_SIGNATURE:
+                return True
+        return False
 
 
-xtmsi.__doc__ = xtmsi.__doc__.format(FN=xtmsi._SYNTHETIC_STREAMS_FILENAME)
+if _d := xtmsi.__doc__:
+    xtmsi.__doc__ = _d.format(FN=xtmsi._SYNTHETIC_STREAMS_FILENAME)
