@@ -1,11 +1,10 @@
 from __future__ import annotations
 
-from typing import Optional
-from uuid import UUID
+import re
 
-from refinery.units.formats import PathExtractorUnit, UnpackResult
-from refinery.lib.structures import MemoryFile
 from refinery.lib.mime import get_cached_file_magic_info
+from refinery.lib.structures import MemoryFile
+from refinery.units.formats import PathExtractorUnit, UnpackResult
 
 
 class xtone(PathExtractorUnit):
@@ -30,5 +29,7 @@ class xtone(PathExtractorUnit):
             yield UnpackResult(F'{guid}{extension}', chunk)
 
     @classmethod
-    def handles(cls, data: bytearray) -> Optional[bool]:
-        return UUID('e4525c7b-8cd8-a74d-aeb1-5378d02996d3').bytes in data
+    def handles(cls, data) -> bool | None:
+        return re.search(
+            br'\xE4\x52\x5C\x7B\x8C\xD8\xA7\x4D\xAE\xB1\x53\x78\xD0\x29\x96\xD3', data
+        ) is not None
