@@ -1,18 +1,20 @@
 from __future__ import annotations
 
-from typing import List, Union, TYPE_CHECKING
-
-from refinery.units import Arg, Unit
-from refinery.lib.vfs import VirtualFileSystem, VirtualFile
-from refinery.lib.structures import MemoryFile
-from refinery.lib.tools import NoLogging
-
 import dataclasses
 
+from typing import TYPE_CHECKING, Union
+
+from refinery.lib.structures import MemoryFile
+from refinery.lib.tools import NoLogging
+from refinery.lib.types import Param
+from refinery.lib.vfs import VirtualFile, VirtualFileSystem
+from refinery.units import Arg, Unit
+
 if TYPE_CHECKING:
     from ipaddress import IPv4Address, IPv6Address
-    from pcapkit.foundation.reassembly.data.tcp import Datagram, DatagramID
+
     from pcapkit.foundation.extraction import Packet
+    from pcapkit.foundation.reassembly.data.tcp import Datagram, DatagramID
     TIPAddr = Union[IPv4Address, IPv6Address]
 
 
@@ -43,10 +45,6 @@ class Conversation:
             (self.src, self.src_port),
             (self.dst, self.dst_port))))
 
-    def swapped(self):
-        cls = self.__class__
-        return cls(self.dst_addr, self.src_addr, self.dst_port, self.src_port)
-
     def __eq__(self, other):
         return hash(self) == hash(other)
 
@@ -72,9 +70,9 @@ class pcap(Unit):
 
     def __init__(
         self,
-        merge: Arg.Switch('-m', help='Merge both parts of each TCP conversation into one chunk.') = False,
-        client: Arg.Switch('-c', group='D', help='Show only the client part of each conversation.') = False,
-        server: Arg.Switch('-s', group='D', help='Show only the server part of each conversation.') = False,
+        merge: Param[bool, Arg.Switch('-m', help='Merge both parts of each TCP conversation into one chunk.')] = False,
+        client: Param[bool, Arg.Switch('-c', group='D', help='Show only the client part of each conversation.')] = False,
+        server: Param[bool, Arg.Switch('-s', group='D', help='Show only the server part of each conversation.')] = False,
     ):
         super().__init__(merge=merge, client=client, server=server)
 
@@ -100,7 +98,7 @@ class pcap(Unit):
             vf = VirtualFile(fs, data, 'pcap')
             pcap = pcapkit.extract(
                 fin=vf.path,
-                engine=pcapkit.Scapy,
+                engine='scapy',
                 store=True,
                 nofile=True,
                 extension=False,
@@ -109,7 +107,7 @@ class pcap(Unit):
                 reassembly=True,
                 reasm_strict=True,
             )
-            tcp: List[Datagram] = list(pcap.reassembly.tcp)
+            tcp: list[Datagram] = list(pcap.reassembly.tcp)
             tcp.sort(key=lambda p: min(p.index, default=0))
 
         count, convo = 0, None
@@ -120,15 +118,13 @@ class pcap(Unit):
         PT = self._scapy.packet
 
         def payload(packet: Packet):
-            ok = (bytes, bytearray, PT.Raw)
-            no = (PT.NoPayload, PT.Padding)
             circle = set()
             while True:
                 try:
                     inner = packet.payload
                 except AttributeError:
                     break
-                if isinstance(packet, ok) and not isinstance(packet, no):
+                if isinstance(packet, PT.Raw) and not isinstance(packet, (PT.NoPayload, PT.Padding)):
                     return packet.original
                 if id(inner) in circle:
                     break
@@ -155,21 +151,25 @@ class pcap(Unit):
         def commit():
             if src_buffer.tell():
                 if not server:
+                    assert convo is not None
                     yield self.labelled(src_buffer.getvalue(), **convo.src_to_dst())
                 src_buffer.truncate(0)
             if dst_buffer.tell():
                 if not client:
+                    assert convo is not None
                     yield self.labelled(dst_buffer.getvalue(), **convo.dst_to_src())
                 dst_buffer.truncate(0)
 
         for datagram in tcp:
+            self.log_info(datagram.header)
+
             this_convo = Conversation.FromID(datagram.id)
             if this_convo != convo:
                 if count and merge:
                     yield from commit()
                 count = count + 1
                 convo = this_convo
-
+            assert convo is not None
             data = bytearray()
             for index in sorted(datagram.index, key=sequence):
                 data.extend(payload(pcap.frame[index - 1]))
