@@ -3,13 +3,14 @@ A package containing Portable Executable (PE) file related units.
 """
 from __future__ import annotations
 
-from typing import Union, ByteString
-from refinery.units import Arg, Unit
 from refinery.lib import lief
+from refinery.lib.tools import NoLoggingProxy
+from refinery.lib.types import Param, buf
+from refinery.units import Arg, Unit
 
 
 def get_pe_size(
-    pe: Union[lief.PE.Binary, ByteString],
+    pe: buf | lief.PE.Binary,
     overlay=True,
     sections=True,
     directories=True,
@@ -20,8 +21,8 @@ def get_pe_size(
     This fuction determines the size of a PE file, optionally taking into account the PE overlay
     computation, section information, data directory information, and certificate entries.
     """
-    if not hasattr(pe, 'overlay_offset'):
-        pe = lief.load_pe(pe)
+    if not isinstance(pe, (NoLoggingProxy, lief.PE.Binary)):
+        pe = lief.load_pe_fast(pe)
 
     overlay_value = overlay and pe.overlay_offset or 0
 
@@ -41,7 +42,7 @@ def get_pe_size(
         cert_entry = None
         certificate = False
 
-    if certificate:
+    if certificate and cert_entry:
         # The certificate overlay is given as a file offset
         # rather than a virtual address.
         cert_value = cert_entry.rva + cert_entry.size
@@ -69,11 +70,11 @@ def get_pe_size(
 class OverlayUnit(Unit, abstract=True):
     def __init__(
         self,
-        certificate: Arg.Switch('--cert', '-c',
-            help='Include digital signatures for the size computation.') = False,
-        directories: Arg.Switch('--dirs', '-d',
-            help='Include data directories for size computation.') = False,
-        memdump: Arg.Switch('-m', help='Assume that the file data was a memory-mapped PE file.') = False,
+        certificate: Param[bool, Arg.Switch('--cert', '-c',
+            help='Include digital signatures for the size computation.')] = False,
+        directories: Param[bool, Arg.Switch('--dirs', '-d',
+            help='Include data directories for size computation.')] = False,
+        memdump: Param[bool, Arg.Switch('-m', help='Assume that the file data was a memory-mapped PE file.')] = False,
         **other
     ):
         super().__init__(certificate=certificate, directories=directories, memdump=memdump, **other)
