@@ -1,7 +1,7 @@
 from __future__ import annotations
 
-from refinery.units import Unit
 from refinery.lib.dotnet.header import DotNetHeader
+from refinery.units import Unit
 
 
 class dnblob(Unit):
@@ -10,8 +10,7 @@ class dnblob(Unit):
     """
     def process(self, data):
         header = DotNetHeader(data, parse_resources=False)
-        for blob in header.meta.Streams.Blob.values():
-            yield blob
+        yield from header.meta.Streams.Blob.values()
 
     @classmethod
     def handles(cls, data):
