@@ -3,13 +3,13 @@ from __future__ import annotations
 import re
 import struct
 
-from typing import NamedTuple, Optional, Dict, Tuple
 from collections import Counter
+from typing import NamedTuple
 
-from refinery.units.formats import PathExtractorUnit, UnpackResult
-from refinery.units.formats.pe.dotnet import CodePath
 from refinery.lib.dotnet import integer_from_ldc
 from refinery.lib.dotnet.header import DotNetHeader
+from refinery.units.formats import PathExtractorUnit, UnpackResult
+from refinery.units.formats.pe.dotnet import CodePath
 
 
 class FieldInfo(NamedTuple):
@@ -41,16 +41,16 @@ class dnfields(PathExtractorUnit):
         if not fields:
             return
 
-        icache: Dict[bytes, FieldInfo] = {}
+        icache: dict[bytes, FieldInfo] = {}
         memory = memoryview(data)
 
-        def _guess_field_info(t: int, signature: bytes, field_name: Optional[str] = None, sizemap: dict = {
+        def _guess_field_info(t: int, signature: bytes, field_name: str | None = None, sizemap: dict = {
             '^s?byte$'       : 1,
             '^s?char$'       : 2,
             '^[us]?int.?16$' : 2,
             '^[us]?int.?32$' : 4,
             '^[us]?int.?64$' : 8,
-        }) -> Tuple[Optional[str], FieldInfo]:
+        }) -> tuple[str | None, FieldInfo]:
             try:
                 info = icache[signature]
             except KeyError:
@@ -105,7 +105,7 @@ class dnfields(PathExtractorUnit):
         remaining_field_indices = set(range(len(tables.Field)))
 
         unpack = []
-        name_count = Counter(tables.Field[rv.Field.Index - 1].Name for rv in fields)
+        name_count = Counter(tables[rv.Field].Name for rv in fields)
         name_width = len(str(len(fields)))
 
         for k, rv in enumerate(fields):
@@ -116,7 +116,7 @@ class dnfields(PathExtractorUnit):
                 continue
             fname = field.Name
             type = None
-            signature: bytes = field.Signature
+            signature = bytes(field.Signature)
             offset = header.pe.rva_to_offset(rv.RVA)
 
             if len(signature) == 2:
