@@ -1,8 +1,10 @@
 from __future__ import annotations
 
+from refinery.lib.dotnet.header import DotNetHeader
+from refinery.lib.structures import struct_to_json
+from refinery.lib.types import Param
 from refinery.units import Arg
 from refinery.units.formats.pe.dotnet import DotNetJSONEncoderUnit
-from refinery.lib.dotnet.header import DotNetHeader
 
 
 class dnhdr(DotNetJSONEncoderUnit):
@@ -12,22 +14,20 @@ class dnhdr(DotNetJSONEncoderUnit):
     """
     def __init__(
         self,
-        resources: Arg.Switch('-r', '--resources', help='Also parse .NET resources.') = False,
+        resources: Param[bool, Arg.Switch('-r', '--resources', help='Also parse .NET resources.')] = False,
         encode=None, digest=None, arrays=False,
     ):
         super().__init__(encode=encode, digest=digest, arrays=arrays, resources=resources)
 
     def process(self, data):
         dn = DotNetHeader(data, parse_resources=self.args.resources)
-        dn = {
-            'Head': dn.head,
-            'Meta': dn.meta
+        result = {
+            'Head': struct_to_json(dn.head),
+            'Meta': struct_to_json(dn.meta),
         }
-
         if self.args.resources:
-            dn['RSRC'] = dn.resources
-
-        return self.to_json(dn)
+            result['RSRC'] = struct_to_json(dn.resources)
+        return self.to_json(result)
 
     @classmethod
     def handles(cls, data):
