@@ -1,9 +1,12 @@
 from __future__ import annotations
 
-from refinery.units.formats import Arg, PathExtractorUnit, UnpackResult
-from refinery.units import RefineryPartialResult
-from refinery.lib.tools import isbuffer
+import codecs
+
 from refinery.lib.dotnet.resources import NetStructuredResources, NoManagedResource
+from refinery.lib.tools import asbuffer
+from refinery.lib.types import Param
+from refinery.units import RefineryPartialResult
+from refinery.units.formats import Arg, PathExtractorUnit, UnpackResult
 
 
 class dnmr(PathExtractorUnit):
@@ -12,7 +15,7 @@ class dnmr(PathExtractorUnit):
     """
     def __init__(
         self, *paths, list=False, join_path=False, drop_path=False, exact=False, fuzzy=0, regex=False, path=b'name',
-        raw: Arg.Switch('-w', help='Do not deserialize the managed resource entry data.') = False
+        raw: Param[bool, Arg.Switch('-w', help='Do not deserialize the managed resource entry data.')] = False
     ):
         super().__init__(
             *paths,
@@ -38,10 +41,10 @@ class dnmr(PathExtractorUnit):
                 self.log_warn(F'entry {entry.Name} carried error message: {entry.Error}')
             data = entry.Data
             if not self.args.raw:
-                if isinstance(entry.Value, str):
-                    data = entry.Value.encode('utf-16le')
-                elif isbuffer(entry.Value):
-                    data = entry.Value
+                if b := asbuffer(v := entry.Value):
+                    data = b
+                elif isinstance(v, str):
+                    data = codecs.encode(v, self.codec)
             yield UnpackResult(entry.Name, data)
 
     @classmethod
