@@ -4,19 +4,20 @@ import itertools
 import json
 
 from contextlib import suppress
-from datetime import datetime, timedelta, timezone
 from dataclasses import dataclass
+from datetime import datetime, timedelta, timezone
 from enum import Enum
 
 from refinery.lib import lief
 from refinery.lib.dotnet.header import DotNetHeader
-from refinery.lib.tools import NoLoggingProxy
-from refinery.units import Arg, Unit
-from refinery.units.sinks.ppjson import ppjson
-from refinery.units.formats.pe import get_pe_size
-from refinery.lib.tools import date_from_timestamp
+from refinery.lib.id import is_likely_pe
 from refinery.lib.lcid import LCID
 from refinery.lib.resources import datapath
+from refinery.lib.tools import NoLoggingProxy, date_from_timestamp, unwrap
+from refinery.lib.types import Param
+from refinery.units import Arg, Unit
+from refinery.units.formats.pe import get_pe_size
+from refinery.units.sinks.ppjson import ppjson
 
 
 def _FILETIME(value: int) -> datetime:
@@ -151,18 +152,29 @@ class pemeta(Unit):
     extracted.
     """
     def __init__(
-        self, custom : Arg('-c', '--custom',
-            help='Unless enabled, all default categories will be extracted.') = False,
-        debug      : Arg.Switch('-D', help='Parse the PDB path from the debug directory.') = False,
-        dotnet     : Arg.Switch('-N', help='Parse the .NET header.') = False,
-        signatures : Arg.Switch('-S', help='Parse digital signatures.') = False,
-        timestamps : Arg.Counts('-T', help='Extract time stamps. Specify twice for more detail.') = 0,
-        version    : Arg.Switch('-V', help='Parse the VERSION resource.') = False,
-        header     : Arg.Switch('-H', help='Parse base data from the PE header.') = False,
-        exports    : Arg.Counts('-E', help='List all exported functions. Specify twice to include addresses.') = 0,
-        imports    : Arg.Counts('-I', help='List all imported functions. Specify twice to include addresses.') = 0,
-        tabular    : Arg.Switch('-t', help='Print information in a table rather than as JSON') = False,
-        timeraw    : Arg.Switch('-r', help='Extract time stamps as numbers instead of human-readable format.') = False,
+        self,
+        custom: Param[bool, Arg('-c', '--custom',
+            help='Unless enabled, all default categories will be extracted.')] = False,
+        debug: Param[bool, Arg.Switch('-D',
+            help='Parse the PDB path from the debug directory.')] = False,
+        dotnet: Param[bool, Arg.Switch('-N',
+            help='Parse the .NET header.')] = False,
+        signatures: Param[bool, Arg.Switch('-S',
+            help='Parse digital signatures.')] = False,
+        timestamps: Param[int, Arg.Counts('-T',
+            help='Extract time stamps. Specify twice for more detail.')] = 0,
+        version: Param[bool, Arg.Switch('-V',
+            help='Parse the VERSION resource.')] = False,
+        header: Param[bool, Arg.Switch('-H',
+            help='Parse base data from the PE header.')] = False,
+        exports: Param[int, Arg.Counts('-E',
+            help='List all exported functions. Specify twice to include addresses.')] = 0,
+        imports: Param[int, Arg.Counts('-I',
+            help='List all imported functions. Specify twice to include addresses.')] = 0,
+        tabular: Param[bool, Arg.Switch('-t',
+            help='Print information in a table rather than as JSON')] = False,
+        timeraw: Param[bool, Arg.Switch('-r',
+            help='Extract time stamps as numbers instead of human-readable format.')] = False,
     ):
         if not custom and not any((debug, dotnet, signatures, timestamps, version, header)):
             debug = dotnet = signatures = timestamps = version = header = True
@@ -181,7 +193,7 @@ class pemeta(Unit):
 
     @classmethod
     def handles(cls, data):
-        return data[:2] == B'MZ'
+        return is_likely_pe(data)
 
     @classmethod
     def _ensure_string(cls, x):
@@ -190,11 +202,10 @@ class pemeta(Unit):
         return x
 
     @classmethod
-    def _parse_pedict(cls, bin):
-        return dict((
-            cls._ensure_string(key).replace(" ", ""),
-            cls._ensure_string(val)
-        ) for key, val in bin.items() if val)
+    def _parse_pedict(cls, bin: dict):
+        return {
+            cls._ensure_string(key).replace(" ", ""): cls._ensure_string(val)
+            for key, val in bin.items() if val}
 
     @classmethod
     def parse_signature(cls, data: bytearray) -> dict:
@@ -221,7 +232,7 @@ class pemeta(Unit):
                 raise LookupError
             return value[0]
 
-        def find_timestamps(entry) -> dict:
+        def find_timestamps(entry) -> dict | None:
             if isinstance(entry, dict):
                 try:
                     return {'Timestamp': _value(entry, 'signing_time')}
@@ -268,12 +279,17 @@ class pemeta(Unit):
                 if len(serial) % 2 != 0:
                     serial = F'0{serial}'
                 assert bytes.fromhex(serial) in data
-                subject = crt['subject']
-                location = [subject.get(t, '') for t in ('locality_name', 'state_or_province_name', 'country_name')]
+                subject: dict = crt['subject']
+                location = [subject.get(t, '') for t in (
+                    'locality_name', 'state_or_province_name', 'country_name')]
                 cert_info = {}
                 cert_info.update(Subject=subject['common_name'])
+                with suppress(KeyError):
+                    cert_info.update(SubjectEmail=subject['email_address'])
                 if any(location):
                     cert_info.update(SubjectLocation=', '.join(filter(None, location)))
+                with suppress(KeyError):
+                    cert_info.update(SubjectOrg=subject['organization_name'])
                 for attr in signer['signed_attrs']:
                     if attr['type'] == 'authenticode_info':
                         auth = _value(attr)
@@ -286,8 +302,13 @@ class pemeta(Unit):
                     pass
                 else:
                     cert_info.update(ValidSince=valid_since, ValidUntil=valid_until)
-                cert_info.update(
-                    Issuer=crt['issuer']['common_name'], Fingerprint=certificate['fingerprint'], Serial=serial)
+                cert_info.update(Serial=serial)
+                with suppress(KeyError):
+                    cert_info.update(Issuer=crt['issuer']['common_name'])
+                with suppress(KeyError):
+                    cert_info.update(IssuerOrg=crt['issuer']['organization_name'])
+                with suppress(KeyError):
+                    cert_info.update(Fingerprint=certificate['fingerprint'])
                 signer_certificates.append(cert_info)
 
         if len(signer_certificates) == 1:
@@ -325,27 +346,47 @@ class pemeta(Unit):
         aw = self._pe_address_width(pe)
         return F'0x{value:0{aw}X}'
 
-    def parse_version(self, pe: lief.PE.Binary, data=None) -> dict:
+    def parse_version(self, pe: lief.PE.Binary, data=None) -> dict | None:
         """
         Extracts a JSON-serializable and human-readable dictionary with information about
         the version resource of an input PE file, if available.
         """
         version_info = {}
-        if not pe.resources_manager.has_version:
+        rsrc = unwrap(pe.resources_manager)
+        if isinstance(rsrc, lief.lib.lief_errors) or not rsrc.has_version:
             return None
-        version = pe.resources_manager.version
+        version = rsrc.version[0]
 
         if info := version.string_file_info:
-            for lng in info.langcode_items:
+            for child in info.children:
+                entries = {e.key: e.value for e in child.entries}
                 version_info.update({
-                    k.replace(' ', ''): _STRING(v) for k, v in lng.items.items()
+                    k.replace(' ', ''): _STRING(v) for k, v in entries.items()
                 })
-                version_info.update(
-                    CodePage=lng.code_page.name,
-                    LangID=self._vint(pe, lng.lang << 0x10 | lng.sublang),
-                    Language=LCID.get(lng.lang, 'Language Neutral'),
-                    Charset=self._CHARSET.get(lng.sublang, 'Unknown Charset'),
-                )
+
+        if rsrc.has_icons:
+            icon = next(iter(rsrc.icons))
+            version_info.update(
+                LangID=self._vint(pe, icon.lang << 0x10 | icon.sublang),
+                Language=LCID.get(icon.lang, 'Language Neutral'),
+                Charset=self._CHARSET.get(icon.sublang, 'Unknown Charset'),
+            )
+
+        def _code_pages(d: lief.PE.ResourceDirectory | lief.PE.ResourceData):
+            if isinstance(d, lief.PE.ResourceData):
+                yield d.code_page
+                return
+            for child in d.childs:
+                yield from _code_pages(child)
+
+        code_pages: set[int] = set()
+
+        for t in rsrc.types:
+            code_pages.update(_code_pages(rsrc.get_node_type(t)))
+
+        if len(code_pages) == 1:
+            cp = next(iter(code_pages))
+            version_info.update(CodePage=cp)
 
         def _to_version_string(hi: int, lo: int):
             a = hi >> 0x10
@@ -359,18 +400,23 @@ class pemeta(Unit):
         # TODO: Missing: Version.LegalCopyright
         # TODO: Missing: Version.ProductName
 
-        if info := version.fixed_file_info:
-            version_info.update(
-                OSName=info.file_os.name,
-                FileType=info.file_type.name,
-            )
-            if (s := info.file_subtype).value:
-                version_info.update(FileSubType=s)
-            if t := info.file_date_MS << 32 | info.file_date_LS:
+        if info := version.file_info:
+            for name, val, T in (
+                ('FileType', info.file_type, info.FILE_TYPE),
+                ('OSName', info.file_os, info.VERSION_OS),
+                ('FileSubType', info.file_subtype, info.FILE_TYPE_DETAILS),
+            ):
+                if not val:
+                    continue
+                try:
+                    version_info[name] = T(val).name
+                except Exception:
+                    continue
+            if t := info.file_date_ms << 32 | info.file_date_ls:
                 version_info.update(Timestamp=_FILETIME(t))
             version_info.update(
-                ProductVersion=_to_version_string(info.product_version_MS, info.product_version_LS),
-                FileVersion=_to_version_string(info.file_version_MS, info.file_version_LS),
+                ProductVersion=_to_version_string(info.product_version_ms, info.product_version_ls),
+                FileVersion=_to_version_string(info.file_version_ms, info.file_version_ls),
             )
 
         if info := version.var_file_info:
@@ -543,13 +589,13 @@ class pemeta(Unit):
         """
         header = DotNetHeader(data, pe)
         tables = header.meta.Streams.Tables
-        info = dict(
+        info: dict[str, str | int | list[str]] = dict(
             RuntimeVersion=F'{header.head.MajorRuntimeVersion}.{header.head.MinorRuntimeVersion}',
             Version=F'{header.meta.MajorVersion}.{header.meta.MinorVersion}',
             VersionString=header.meta.VersionString
         )
 
-        info['Flags'] = [name for name, check in header.head.KnownFlags.items() if check]
+        info['Flags'] = [repr(flag) for flag in header.head.KnownFlags]
 
         if len(tables.Assembly) == 1:
             assembly = tables.Assembly[0]
@@ -600,7 +646,7 @@ class pemeta(Unit):
 
         pe = lief.load_pe(
             data,
-            parse_exports=self.args.exports,
+            parse_exports=True,
             parse_imports=self.args.imports,
             parse_rsrc=self.args.version,
             parse_reloc=False,
