@@ -1,16 +1,16 @@
 from __future__ import annotations
-from typing import List, Dict, Tuple, Callable, Optional
 
 import enum
-import struct
 import json
+import struct
 
-from refinery.units.formats import UnpackResult, PathExtractorUnit, Arg
+from typing import Callable
 
 from refinery.lib import lief
 from refinery.lib.lcid import LCID
 from refinery.lib.structures import Struct, StructReader
-from refinery.lib.types import ByteStr
+from refinery.lib.types import Param, buf
+from refinery.units.formats import Arg, PathExtractorUnit, UnpackResult
 
 
 class RSRC(enum.IntEnum):
@@ -39,6 +39,9 @@ class RSRC(enum.IntEnum):
         return self.name
 
 
+_RSRC = set(RSRC)
+
+
 class GRPICONDIRENTRY(Struct):
     def __init__(self, reader: StructReader):
         self.width = reader.u8()
@@ -59,13 +62,22 @@ class GRPICONDIR(Struct):
         self.entries = [GRPICONDIRENTRY(reader) for _ in range(count)]
 
 
+def _mktbl(ids: list[tuple[int, int, int]]) -> dict[int, dict[int, int]]:
+    table = {}
+    for pid, sid, lcid in ids:
+        if pid not in table:
+            table[pid] = {0: lcid}
+        table[pid][sid] = lcid
+    return table
+
+
 class perc(PathExtractorUnit):
     """
     Extract PE file resources.
     """
     def __init__(
         self, *paths,
-        pretty: Arg.Switch('-p', help='Add missing headers to bitmap and icon resources.') = False,
+        pretty: Param[bool, Arg.Switch('-p', help='Add missing headers to bitmap and icon resources.')] = False,
         **kwargs
     ):
         super().__init__(*paths, pretty=pretty, **kwargs)
@@ -74,28 +86,30 @@ class perc(PathExtractorUnit):
         for manifest_entry in pe.resources.childs:
             if manifest_entry.id != RSRC.ICON_GROUP.value:
                 continue
-            child: lief.PE.ResourceData = manifest_entry.childs[0].childs[0]
-            return GRPICONDIR(bytearray(child.content))
+            child = manifest_entry.childs[0].childs[0]
+            if isinstance(child, lief.PE.ResourceData):
+                return GRPICONDIR.Parse(bytearray(child.content))
 
     def _search(self, pe: lief.PE.Binary, directory: lief.PE.ResourceDirectory, *parts):
         if directory.depth >= 3:
             self.log_warn(F'unexpected resource tree level {directory.depth + 1:d}')
         for entry in directory.childs:
-            entry: lief.PE.ResourceData
             if entry.has_name:
                 identifier = str(entry.name)
-            elif directory.depth == 0 and entry.id in iter(RSRC):
+            elif directory.depth == 0 and entry.id in _RSRC:
                 identifier = RSRC(entry.id)
             elif entry.id is not None:
                 identifier = entry.id
             else:
                 self.log_warn(F'resource entry has name {entry.name} and id {entry.id} at level {directory.depth + 1:d}')
                 continue
-            if entry.is_directory:
+            if isinstance(entry, lief.PE.ResourceDirectory):
                 yield from self._search(pe, entry, *parts, identifier)
-            else:
-                def extract(_=pe, e=entry):
+                continue
+            if isinstance(entry, lief.PE.ResourceData):
+                def _extract_raw_data(_=pe, e=entry):
                     return bytearray(e.content)
+                extract: buf | Callable[[], buf] = _extract_raw_data
                 path = '/'.join(str(p) for p in (*parts, identifier))
                 if self.args.pretty:
                     if parts[0] is RSRC.BITMAP:
@@ -103,8 +117,8 @@ class perc(PathExtractorUnit):
                     elif parts[0] is RSRC.ICON:
                         extract = self._handle_icon(pe, extract, parts)
                     elif parts[0] is RSRC.ICON_GROUP:
-                        def extract(_=pe, e=entry):
-                            data = GRPICONDIR(e.content)
+                        def _extract_ico_grp(_=pe, e=entry):
+                            data = GRPICONDIR.Parse(e.content)
                             return json.dumps({
                                 entry.nid: {
                                     'width'         : entry.width,
@@ -118,6 +132,7 @@ class perc(PathExtractorUnit):
                                 } for entry in data.entries},
                                 indent=4
                             ).encode(self.codec)
+                        extract = _extract_ico_grp
 
                 yield UnpackResult(
                     path,
@@ -126,7 +141,7 @@ class perc(PathExtractorUnit):
                     offset=entry.offset,
                 )
 
-    def _get_lcid(self, node_data) -> Optional[str]:
+    def _get_lcid(self, node_data) -> str | None:
         try:
             pid = node_data.id & 0x3FF
             sid = node_data.id >> 0x0A
@@ -139,7 +154,7 @@ class perc(PathExtractorUnit):
         lcid = pid.get(sid, 0)
         return LCID.get(lcid)
 
-    def _handle_bitmap(self, extract_raw_data: Callable[[], ByteStr]) -> ByteStr:
+    def _handle_bitmap(self, extract_raw_data: Callable[[], buf]) -> Callable[[], buf]:
         def extract():
             bitmap = extract_raw_data()
             total = (len(bitmap) + 14).to_bytes(4, 'little')
@@ -149,11 +164,13 @@ class perc(PathExtractorUnit):
     def _handle_icon(
         self,
         pe: lief.PE.Binary,
-        extract_raw_data: Callable[[], ByteStr],
-        parts: Tuple[RSRC, int, int]
-    ) -> ByteStr:
+        extract_raw_data: Callable[[], buf],
+        parts: tuple[RSRC, int, int]
+    ) -> buf | Callable[[], buf]:
         try:
             icondir = self._get_icon_dir(pe)
+            if icondir is None:
+                raise RuntimeError
             index = int(parts[1]) - 1
             info = icondir.entries[index]
         except IndexError:
@@ -164,7 +181,7 @@ class perc(PathExtractorUnit):
 
         def extract(info=info):
             icon = extract_raw_data()
-            if icon.startswith(B'(\0\0\0'):
+            if icon[:4] == B'\x28\0\0\0':
                 header = struct.pack('<HHHBBBBHHII',
                     0,
                     1,
@@ -187,15 +204,9 @@ class perc(PathExtractorUnit):
         pe = lief.load_pe_fast(data, parse_rsrc=True)
         if not pe.has_resources:
             return
-        yield from self._search(pe, pe.resources)
-
-    def _mktbl(ids: List[Tuple[int, int, int]]) -> Dict[int, Dict[int, int]]:
-        table = {}
-        for pid, sid, lcid in ids:
-            if pid not in table:
-                table[pid] = {0: lcid}
-            table[pid][sid] = lcid
-        return table
+        if not isinstance(rsrc := pe.resources, lief.PE.ResourceDirectory):
+            raise TypeError('resource root entry was unexpectedly not a directory')
+        yield from self._search(pe, rsrc)
 
     _LANG_ID_TO_LCID = _mktbl([
         (0x00, 0x03, 0x0C00),
