@@ -2,11 +2,11 @@ from __future__ import annotations
 
 import re
 
-from refinery.units import Unit
-from refinery.units.formats.archive.xtpyi import disassemble_code, extract_code_from_buffer
-
 from types import CodeType
 
+from refinery.lib.py import disassemble_code, extract_code_from_buffer
+from refinery.units import Unit
+
 
 class _WrongKey(ValueError):
     pass
@@ -20,7 +20,7 @@ class kramer(Unit):
     _LINEBREAK_MAGIC = 950
 
     def process(self, data):
-        kramer = str()
+        kramer = ''
         secret = set()
         _pyver = None
 
