@@ -3,7 +3,8 @@ from __future__ import annotations
 from base64 import b16decode
 from binascii import Error
 
-from refinery import Unit, Arg
+from refinery import Arg, Unit
+from refinery.lib.types import Param, buf
 from refinery.units.blockwise.xor import xor
 
 
@@ -11,7 +12,7 @@ class n40(Unit):
     """
     Decrypts hex-encoded strings in various latin-american banker families, including N40.
     """
-    def __init__(self, key: Arg(help='Decryption key.')):
+    def __init__(self, key: Param[buf, Arg(help='Decryption key.')]):
         ...
 
     def process(self, data):
