@@ -3,8 +3,9 @@ from __future__ import annotations
 from collections import defaultdict
 from typing import Generator, Iterable
 
-from refinery.units import Arg, Unit, Chunk
 from refinery.lib.meta import check_variable_name
+from refinery.lib.types import Param
+from refinery.units import Arg, Chunk, Unit
 
 
 class groupby(Unit):
@@ -13,13 +14,13 @@ class groupby(Unit):
     blocks and cannot stream any output until the input frame is consumed: It has
     to read every input chunk to make sure that all groupings are complete.
     """
-    def __init__(self, name: Arg(type=str, help='name of the meta variable')):
+    def __init__(self, name: Param[str, Arg.String(help='name of the meta variable')]):
         super().__init__(name=check_variable_name(name))
 
     def process(self, data):
         yield from data.temp
 
-    def filter(self, chunks: Iterable[Chunk]) -> Generator[Chunk, None, None]:
+    def filter(self, chunks: Iterable[Chunk]) -> Generator[Chunk]:
         name = self.args.name
         members = defaultdict(list)
         for chunk in chunks:
