@@ -1,12 +1,12 @@
 from __future__ import annotations
 
 from refinery.lib.patterns import formats, indicators, pattern
-
+from refinery.lib.types import Param
 from refinery.units.meta import Arg, ConditionalUnit
 
-_PATTERNS = {}
-_PATTERNS.update({p.name: p.value for p in formats})
-_PATTERNS.update({p.name: p.value for p in indicators})
+_PATTERNS = {
+    p.name: p.value for d in (formats, indicators) for p in d
+}
 
 
 class iffp(ConditionalUnit, docs='{0}{p}{1}'):
@@ -17,8 +17,8 @@ class iffp(ConditionalUnit, docs='{0}{p}{1}'):
 
     def __init__(
         self,
-        *patterns: Arg.Choice(metavar='pattern', choices=_PATTERNS),
-        partial: Arg.Switch('-p', help='Allow partial matches on the data.') = False,
+        *patterns: Param[str, Arg.Choice(metavar='pattern', choices=list(_PATTERNS))],
+        partial: Param[bool, Arg.Switch('-p', help='Allow partial matches on the data.')] = False,
         retain=False
     ):
         super().__init__(
@@ -36,4 +36,5 @@ class iffp(ConditionalUnit, docs='{0}{p}{1}'):
         return False
 
 
-iffp.__doc__ = iffp.__doc__.format(", ".join(_PATTERNS))
+if __doc := iffp.__doc__:
+    iffp.__doc__ = __doc.format(", ".join(_PATTERNS))
