@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+from refinery.lib.types import Param, buf
 from refinery.units.meta import Arg, ConditionalUnit
 
 
@@ -9,7 +10,7 @@ class iffs(ConditionalUnit, docs='{0}{p}{1}'):
     """
     def __init__(
         self,
-        needle: Arg(help='the string to search for'),
+        needle: Param[buf, Arg(help='the string to search for')],
         retain=False,
     ):
         super().__init__(
