@@ -1,12 +1,11 @@
 from __future__ import annotations
 
 import itertools
-import functools
 
-from refinery.units import Arg, Unit, Chunk
-from refinery.lib.argformats import numseq
-from refinery.lib.tools import isbuffer, typename
 from refinery.lib.meta import check_variable_name
+from refinery.lib.tools import isbuffer, typename
+from refinery.lib.types import Param, isq
+from refinery.units import Arg, Chunk, Unit
 
 _EMPTY = object()
 
@@ -18,9 +17,10 @@ class put(Unit):
     """
     def __init__(
         self,
-        name : Arg(help='The name of the variable to be used.', type=str),
-        value: Arg(help='The value for the variable. If no value is given, the entire current chunk is stored.',
-            type=functools.partial(numseq, typecheck=False)) = _EMPTY
+        name: Param[str, Arg.String(help='The name of the variable to be used.')],
+        value: Param[isq, Arg.NumSeq(check=False, help=(
+            'The value for the variable. If no value is given, the entire current chunk is stored.'
+        ))] = _EMPTY
     ):
         super().__init__(name=check_variable_name(name), value=value)
 
