@@ -3,23 +3,23 @@ A simple tool to queue binary data as one or more chunks in the current frame.
 """
 from __future__ import annotations
 
-from typing import ByteString, Iterable, Generator, Union
-from refinery.units import Arg, Unit, Chunk
+from refinery.lib.types import Generator, Iterable, Param, buf
+from refinery.units import Arg, Chunk, Unit
 
 
 class QueueUnit(Unit, abstract=True):
-    def __init__(self, *data: Arg(help=(
+    def __init__(self, *data: Param[buf, Arg(help=(
         'The arguments are inserted into the current frame in the given order. These arguments '
         'are multibin expressions; If the expression depends on the input data, it will always '
         'refer to the first chunk in the current frame. If no argument is given, a single empty'
         ' chunk is inserted.'
-    ))):
+    ))]):
         super().__init__(data=data)
 
-    def act(self, data: Union[Chunk, ByteString]):
+    def act(self, data: Chunk):
         yield data
 
-    def _queue(self, chunks: Iterable[Chunk], front: bool) -> Generator[Chunk, None, None]:
+    def _queue(self, chunks: Iterable[Chunk], front: bool) -> Generator[Chunk]:
         it = iter(chunks)
 
         try:
@@ -30,7 +30,7 @@ class QueueUnit(Unit, abstract=True):
         def queue():
             data = self.args.data or [B'']
             for bin in data:
-                chunk = head.copy(meta=False, data=False)
+                chunk = head.copy(meta=True, data=False)
                 chunk.visible = True
                 chunk[:] = bin
                 yield chunk
