@@ -1,7 +1,8 @@
 from __future__ import annotations
 
-from refinery.units import Unit, Chunk, Arg
 from refinery.lib.meta import metavars
+from refinery.lib.types import Param
+from refinery.units import Arg, Chunk, Unit
 
 
 class rmv(Unit):
@@ -10,7 +11,7 @@ class rmv(Unit):
     variable names are given, the unit removes all of them. Note that this can recover variables from
     outer frames that were previously shadowed.
     """
-    def __init__(self, *names: Arg(type=str, metavar='name', help='Name of a variable to be removed.')):
+    def __init__(self, *names: Param[str, Arg.String(metavar='name', help='Name of a variable to be removed.')]):
         super().__init__(names=names)
 
     def process(self, data: Chunk):
