@@ -5,7 +5,7 @@ import string
 
 from refinery.lib.patterns import formats
 from refinery.units.obfuscation import Deobfuscator
-from refinery.units.obfuscation.ps1 import string_quote, Ps1StringLiterals
+from refinery.units.obfuscation.ps1 import Ps1StringLiterals, string_quote
 
 
 class deob_ps1_typecast(Deobfuscator):
