@@ -2,17 +2,18 @@ from __future__ import annotations
 
 import re
 
+from enum import Enum
 from fnmatch import fnmatch
 from ipaddress import ip_address
-from typing import AnyStr, Union
-from urllib.parse import urlparse
-from string import ascii_letters
 from pathlib import Path
-from enum import Enum
+from string import ascii_letters
+from typing import AnyStr
+from urllib.parse import urlparse
 
-from refinery.units.pattern import Arg, PatternExtractor
-from refinery.units import RefineryCriticalException
 from refinery.lib.patterns import indicators
+from refinery.lib.types import Param
+from refinery.units import RefineryCriticalException
+from refinery.units.pattern import Arg, PatternExtractor
 
 
 class LetterWeight:
@@ -63,7 +64,7 @@ class xtp(PatternExtractor):
 
     def __init__(
         self,
-        *pattern: Arg('pattern', type=str,
+        *pattern: Param[str, Arg.String('pattern',
             default=(
                 indicators.hostname.name,
                 indicators.url.name,
@@ -73,15 +74,13 @@ class xtp(PatternExtractor):
                 'asterix character to select all available patterns. The available patterns '
                 'are: {}'.format(', '.join(p.display for p in indicators))
             )
-        ),
-        filter: Arg('-f', dest='filter', action='count',
-            help=(
-                'If this setting is enabled, the xtp unit will attempt to reduce the number '
-                'of false positives by certain crude heuristics. Specify multiple times to '
-                'make the filtering more aggressive.'
-            )
-        ) = 0,
-        min=1, max=None, len=None, stripspace=False, duplicates=False, longest=False, take=None
+        )],
+        filter: Param[int, Arg.Counts('-f', help=(
+            'If this setting is enabled, the xtp unit will attempt to reduce the number '
+            'of false positives by certain crude heuristics. Specify multiple times to '
+            'make the filtering more aggressive.')
+        )] = 0,
+        min=1, max=0, len=0, stripspace=False, duplicates=False, longest=False, take=0
     ):
         self.superinit(super(), **vars(), ascii=True, utf16=True)
 
@@ -95,8 +94,8 @@ class xtp(PatternExtractor):
         patterns = [F'(?P<{p.name}>{p.value})' for p in patterns]
         if not patterns:
             raise RefineryCriticalException('The given mask does not match any known indicator pattern.')
-        pattern = '|'.join(patterns)
-        self.args.pattern = re.compile(pattern.encode(self.codec), flags=re.DOTALL)
+        joined = '|'.join(patterns)
+        self.args.pattern = re.compile(joined.encode(self.codec), flags=re.DOTALL)
         self.args.filter = filter
 
     _ALPHABETIC = ascii_letters.encode('ASCII')
@@ -119,6 +118,7 @@ class xtp(PatternExtractor):
         'curl.se'                 : 1,
         'digicert.com'            : 1,
         'dublincore.org'          : 1,
+        'example.com'             : 1,
         'facebook.com'            : 4,
         'fontawesome.com'         : 1,
         'github.com'              : 3,
@@ -132,6 +132,7 @@ class xtp(PatternExtractor):
         'gov'                     : 2,
         'gstatic.com'             : 2,
         'iana.org'                : 1,
+        'ietf.org'                : 1,
         'intel.com'               : 1,
         'jquery.com'              : 1,
         'jsdelivr.net'            : 2,
@@ -151,6 +152,7 @@ class xtp(PatternExtractor):
         'oracle.com'              : 1,
         'purl.org'                : 1,
         'python.org'              : 1,
+        'readthedocs.io'          : 1,
         'schema.org'              : 2,
         'sectigo.com'             : 1,
         'skype.com'               : 1,
@@ -218,7 +220,7 @@ class xtp(PatternExtractor):
                 return False
         return True
 
-    def _check_match(self, data: Union[memoryview, bytearray], pos: int, name: str, value: bytes):
+    def _check_match(self, data: memoryview | bytearray, pos: int, name: str, value: bytes):
         term = self._BRACKETING.get(data[pos - 1], None)
         text = value.decode(self.codec)
         if term:
@@ -252,6 +254,10 @@ class xtp(PatternExtractor):
                     if B'version' in check or b'build' in check:
                         self.log_info(F'excluding ipv4 because it might be a version: {text}')
                         return None
+            small_ocet_count = sum(1 for ocet in ocets if ocet < 10)
+            if small_ocet_count > max(0, 4 - self.args.filter):
+                self.log_info(F'excluding ipv4 because it has too many small ocets: {text}')
+                return None
             ip = ip_address(text)
             if not ip.is_global:
                 if self.args.filter >= 3 or not ip.is_private:
@@ -324,7 +330,7 @@ class xtp(PatternExtractor):
                     if len(hostparts[-1]) > 3:
                         prefix = '.'.join(hostparts[:-1])
                         seen_before = len(set(re.findall(
-                            R'{}(?:\.\w+)+'.format(prefix).encode('ascii'), data)))
+                            fR'{prefix}(?:\.\w+)+'.encode('ascii'), data)))
                         if seen_before > 2:
                             self.log_debug(F'excluding indicator that was already seen: {text}')
                             return None
