@@ -1,11 +1,12 @@
 from __future__ import annotations
 
-import re
 import itertools
+import re
 
-from refinery.units import Arg, Unit
 from refinery.lib.meta import metavars
-from refinery.lib.tools import integers_of_slice
+from refinery.lib.tools import bounds
+from refinery.lib.types import Param, buf
+from refinery.units import Arg, Unit
 
 
 class bruteforce(Unit):
@@ -16,27 +17,27 @@ class bruteforce(Unit):
     """
     def __init__(
         self,
-        name  : Arg.String(help='Name of the meta variable to be populated.'),
-        length: Arg.Bounds(metavar='length', help=(
-            'Specifies the range of characters to brute force, default is {default}.'
-        )) = slice(1, None),
-        format: Arg.String(help=(
+        name: Param[str, Arg.String(help='Name of the meta variable to be populated.')],
+        length: Param[int | slice, Arg.Bounds(metavar='length', intok=True, help=(
+            'Specifies the interval of characters to brute force, default is {default}.'
+        ))] = slice(1, None),
+        format: Param[str | None, Arg.String(help=(
             'Optional format expression for the output string. The format sequence "{0}" is the '
             'current brute force string, the sequence "{1}" represents the input data.'
-        )) = None,
-        alphabet  : Arg.Binary('-a', group='ALPH', help=(
+        ))] = None,
+        alphabet: Param[buf | None, Arg.Binary('-a', group='ALPH', help=(
             'The alphabet from which to choose the letters. Entire byte range by default.'
-        )) = None,
-        pattern   : Arg.RegExp('-r', group='ALPH',
-            help='Provide a regular expression pattern to define the alphabet.') = None,
-        printable : Arg.Switch('-p', group='ALPH',
-            help='Equivalent to --pattern=[\\s\\x20-\\x7E]') = False,
-        digits    : Arg.Switch('-d', group='ALPH',
-            help='Equivalent to --pattern=\\d') = False,
-        identifier: Arg.Switch('-i', group='ALPH',
-            help='Equivalent to --pattern=\\w') = False,
-        letters   : Arg.Switch('-l', group='ALPH',
-            help='Equivalent to --pattern=[a-zA-Z]') = False,
+        ))] = None,
+        pattern: Param[str | None, Arg.RegExp('-r', group='ALPH',
+            help='Provide a regular expression pattern to define the alphabet.')] = None,
+        printable: Param[bool, Arg.Switch('-p', group='ALPH',
+            help='Equivalent to --pattern=[\\s\\x20-\\x7E]')] = False,
+        digits: Param[bool, Arg.Switch('-d', group='ALPH',
+            help='Equivalent to --pattern=\\d')] = False,
+        identifier: Param[bool, Arg.Switch('-i', group='ALPH',
+            help='Equivalent to --pattern=\\w')] = False,
+        letters: Param[bool, Arg.Switch('-l', group='ALPH',
+            help='Equivalent to --pattern=[a-zA-Z]')] = False,
     ):
         options = sum(1 for x in [printable, digits, identifier, letters] if x)
 
@@ -44,13 +45,13 @@ class bruteforce(Unit):
             raise ValueError('Invalid selection.')
 
         if printable:
-            pattern = b'[\\s\\x20-\\x7E]'
+            pattern = '[\\s\\x20-\\x7E]'
         if digits:
-            pattern = b'\\d'
+            pattern = '\\d'
         if identifier:
-            pattern = b'\\w'
+            pattern = '\\w'
         if letters:
-            pattern = b'[a-zA-Z]'
+            pattern = '[a-zA-Z]'
 
         super().__init__(
             name=name,
@@ -61,25 +62,24 @@ class bruteforce(Unit):
         )
 
     def _alphabet(self) -> bytes:
-        alphabet = self.args.alphabet
-        if alphabet:
+        if (alphabet := self.args.alphabet):
             return alphabet
-        alphabet = bytes(range(0x100))
-        pattern = self.args.pattern
-        if not pattern:
-            return alphabet
-        alphabet = B''.join(re.findall(pattern, alphabet, flags=re.DOTALL))
-        if alphabet:
+        else:
+            alphabet = bytes(range(0x100))
+        if not (pattern := self.args.pattern):
             return alphabet
+        if isinstance((regex := Arg.AsRegExp(self.codec, pattern, flags=re.DOTALL)), re.Pattern):
+            if (alphabet := B''.join(regex.findall(alphabet))):
+                return alphabet
         raise ValueError(F'Invalid regular expression: {pattern}')
 
     def process(self, data: bytearray):
         format_spec: str = self.args.format
         meta = metavars(data)
         name = self.args.name
-        kwargs = {name: None}
+        kwargs: dict[str, buf | None] = {name: None}
 
-        for length in integers_of_slice(self.args.length):
+        for length in bounds[self.args.length]:
             self.log_info(F'generating {length} digits')
             if not isinstance(length, int) or length < 0:
                 raise ValueError(F'Unable to brute force {length} characters.')
