@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+from refinery.lib.types import Param, buf
 from refinery.units import Arg, Unit
 
 
@@ -10,9 +11,9 @@ class repl(Unit):
 
     def __init__(
         self,
-        search : Arg(help='This is the search term.'),
-        replace: Arg(help='The substitution string. Leave this empty to remove all occurrences of the search term.') = B'',
-        count  : Arg.Number('-n', help='Only replace the given number of occurrences') = -1
+        search: Param[buf, Arg(help='This is the search term.')],
+        replace: Param[buf, Arg(help='The substitution string. Leave this empty to remove all occurrences of the search term.')] = B'',
+        count: Param[int, Arg.Number('-n', help='Only replace the given number of occurrences')] = -1
     ):
         super().__init__(search=search, replace=replace, count=count)
 
