@@ -1,27 +1,29 @@
 from __future__ import annotations
 
+from refinery.lib.types import Param
 from refinery.units import Arg, Unit
 
 
 class snip(Unit):
     """
-    Snips the input data based on a Python slice expression. For example, the
-    initialization `slice 0::1 1::1` would yield a unit that first extracts
-    every byte at an even position and then, every byte at an odd position. In
-    this case, multiple outputs are produced. The unit can be used in reverse
-    mode, in which case the specified ranges are deleted sequentially from the
-    input.
+    Snips the input data based on a Python slice expression. For example, the initialization
+
+        sbip 0::1 1::1
+
+    would yield a unit that first extracts every byte at an even position and then, every byte at
+    an odd position. In this case, multiple outputs are produced. This is a versatile tool that can
+    be used to skip certain bytes, extract substrings, or even reverse the input.
     """
     def __init__(
         self,
-        slices: Arg(help='Specify start:stop:step in Python slice syntax.') = [slice(None, None)],
-        length: Arg.Switch('-l', help=(
-            'Interpret the end of a slice as a length rather than as an offset.')) = False,
-        stream: Arg.Switch('-s', help=(
+        slices: Param[list[slice], Arg(help='Specify start:stop:step in Python slice syntax.')] = [slice(None, None)],
+        length: Param[bool, Arg.Switch('-l', help=(
+            'Interpret the end of a slice as a length rather than as an offset.'))] = False,
+        stream: Param[bool, Arg.Switch('-s', help=(
             'After each slice, consider only the data that follows after it for subsequent '
-            'slicing. This mode is incompatible with negative step sizes.')) = False,
-        remove: Arg.Switch('-r', help=(
-            'Remove the slices from the input rather than selecting them.')) = False,
+            'slicing. This mode is incompatible with negative step sizes.'))] = False,
+        remove: Param[bool, Arg.Switch('-r', help=(
+            'Remove the slices from the input rather than selecting them.'))] = False,
     ):
         super().__init__(slices=slices, length=length, stream=stream, remove=remove)
 
