@@ -1,7 +1,8 @@
 from __future__ import annotations
 
-from itertools import cycle, repeat, islice
+from itertools import cycle, islice, repeat
 
+from refinery.lib.types import Param
 from refinery.units import Arg, Unit
 
 
@@ -9,10 +10,10 @@ class stretch(Unit):
     """
     Stretch the input data by repeating every byte a number of times.
     """
-    def __init__(self, *count: Arg.Number(metavar='count', help=(
+    def __init__(self, *count: Param[int, Arg.Number(metavar='count', help=(
         'The number of times every byte should be repeated. By default,  '
         'every byte is repeated once.'
-    ))):
+    ))]):
         count = count or (2,)
         if any(k <= 0 for k in count):
             raise ValueError('You can not use a stretching factor of less than 1.')
